//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
//                                  БАЗОВЫЙ ТЕСТ №5 
//    Сенсоры через пины: DHT22,DS1820,HC-SR04;
//    Сенсоры через i2c: комбинированный(AHTxx+BMP280),SHT3x,HTU21, 
//    На консоль однократно выводится протокол по контроллеру и тесту. 
//    На консоль(монитор порта) выводятся значения ADC(по изменению),Кнопка(по изменению),быстродействие проекта-->все вместе периодически.
//    На Lcd1.i2c(0x27): Строка 1:"Кнопка",ADC,Press(BMP280),быстродействие/1000.
//                       Строка 2:"Ok сенсоров",Temp,HUM(AHTxx). 
//    На пин PWM выводится значение с ADC.      
//    На пин Led выводится меандр 500 мс.
//    На пин Control выводится изменение в каждом цикле loop(для логического анализатора).
//    В 74HC595 записывается "бегущая 1".  
//-------------------------------------------------------------------------------------------------
// Для настройки IDE для работы поддерживаемыми платами во вкладке Readme.hpp имеется инструкция и др.полезная информация. 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#include "RT_HW_BASE.h"         //--Подключение базовой библиотеки (не обязательна, так как дублируется в других библиотеках ниже);
#include "RT_HW_LCD44780.h"     //--Подключение библиотеки поддержки LCD дисплеев типа HD44780;
#include "RT_HW_SENSORS.h"      //--Подключение библиотеки поддержки сенсоров;
#include "RT_HW_EXTENDERS.h"    //--Подключение библиотеки поддержки расширителей;
//=============================Подключение внутренних функций========================================
#include "Aobj.hpp"       //--Объекты и переменные;
#include "Bfunc.hpp"      //--Базовые функции;
#include "Csns.hpp"       //--Функции опроса сенсоров;
#include "Dext.hpp"       //--Функции работы с расширителями;
#include "Lcd1.hpp"       //--Функции вывода на дисплей Lcd1(i2c);        
#include "Lcd2.hpp"       //--Функции вывода на дисплей Lcd2(i2c);  
//#include "Lcd3.hpp"       //--Функции вывода на дисплей Lcd3(SPI);        
//#include "Next.hpp"       //--Функции вывода на дисплей Nextion(UART);
//=============================SETUP()===============================================================
void setup(){}
//=============================LOOP()================================================================
void loop(){
//-----------------------------Системные функции--------------------------------------------------- 
initProject();                                                                                     //--Настройка параметров;
RT_HW_Base.sheduler();                                                                             //--Работа дисплея;
if(RT_HW_Base.shed.eventSec){vSpeedMCU=RT_HW_Base.shed.cycle; vSpeedMCUF=filterSpeed.filtered(vSpeedMCU); //--Расчет быстродействия с фильтрацией; 
                             vSpeedSmall=round(((float)vSpeedMCUF)/1000);}                         //--Получение быстродействия/1000;
RT_HW_Base.consoleBegin();                                                                         //--Инициализация консоли;
if(RT_HW_Base.console.head){consoleHead();}                             //--Однократный вывод на консоль заголовка проекта;
RT_HW_Base.pinDigitalWrite(idPinControl,RT_HW_Base.device.pin.control,RT_HW_Base.shed.control);    //--Вывод loop() на контрольный пин;
RT_HW_Base.pinDigitalWrite(idPinLed,    RT_HW_Base.device.pin.led,    RT_HW_Base.shed.blink500);   //--Вывод меандра 500ms на Led; 
//=====================ПИНЫ========================================================================
if(RT_HW_Base.shed.fast.num==2) {fncADC();}      //--Чтение ADC [Быстрая задача, период 100ms, так как ADC на AVR не быстрый]; 
if(RT_HW_Base.shed.fast.num==2) {fncPWM();}      //--Запись PWM [Быстрая задача, период 100ms, так как ADC на AVR не быстрый]; 
if(RT_HW_Base.shed.frdm.num==3) {fncGen();}      //--Генераторы тестовые;
if(RT_HW_Base.shed.frdm.num==4) {fncButton();}   //--Чтение кнопки с защитой от дребезга;
//if(RT_HW_Base.shed.frdm.num==5) {fncServo();}  //--Управление Servo от ADC(пока только для ESP32);
//=====================СЕНСОРЫ через пины==========================================================  
if(RT_HW_Base.shed.frdm.num==10){sns_DHT22();}   //--Управление сенсором DHT22;
if(RT_HW_Base.shed.frdm.num==12){sns_DS1820();}  //--Управление сенсором DS1820 (по шине 1-Wire);  
if(RT_HW_Base.shed.frdm.num==14){sns_SR04();}    //--Управление сенсором HC-SR04 (по прерываниям);  
//=====================СЕНСОРЫ i2c=================================================================
if(RT_HW_Base.shed.frdm.num==16){sns_CMTHP();}   //--AHTxx+BMP280(Комбинированный сенсор);
if(RT_HW_Base.shed.frdm.num==17){sns_SHT3X();}   //--SHT31;
if(RT_HW_Base.shed.frdm.num==18){sns_HTU21();}   //--HTU21;
if(RT_HW_Base.shed.frdm.num==19){sns_BH1750();}  //--BH1750;
//=====================СЕНСОРЫ SPI,SPN=============================================================
if(RT_HW_Base.shed.frdm.num==20){sns_spiMAX6675();}//--Управление сенсором MAX6675 через SPI;  
if(RT_HW_Base.shed.frdm.num==21){sns_spnMAX6675();}//--Управление сенсором MAX6675 через SPN;
//=====================РАСШИРИТЕЛИ SPI,SPN=========================================================
if(RT_HW_Base.shed.frdm.num==22){ext_spi74HC595(genRUN8);}  //--Вывод на 74HC595 "бегущей 1";
//=====================ДИСПЛЕИ LCD HD44780=========================================================
if(RT_HW_Base.shed.frdm.num==25){Lcd1();}          //--Управление дисплеем 1(16x2,i2c,0x27);
if(RT_HW_Base.shed.frdm.num==27){Lcd2();}          //--Управление дисплеем 2(20x4,i2c,0x3E);
//------------------------------------------------------------------------------------------------
}; //++++END loop()================================================================================
   
