//===================================================================================================
//                           ОБЩИЙ ТЕСТ ДЛЯ РАЗЛИЧНЫХ ПЛАТ
// 1)Вывод параметров плат на консоль; 2)Мерцание светодиодом на плате; 3)Вывод контрольного сигннала (loop) на пин;
// 4)Опрос ADC с фильтром; 5)Вывод ADC на PWM; 6)Вывод ADC,PWM,кнопки,быстродействия на консоль; 
//             (краткая вводная инструкция по использованию плат во вкладке Readme.hpp)
//===================================================================================================
#include "RT_HW_BASE.h"   
#include "RT_HW_LCD44780.h"
#include "RT_HW_SENSORS.h" 
#include "RT_HW_EXTENDERS.h" 
//---------------------------------------------------------------------------------------------------
uint8_t  vInit=0;                                                         //--Флаг инициализации параметров;
uint32_t vSpeedMCU=0; uint16_t vSpeedSmall=0; uint32_t vSpeedFltr;        //--Быстродействие проекта в циклах loop() (speedMCU=SpeedMCU/1000);
RT_HW_FILTER_RING_AVERAGE<uint32_t,10> filterSpeed; uint32_t vSpeedMCUF;  //--Параметры для фильтра скорости MCU;
uint8_t  adrLcd;                                                          //--Адрес LCD;
//---------------------------------------------------------------------------------------------------
RT_HW_PIN_DIR_ID   idPinControl;uint8_t pinControl;         //--Объект и переменная для пина "Контроль";
RT_HW_PIN_DIR_ID   idPinLed;    uint8_t pinLed;             //--Объект и переменная для пина "Led на плате";                    
RT_HW_PIN_PWM_ID   idPinPWM;    uint8_t pinPWM;             //--Объект и переменная для пина PWM;
//-------------------------------Кнопка с защитой от дребезга----------------------------------------
RT_HW_PIN_DIR_ID   idPinButton; uint8_t pinButton; uint8_t vButton;  //--Объект и переменные для пина "Button на плате";  
RT_HW_CHAT_V8_ID   idChatButton;                   uint8_t vChatBut; //--Объекты и переменные для обработки кнопки с защитой от дребезга;
//-------------------------------ADC с фильрацией----------------------------------------------------
RT_HW_PIN_ADC_ID   idPinADC;    uint8_t pinADC; uint16_t vADCF; uint16_t agoADC; //--Объект и переменная для ADС;
RT_HW_FILTER_RING_AVERAGE<uint32_t,5>  filterADC;           //--Параметры для для фильтра ADC(скользящий среднего значения 5-го порядка);
//-------------------------------Сенсор DS1820-------------------------------------------------------
RT_HW_STRUCT_DS1820_ID dvDS1820;  RT_HW_ONEWIRE dvNW; uint8_t pinDS1820; //--Объекты и переменные для DS1820;
RT_HW_STRUCT_SHED_TASK shDS1820;  float vDS1820Temp;  uint8_t vDS1820ok; //--Объекты и переменные для DS1820; 
//-------------------------------Управление серво----------------------------------------------------
RT_HW_PIN_SERVO_ID idPinServo; uint8_t pinServo;           //--Объект и переменная для пина Servo;
RT_HW_GENERATOR_EVENT_ID idGenServo; RT_HW_COUNTER_ID  idCntServo; 
//-------------------------------Сенсор HC-SR04------------------------------------------------------
RT_HW_STRUCT_SR04_ID     dvSR04;   RT_HW_STRUCT_SHED_TASK shSR04;   uint16_t vSR04Len; uint8_t vSR04ok; uint8_t pinSR04;
RT_HW_IRAM_ATTR void RT_HW_IRQ_SR04_handler_dvSR04(){ //++++Обработчик прерываний HC-SR04------------
  if(dvSR04.state==2) {if( RT_HW_PIN_DIR_READ(dvSR04.ddPin)){dvSR04.timeBegin=micros(); dvSR04.state=3;}}
  if(dvSR04.state==3) {if(!RT_HW_PIN_DIR_READ(dvSR04.ddPin)){dvSR04.timeEnd  =micros(); dvSR04.state=4;}}}
RT_HW_FILTER_MEDIAN_3 <uint16_t> ftrSR04; uint16_t vSR04LenF;   //--Параметры для фильтра HC-SR04(медианный 3-го порядка);
//-------------------------------Расширитель 74HC595-------------------------------------------------
RT_HW_SPIhc595 dvSPIhc595; RT_HW_STRUCT_SHED_TASK shSPIhc595; uint8_t pinCShc595;
//-------------------------------Генераторы и счетчики-----------------------------------------------
RT_HW_GENERATOR_EVENT_ID idGen595; RT_HW_COUNTER_ID idCnt595; int16_t gen595=0;    //--Для генератора бегущей единицы;
RT_HW_GENERATOR_EVENT_ID idGenDAC; RT_HW_COUNTER_ID idCntDAC; int16_t genDAC=0;    //--Для генератора 0-4095 DAC;
//===================================================================================================
#include "Device.hpp"
#include "Lcd.hpp"
//===================================================================================================
void setup(){}
//===================================================================================================
void loop(){
initProject();              //--Инициализация параметров;
RT_HW_Base.sheduler();      //--Диспетчер;
RT_HW_Base.consoleBegin();  //--Инициализация консоли;
if(RT_HW_Base.shed.eventSec){vSpeedMCU=RT_HW_Base.shed.cycle; vSpeedMCUF=filterSpeed.filtered(vSpeedMCU); vSpeedSmall=round(((float)vSpeedMCUF)/1000);} //--Получение быстродействия MCU;
RT_HW_Base.pinDigitalWrite(idPinControl,pinControl,RT_HW_Base.shed.control);    //--Вывод loop() на контрольный пин;
RT_HW_Base.pinDigitalWrite(idPinLed,    pinLed,    RT_HW_Base.shed.blink500); //--Вывод меандра 500ms на Led; 
//=================================================================================================
if(RT_HW_Base.console.head){consoleHead();}             //--Вывод заголовков; 
if(RT_HW_Base.shed.fast.num==2) {getAnalog();}          //--Чтение ADC; 
if(RT_HW_Base.shed.fast.num==2) {setPWM();}             //--Запись PWM; 
if(RT_HW_Base.shed.frdm.num==8) {cntTest();}            //--Тестовые генераторы значений;
if(RT_HW_Base.shed.frdm.num==10){getButton();}          //--Чтение кнопки с защитой от дребезга;
if(RT_HW_Base.shed.frdm.num==12){extSPIhc595(gen595);}  //--Запись в 74HC595;
if(RT_HW_Base.shed.frdm.num==14){setServo();}           //--Запись Servo; 
if(RT_HW_Base.shed.frdm.num==16){snsDS1820();}          //--Сенсор DS1820;
if(RT_HW_Base.shed.frdm.num==18){snsSR04();}            //--Сенсор HC-SR04;
if(RT_HW_Base.shed.frdm.num==20){Lcd1();}               //--Вывод на дисплей;
}; //++++END loop()================================================================================
   
