//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//					1.СТРУКТУРЫ ДЛЯ ВВОДА С ПИНОВ, датчика ХОЛЛА
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//=================================================================================================
//						1.1.DI(дискретного ввода)
//  RT_HW_PIN_DIR_ID - определена в описании ядра;
//=================================================================================================
//=================================================================================================
//						1.2.ADC(аналоговый ввода)
//================================================================================================= 
struct RT_HW_PIN_ADC_ID{
uint8_t  dir=0;				//--Номер этапа в исполнительной функции; 
uint8_t  pin=255;			//--Номер пина;
uint16_t adcVar,adcMax;		//--Внутренние переменные;
uint16_t vPin;				//--Значение с пина [analogRead(pin)];
uint16_t vFltr;	    		//--vPin+фильтр дрейфа измерений(device.depth.drift) и дрейфа нуляdevice.depth.vNull); 
uint16_t vNorm;				//--vFltr+нормализация значений до системного SYS с разрядностью=device.depth.sys; 
};
//=================================================================================================
//						1.3.TCH(сенсорный ввод)[ESP32]
//================================================================================================= 
struct RT_HW_PIN_TCH_ID{
uint8_t  dir=0;				//--Номер этапа в исполнительной функции; 
uint8_t  pin=255;			//--Номер пина;
uint16_t vPin;	    		//--Значение с пина [touchRead(pin)]; 
uint16_t vNorm;				//--vPin+нормализация значений до системного SYS с разрядностью=device.depth.sys;
};
//=================================================================================================
//						1.4.HALL(датчика Холла)[ESP32]
//  В зависимости он направления магнитного поля вырабатывает значения с разным знаком.
//================================================================================================= 
struct RT_HW_PIN_HALL_ID{
uint8_t  dir=0;				//--Номер этапа в исполнительной функции; 
int16_t  vPin;	    		//--Значение с сенсора [hallRead()];
uint16_t vNorm;				//--vPin+нормализация значений до системного SYS с разрядностью=device.depth.sys;
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//					2.СТРУКТУРЫ ДЛЯ ВЫВОДА НА ПИНЫ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//=================================================================================================
//						2.1.DO(дискретного вывод)
//  RT_HW_PIN_DIR_ID - определена в описании ядра;
//=================================================================================================

//=================================================================================================
//						2.2.PWM(широтно-импульсный вывод)
//================================================================================================= 
struct RT_HW_PIN_PWM_ID{
uint8_t  dir=0;				//--Номер этапа в исполнительной функции; 
uint8_t  pin=255;			//--Номер пина;
uint8_t  channel=255;		//--Номер канала;
uint8_t  depth=255;			//--Разрядность PWM (ESP32). Если =255, то при инициализации устанавливается значение=device.depth.pwm;
uint16_t freq=1000;			//--Частота;
char     mode='N';			//--Тип выхода
//---------------------------------------------------------
//uint16_t minWidth=0;		//--Минимальная  ширина импульса(mcs);    
//uint16_t maxWidth=0xFFFF;	//--Максимальная ширина импульса(mcs); 
//----------------------------------------------------------
uint8_t  idx;				//--Рабочий индекс;
uint32_t var;
uint16_t duration;			//--Нормализованное под разрядность depth значение импульса PWM;
uint16_t duty;				//--Длительность импульса с учетом
};
//================================================================================================= 
//						2.3.DAC(аналоговый вывод)
//=================================================================================================  
struct RT_HW_PIN_DAC_ID{
uint8_t  dir=0;				//--Номер этапа в исполнительной функции; 
uint8_t  pin=255;			//--Номер пина;
uint16_t var;
uint16_t vPin;
};
//================================================================================================= 
//						2.4.SERVO(вывод на сенсор)
//=================================================================================================  
struct RT_HW_PIN_SERVO_ID{
uint8_t  dir=0;				//--Номер этапа в исполнительной функции; 
uint8_t  pin=255;			//--Номер пина;
uint8_t  channel=4;			//--Номер канала;
uint8_t  depth=16;			//--Разрядность [ESP32];
uint16_t freq=50;			//--Частота;
uint16_t maxAngle=180;		//--Максимальный угол поворота;
uint16_t minWidth=800;		//--Минимальная  ширина импульса(mcs);    
uint16_t maxWidth=2300;		//--Максимальная ширина импульса(mcs);   
uint16_t maxDepth=0xFFFF;   //--Максимальное значение в у.е. для выбранной разрядности;
uint16_t period=20000;		//--Период управляющих импульсов - вычисляемая переменная=1000000/freq; 
//-------------------------------
uint8_t  idx;				//--рабочий индекс;
uint16_t var;				//--Внешняя уставка длительности импульса;
uint16_t duration;			//--Выводимая длительность импульса;
uint16_t duty;				//--Нормализованная под разрядность=depth разрядность;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//					3.ДОПОЛНИТЕЛЬНЫЕ СТРУКТУРЫ ДЛЯ РАБОТЫ С ПИНАМИ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



//=================================================================================================
//						           Защита от дребезга
//=================================================================================================
struct RT_HW_CHANGE_PINDI_ID    {uint32_t time; uint8_t dir=0; uint8_t ago=0;};				    //--Для защиты от дребезга пина DI;
struct RT_HW_CHAT_V8_ID {
uint8_t  dir=0;				//--Номер этапа в исполнительной функции;  
uint8_t  ago=0;				//--Предыдущее значение контролируемого значения; 
uint16_t period; 			//--Период защиты от дребезга;
uint32_t timeBegin;			//--Рабочий регистр;
};
//=================================================================================================
//						           Структура счетчиков
//=================================================================================================