//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//								СИСТЕМНЫЕ ФУНКЦИИ ОБЩИЕ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		
//=================================================================================================
//				Функции нормализации значений ADC,PWW,DAC до системного значения 
//=================================================================================================	
uint16_t maxBitDepth (uint8_t &depth){if(depth>=16) {return 0xFFFF;} if(depth==0){return 0;} return ((1<<depth)-1);}; //-return	max var разрядности 1:16;
uint16_t normADC(uint16_t v){return map(v,0,maxBitDepth(device.depth.adc),0,maxBitDepth(device.depth.sys));}; //--нормализация для ADC;
uint16_t normPWM(uint16_t v){return map(v,0,maxBitDepth(device.depth.sys),0,maxBitDepth(device.depth.pwm));}; //--нормализация для PWM;
uint16_t normDAC(uint16_t v){return map(v,0,maxBitDepth(device.depth.sys),0,maxBitDepth(device.depth.dac));}; //--нормализация для DAC;
uint16_t normDepth(uint16_t v,uint8_t depth){if(depth>16){depth=16;} 										  //--нормализация системного сигнала SYS ;
                             return map(v,0,maxBitDepth(device.depth.sys),0,maxBitDepth(depth));};			   			
uint16_t normDepthFull(uint16_t v,uint8_t inDepth, uint8_t outDepth){if(inDepth>16){inDepth=16;} if(outDepth>16){outDepth=16;}	//--универсальный норматизатор;  
		                     return map(v,0,maxBitDepth(inDepth),0,maxBitDepth(outDepth));};;
//=================================================================================================
//				Функции гистерезиса меняют значение var на cur при его отклонение на delta) 
//=================================================================================================	
uint16_t makeDelta(uint16_t &var, uint16_t cur, uint8_t delta){			
  if(cur>var){if((cur-var)>delta){var=cur;}} 
  if(cur<var){if((var-cur)>delta){var=cur;}} return var;} 
//=================================================================================================
//						Обмен битов uint8_t
//=================================================================================================
uint8_t changeBitByte(uint8_t val){
return((val&0x01)<<7)|((val&0x02)<<5)|((val&0x04)<<3)|((val&0x08)<<1)|((val&0x10)>>1)|((val&0x20)>>3)|((val&0x40)>>5)|((val&0x80)>>7);};	  
//=================================================================================================
//						Обмен битов uint16_t
//=================================================================================================
uint16_t changeBitWord(uint16_t val){
return((val&0x0001)<<15)|((val&0x0002)<<13)|((val&0x0004)<<11)|((val&0x0008)<<9)|((val&0x0010)<<7)|((val&0x0020)<<5)|((val&0x0040)<<3) | ((val&0x0080)<<1) |
      ((val&0x8000)>>15)|((val&0x4000)>>13)|((val&0x2000)>>11)|((val&0x1000)>>9)|((val&0x0800)>>7)|((val&0x0400)>>5)|((val&0x0200)>>3) | ((val&0x0100)>>1);}                                     
//=================================================================================================
//						Реверс бит в v8,v16,v24,v32
//=================================================================================================
inline uint8_t reverseV8(uint8_t b) {
    b = (b & 0xF0) >> 4 | (b & 0x0F) << 4;
    b = (b & 0xCC) >> 2 | (b & 0x33) << 2;
    b = (b & 0xAA) >> 1 | (b & 0x55) << 1;
    return b;}
uint16_t reverseV16(uint16_t v) {//uint8_t v0=v & 0xFF; uint8_t v1=(v>>8) & 0xFF; 
return ((reverseV8(v & 0xFF)<< 8) | reverseV8((v>>8) & 0xFF));}
uint32_t reverseV24(uint32_t v) {uint8_t v0=v & 0xFF; uint8_t v1=(v>>8) & 0xFF; uint8_t v2=(v>>16) & 0xFF; 
return (                                    (((uint32_t)reverseV8(v0))<<16) | (((uint32_t)reverseV8(v1))<< 8) | reverseV8(v2));}
uint32_t reverseV32(uint32_t v) {uint8_t v0=v & 0xFF; uint8_t v1=(v>>8) & 0xFF; uint8_t v2=(v>>16) & 0xFF; uint8_t v3=(v>>24) & 0xFF; 
return ((((uint32_t)reverseV8(v0))<<24) | (((uint32_t)reverseV8(v1))<<16) | (((uint32_t)reverseV8(v2))<< 8) | reverseV8(v3));}
//=================================================================================================
//						Извлечение цифры из числа
//=================================================================================================
char 	 getCharVal(uint32_t val,uint8_t upper,char mode,uint8_t len,uint8_t n){//==Возвращает цифру(как символ) из val начиная со старшей позиции;
//---mode B,b,H,h,D,I,0,1,2,3,T; upper =0 верхний регистр, =1 нижний регистр;
uint32_t buff32=0;
if((mode=='B') || (mode=='b')) {buff32=(val>>(len-n))&1;}
if((mode=='H') || (mode=='h')) {buff32=(val>>(len-n)*4)&0xf;}
if((mode=='D') || (mode=='I') ||(mode=='U')|| (mode=='0') ||(mode=='1') ||(mode=='2') || (mode=='3') || (mode=='T')) 
                                        {n=len-n+1; while(n!=0) {buff32=val; val/=10; buff32=buff32-(val*10); n--;};}
if(buff32<=9) {buff32=buff32 +'0';} else {buff32=buff32-10+(upper ? 'A' : 'a') ;}
return (uint8_t)buff32;}; 




//=================================================================================================
//							Поиск номера первого свободного бита в регистре состояний						
// Возвращает номер первого свободного бита (0:15) или -1;	
//=================================================================================================
int8_t getFistFreeBit    (uint16_t &reg, uint8_t maxNum=15){
 if(maxNum>16) {return -1;} for(uint8_t i=0;i<maxNum;i++){if(!bitRead(reg,i)) return i;} return -1;};
//=================================================================================================
//		Функции поиска и захвата первый свободный бит в регистре состояний						
// Возвращает номер первого свободного бита (0:15) или -1;	
//=================================================================================================
int8_t setFirstFreeBit   (uint16_t &reg, uint8_t maxNum=15){
if(maxNum>16) {return -1;} for(uint8_t i=0;i<maxNum;i++){if(!bitRead(reg,i)) {bitSet(reg,i); return i;}} return -1;};	
//=================================================================================================
//							Проверяет занятость в reg номера бита num(0:15) 
// при ограничении его номера maxNum(if(num>max){return 0;}; 					
// Возвращает номер первого свободного бита (0:15) или -1;	
//=================================================================================================
bool checkBusyNumBit   (uint16_t &reg, uint8_t num, uint8_t maxNum=15){if(maxNum>16) {return 0;} return bitRead(reg,num);};		
//=================================================================================================
//							Занимает в reg номер первого свободного устройства(1:16) и возращает его номер или 0; 								
//=================================================================================================
uint8_t  setFistFreeDevice (uint16_t &reg, uint8_t maxNum=16){				//==Занимает в reg номер первого свободного устройства(1:16) и возращает его номер или 0; 	
 if((maxNum>16) || (maxNum==0)) {return 0;} 
 for(uint8_t i=1;i<=maxNum;i++){if(!bitRead(reg,(i-1))) {bitSet(reg,(i-1)); return i;}} return 0;};	
//=================================================================================================
//							Занимает в reg номер первого свободного устройства(1:16) и возращает его номер или 0; 
//==Проверяет занятость номера устройства(1:16); 								
//=================================================================================================	
bool  checkBusyNumDevice(uint16_t &reg, uint8_t num, uint8_t maxNum=16) {if((maxNum>16) || (num==0)) {return 0;} return bitRead(reg,(num-1));};	
//=================================================================================================
//							Проверка кодов архитектуры, платы
//=================================================================================================
bool checkArch (uint8_t codeArch) {if(device.board.codeArch ==codeArch) {return 1;} else {return 0;}}; //--Проверка кода архитектуры;
bool checkBoard(uint8_t codeBoard){if(device.board.codeBoard==codeBoard){return 1;} else {return 0;}}; //--Проверка кода платы;



//uint8_t  map_v8(uint8_t x, uint8_t in_min, uint8_t in_max, uint8_t out_min, uint8_t out_max){
//}

//long map(long x, long in_min, long in_max, long out_min, long out_max)
//{
//  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
//}