#include "RT_HW_BASE.h"
//=================================================================================================
//							Генератор события по изменению фронта EN [F,C,B,H,D,N]
//	Вход: ago-рабочий регистр, EN-внешний вход, mode-режим работы.
//	Выход: возвращаемое значение bool;
//=================================================================================================
bool generatorChangeEN(uint8_t &ago, char mode, uint8_t EN=1){	
if(mode=='F') {if(ago!=EN) {ago=EN; if( EN) {return 1;}}}	//--Выделение заднего фронта;	
if(mode=='C') {if(ago!=EN) {ago=EN;          return 1;}}	//--Выделение изменения сигнала;	
if(mode=='R') {if(ago!=EN) {ago=EN; if(!EN) {return 1;}}}	//--Выделение переднего фронта;	
if(mode=='H') {								 return EN;}    //--По высокому уровню;
if(mode=='L') {								 return (!EN);} //--По низкому уровню;                
return 0;};
//=================================================================================================
//										Генератор событий
//	Вход: id-рабочая структура, EN-внешний вход, period-периодичность.
//	Выход: возвращаемое значение bool=id.event;
//		   id.event-событие; id.meander-меандр.
//=================================================================================================
bool generatorEvent(RT_HW_GENERATOR_EVENT_ID &id, uint16_t period, uint8_t EN=1){
id.event=0;
if(id.dir==0) {id.time=millis(); id.event=id.meander=1; id.dir++;}
if(id.dir==1) {if(EN) {id.dir++;} else {id.event=id.meander=0; id.time=RT_HW_MILLIS;}}
if(id.dir==2) {if(periodMs(id.time,period)) {id.event=1; if(EN) {id.meander=!id.meander;} else {id.meander=0;} id.dir=1;}} 
return id.event;};  
//=================================================================================================
//										Генератор импульсов uint16_t
//	Вход: id-рабочая структура, EN-внешний вход, timeON-время включения в ms, timeOFF-время выключения в ms.
//	Выход: возвращаемое значение bool -генерируемый импульс.
//=================================================================================================  
bool  generatorPulse(RT_HW_GENERATOR_PULSE_ID &id, uint16_t timeON, uint16_t timeOFF, uint8_t EN=1){		//==Генератор импульсов (v16);
if(id.dir==0) {if(EN) {id.dir=3;} else {return 0;}}
if(id.dir==1) {if(RT_HW_MILLIS>=id.time) {id.time+=timeOFF; id.dir=2; return 0;} else {return 1;}}
if(id.dir==2) {if(RT_HW_MILLIS>=id.time) {id.dir=3;}}
if(id.dir==3) {if(EN) {id.time=millis()+timeON; 		id.dir=1; return 1;}
			   else 								   {id.dir=0; return 0;}}	   return 0;};
//=================================================================================================
//										Генератор импульсов uint32_t
//	Вход: id-рабочая структура, EN-внешний вход, timeON-время включения в ms, timeOFF-время выключения в ms.
//	Выход: возвращаемое значение bool -генерируемый импульс.
//================================================================================================= 
bool generatorPul32(RT_HW_GENERATOR_PULSE_ID &id, uint32_t timeON, uint32_t timeOFF, uint8_t EN=1){		//==Генератор импульсов (v32);		
if(id.dir==0) {id.dir=3;}
if(id.dir==1) {if((RT_HW_MILLIS-id.time)>= timeON) 			{id.dir=2; 	 return 0;} else {return 1;}}
if(id.dir==2) {if((RT_HW_MILLIS-id.time)>=(timeON+timeOFF)) {id.dir=3;}}
if(id.dir==3) {if(!EN) 		 {return 0;} id.time=RT_HW_MILLIS; 
			   if(timeON ==0){return 0;} 
			   if(timeOFF==0){return 1;}
			   id.dir=1; 	  return 1;}	return 0;}
//=================================================================================================
//										Генератор импульсов uint32_t
//	Вход: id-рабочая структура, timeON-время включения в ms, timeOFF-время выключения в ms.
//	Выход: возвращаемое значение bool -генерируемый импульс.
//================================================================================================= 
bool generatorPul32mcs(RT_HW_GENERATOR_PULSE_ID &id, uint32_t timeON, uint32_t timeOFF, uint8_t EN=1){		//==Генератор импульсов (v32);		
if(id.dir==0) {id.dir=3;}
if(id.dir==1) {if((RT_HW_MICROS-id.time)>= timeON) 			{id.dir=2; 	 return 0;} else {return 1;}}
if(id.dir==2) {if((RT_HW_MICROS-id.time)>=(timeON+timeOFF)) {id.dir=3;}}
if(id.dir==3) {if(!EN) 		 {return 0;} id.time=micros(); 
			   if(timeON ==0){return 0;} 
			   if(timeOFF==0){return 1;}
			   id.dir=1; 	  return 1;}	return 0;}
//=================================================================================================
//										Генератор PWW(период ms, скважность)
//	Вход: id-рабочая структура, timeON-время включения в ms, timeOFF-время выключения в ms.
//	Выход: возвращаемое значение bool -генерируемый импульс.
//================================================================================================= 			   
bool generatorPWM  (RT_HW_GENERATOR_PWM_ID   &id, uint32_t period, uint8_t wprc, uint8_t EN=1){ 			//==Генератор PWM;
if(id.dir==0) {id.dir=3;}
if(id.dir==1) {if((RT_HW_MILLIS-id.time)>= id.timeON) 	 {id.dir=2; 	 return 0;} else {return 1;}}
if(id.dir==2) {if((RT_HW_MILLIS-id.time)>=(id.timeFULL)) {id.dir=3;}}
if(id.dir==3) {if(!EN) 		    {return 0;} id.time=RT_HW_MILLIS; 
			   if(period==0)    {return 0;}			   
			   if(wprc==0) 	    {return 0;} 
			   if(wprc>=100) 	{return 1;} 
			   id.timeFULL=period; 			  
			   id.timeON=(((uint32_t)period*wprc)/100); 
			   id.dir=1; return 1;}		return 0;}
//=================================================================================================
//										Генератор PWW (период mcs, скважность)
//	Вход: id-рабочая структура, timeON-время включения в mсs, timeOFF-время выключения в ms.
//	Выход: возвращаемое значение bool -генерируемый импульс.
//================================================================================================= 			   
bool generatorPWMmcs  (RT_HW_GENERATOR_PWM_ID   &id, uint32_t period, uint8_t wprc, uint8_t EN=1){ 			//==Генератор PWM;
if(id.dir==0) {id.dir=3;}
if(id.dir==1) {if((RT_HW_MICROS-id.time)>= id.timeON) 	 {id.dir=2; 	 return 0;} else {return 1;}}
if(id.dir==2) {if((RT_HW_MICROS-id.time)>=(id.timeFULL)) {id.dir=3;}}
if(id.dir==3) {if(!EN) 		    {return 0;} id.time=RT_HW_MICROS; 
			   if(period==0)    {return 0;}			   
			   if(wprc==0) 	    {return 0;} 
			   if(wprc>=100) 	{return 1;} 
			   id.timeFULL=period; 			  
			   id.timeON=(((uint32_t)period*wprc)/100); 
			   id.dir=1; return 1;}		return 0;}
//=================================================================================================
//										Генератор событий с делителем
//================================================================================================= 
bool generatorFront(RT_HW_GENERATOR_FRONT_ID &id, char mode, uint8_t divide, uint8_t EN=1){
id.event=0;
if(EN){if(generatorChangeEN(id.ago,mode,EN)) {
if(++id.cnt > divide) {id.cnt=1; id.event=1; if(id.meander) {id.meander=0;} else {id.meander=1;}}}}
				 else {id.event=id.meander=0;}
return id.event; 	
};	
//=================================================================================================
//										Генератор (пачки)пачек импульсов
// 
//================================================================================================= 
bool generatorGroup(RT_HW_GENERATOR_GROUP_ID &id, uint16_t period, uint16_t periodFULL, uint8_t qnt, uint8_t EN=1){	
if(id.dir==0) {id.ago=0; id.dir++;}
if(id.dir==1) {id.meander=id.cnt=0;
			   if(periodFULL==0) {id.dir=2;} else   {id.time=RT_HW_MILLIS; id.dir=3;}}
if(id.dir==2) {if(generatorChangeEN(id.ago,'F',EN)) {id.time=RT_HW_MILLIS; id.dir=4;}} //--Обработка задержки
if(id.dir==3) {if(getPastMs(id.time)>periodFULL)    {id.dir=4;}} //     по времени при EN=1;
if(id.dir==4) {if(id.cnt>=qnt) {id.dir=1; id.meander=0;} else {id.dir++;}}
if(id.dir==5) {id.time=RT_HW_MILLIS; id.meander=1; id.dir++;}
if(id.dir==6) {if(getPastMs(id.time)>=period) {id.meander=0; id.time=RT_HW_MILLIS; id.dir++; }}
if(id.dir==7) {if(getPastMs(id.time)>=period) {++id.cnt; id.dir=4;}}
return id.meander;};
//=================================================================================================
//										Генератор задач с таймером
//================================================================================================= 
uint8_t generatorTask (RT_HW_GENERATOR_TASK_ID  &id, uint16_t period, uint8_t qnt, uint8_t EN=1){ 			
// id.event-номер задачи как событие, id.cnt-последний номер задачи; возвращает id.event;
if(EN){
if(periodMs(id.time,period)) {
if(++id.cnt > qnt) {id.cnt=1;} id.event=id.cnt; } else {id.event=0;};}
else {id.event=id.cnt=0;}
return id.event;}
//=================================================================================================
//										Генератор задач автономный
//================================================================================================= 
uint8_t generatorTask (uint8_t &cnt, uint8_t qnt, uint8_t EN=1){ 											
if(EN){if(++cnt > qnt) {cnt=1;} return cnt;} else {return 0;}}
//=================================================================================================
//							Защита от дребезга байта
//=================================================================================================
bool chatterV8(RT_HW_CHAT_V8_ID &id, uint8_t var){
if(id.dir==0) {if(id.ago!=var){id.timeBegin=RT_HW_MILLIS; id.dir++;}                    return id.ago;}
if(id.dir==1) {if((getPastMs  (id.timeBegin)< (id.period)) && (id.ago==var)) {id.dir=0; return id.ago;} 
			   if( getPastMs  (id.timeBegin)>= id.period)     {id.ago=var;	  id.dir=0; return id.ago;}}
																					    return id.ago;};





//=================================================================================================
//						Генератор событий с делителем (без структуры) bit::2-ago veN, 0-event, 1-meander;
//================================================================================================= 
void generatorDivider(uint8_t &state, uint8_t &cnt, uint8_t divider, bool v){
state&=0xFE; if(bitRead(state,2)!=v) {bitWrite(state,2,v);  if(v){if(++cnt>divider) {cnt=1; state|=1; state^=2;}}}};
void generatorDivider(uint8_t &state, int16_t &cnt, int16_t divider, bool v){
state&=0xFE; if(bitRead(state,2)!=v) {bitWrite(state,2,v);  if(v){if(++cnt>divider) {cnt=1; state|=1; state^=2;}}}};	
void generatorDivider(uint8_t &state, uint8_t &cnt, uint8_t divider){state&=0xFE; if(++cnt>divider) {cnt=1; state|=1; state^=2;}};
void generatorDivider(uint8_t &state, int16_t &cnt, int16_t divider){state&=0xFE; if(++cnt>divider) {cnt=1; state|=1; state^=2;}};
//================================================================================================= 
//							 Кодировщик числа в номер бита
//================================================================================================= 
uint8_t  coder8 (uint8_t  v) {if((v==0) || (v>8)) {return 0;} uint8_t  dc=0; bitSet(dc,(v-1)); return dc;};  //-- 8-разрядный;
uint16_t coder16(uint16_t v) {if((v==0) || (v>16)){return 0;} uint16_t dc=0; bitSet(dc,(v-1)); return dc;};  //--16-разрядный;
//=================================================================================================