//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							1.Справочная информация
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ESP32->M5STACK:
// setRxTimeout устанавливает таймаут, по истечении которого будет вызван обратный вызов onReceive 
//  (после получения данных он ждет это время бездействия UART rx для вызова обратного вызова fnc)
// параметр symbol_timeout определяет порог тайм-аута в периодах символа uart. 
// Установка таймаута символа 0 отключает обратный вызов по таймауту.
// Настройка максимального тайм-аута рассчитывается IDF автоматически. 
// Если установлено выше максимального значения, оно игнорируется, 
//                                             и на Serial0 печатается ошибка (проверьте консоль).
// Примеры: максимум для 11-битного символа — 92 (SERIAL_8N2, SERIAL_8E1, SERIAL_8O1 и т. д.), 
//          максимум для 10-битного символа — 101 (SERIAL_8N1).
// Приме:   symbols_timeout=1 определяет тайм-аут, равный времени передачи одного символа (~11 бит) 
//                                                               на текущей скорости передачи данных.
//          Для скорости 9600 бод, SERIAL_8N1 (10-битный символ) и symbol_timeout = 3 
//                                                       тайм-аут будет 3 / (9600 / 10) = 3,125 мс;
//-------------------------------------------------------------------------------------------------
// ESP32->M5STACK:
// onReceive установит обратный вызов, который будет вызываться всякий раз, 
//              когда происходит прерывание UART (UART_INTR_RXFIFO_FULL или UART_INTR_RXFIFO_TOUT).
// Прерывание UART_INTR_RXFIFO_FULL срабатывает при получении UART_FULL_THRESH_DEFAULT байтов 
//                                                      (по умолчанию в IDF определено 120 байтов).
// Прерывание UART_INTR_RXFIFO_TOUT срабатывает при передаче символов UART_TOUT_THRESH_DEFAULT без приема 
//                                                      (по умолчанию в IDF определено 10 символов).
// Параметр onlyOnTimeout будет определять, как будет вести себя onReceive:
// По умолчанию: true -- Обратный вызов будет вызван только тогда, когда произойдет тайм-аут RX.
//                       Весь поток байтов будет сразу готов к чтению функцией обратного вызова.
//                       Эта опция может привести к переполнению Rx в зависимости от размера буфера Rx 
//                                                   и количества байтов, полученных в потоковой передаче.
//               false --Обратный вызов будет вызван, когда FIFO достигнет 120 байт, а также по тайм-ауту RX.
//                       Поток непереданных байтов будет "разбит" на блоки по 120 байт при каждом обратном вызове.
//                       Этот параметр позволяет избежать любого вида переполнения Rx, 
//                                       но оставляет работу по повторной сборке пакетов UART приложению.
//-------------------------------------------------------------------------------------------------
//void begin(unsigned long baud, uint32_t config=SERIAL_8N1, int8_t rxPin=-1, int8_t txPin=-1, bool invert=false, unsigned long timeout_ms = 20000UL, uint8_t rxfifo_full_thrhd = 112);
// uart_t* uartBegin(uint8_t uart_nr, uint32_t baudrate, uint32_t config, int8_t rxPin, int8_t txPin, uint16_t rx_buffer_size, uint16_t tx_buffer_size, bool inverted, uint8_t rxfifo_full_thrhd); //ESP32
// ESP32->M5STACK:
// uart_t* uartBegin(uint8_t uart_nr, uint32_t baudrate, uint32_t config, int8_t rxPin, int8_t txPin, uint16_t rx_buffer_size, uint16_t tx_buffer_size, bool inverted, uint8_t rxfifo_full_thrhd); //M5STACK
// setPins(int8_t rxPin, int8_t txPin, int8_t ctsPin = -1, int8_t rtsPin = -1) устанавливает пины UART.
//          										setPins() должна вызываться после Serial.begin().          
// setHwFlowCtrlMode(uint8_t mode = HW_FLOWCTRL_CTS_RTS, uint8_t threshold = 64) -
//                Включает или отключает аппаратное управление потоком с использованием выводов RTS и/или CTS 
//                                                          (перед этим необходимо использовать setPins()).

//-------------------------------------------------------------------------------------------------
// ESP32->ESP32S,ESP32S2,ESPS3,ESP32C3,M5STACK:
//extern HardwareSerial Serial0;
//#else
//extern HardwareSerial Serial;
//#endif
//
//#if SOC_UART_NUM > 1
//extern HardwareSerial Serial1;
//#endif
//#if SOC_UART_NUM > 2
//extern HardwareSerial Serial2;
//#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							1.СОЗДАНИЕ ОБЪЕКТОВ с данными
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_STRUCT_UART_RXTX uart;
#if defined(RT_HW_CORE_STM32_D) || defined(RT_HW_CORE_SEEED) || defined(RT_HW_CORE_RP_PICO) || defined(RT_HW_CORE_ESP)
RT_HW_STRUCT_UART_USB uartUSB;
#endif
#if defined(RT_HW_PERMIT_UART0) 
RT_HW_STRUCT_UART LinkUART0;
#endif
#if defined(RT_HW_PERMIT_UART1)
RT_HW_STRUCT_UART LinkUART1;
#endif
#if defined(RT_HW_PERMIT_UART2)
RT_HW_STRUCT_UART LinkUART2;
#endif
#if defined(RT_HW_PERMIT_UART3)
RT_HW_STRUCT_UART LinkUART3;
#endif
#if defined(RT_HW_PERMIT_UART4)
RT_HW_STRUCT_UART LinkUART4;
#endif
#if defined(RT_HW_PERMIT_UART5)
RT_HW_STRUCT_UART LinkUART5;
#endif
#if defined(RT_HW_PERMIT_UART6)
RT_HW_STRUCT_UART LinkUART6;
#endif
#if defined(RT_HW_PERMIT_UART7)
RT_HW_STRUCT_UART LinkUART7;
#endif
#if defined(RT_HW_PERMIT_UART8)
RT_HW_STRUCT_UART LinkUART8;
#endif
#if defined(RT_HW_PERMIT_UARTS)
RT_HW_STRUCT_UART LinkUARTS;
#endif
#if defined(RT_HW_PERMIT_UARTB)
RT_HW_STRUCT_BLUETOOTH LinkUARTB;	
#endif					    

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//										API UART
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
//=================================================================================================
//							2.СЕРВИСНЫЕ УТИЛИТЫ
//=================================================================================================
void    uartInitParam (uint8_t bus){						//++++2.1.Инициализация default параметров
#if defined(RT_HW_PERMIT_UART0)							
if(bus==0) {uartSetPinRX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,0),0); //--set pin RX;
            uartSetPinTX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,1),0); //--set pin TX;
return;}
#endif
#if defined(RT_HW_PERMIT_UART1)							
if(bus==1) {uartSetPinRX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,2),1); //--set pin RX;
            uartSetPinTX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,3),1); //--set pin TX;
//-----------------------------------------------------------------------------
// Временная вставка. FLProg modbus настраивает Serial1 для ESP32 с недопустимыми пинами
// что приводит к перезагрузкам при запуске программмы заранее настраиваем ESP32 
// с пинами для контроллеров, для которых по их описанию допустимо использовать UART1.
//-----------------------------------------------------------------------------
#if defined(RT_HW_CORE_ESP32) 	
uartBegin(1); 																												
#endif	
return;}
#endif
#if defined(RT_HW_PERMIT_UART2)							
if(bus==2) {uartSetPinRX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,4),2); //--set pin RX;
            uartSetPinTX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,5),2); //--set pin TX;
return;}
#endif
#if defined(RT_HW_PERMIT_UART3)							
if(bus==3) {uartSetPinRX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,6),3); //--set pin RX;
            uartSetPinTX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,7),3); //--set pin TX;
return;}
#endif
#if defined(RT_HW_PERMIT_UART4)							
if(bus==4) {uartSetPinRX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,8),4); //--set pin RX;
            uartSetPinTX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,9),4); //--set pin TX;
return;}
#endif
#if defined(RT_HW_PERMIT_UART5)							
if(bus==5) {uartSetPinRX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,10),5); //--set pin RX;
            uartSetPinTX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,11),5); //--set pin TX;
return;}
#endif
#if defined(RT_HW_PERMIT_UART6)							
if(bus==6) {uartSetPinRX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,12),6); //--set pin RX;
            uartSetPinTX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,13),6); //--set pin TX;
return;}
#endif
#if defined(RT_HW_PERMIT_UART7)							
if(bus==7) {uartSetPinRX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,14),7); //--set pin RX;
            uartSetPinTX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,15),7); //--set pin TX;
return;}
#endif
#if defined(RT_HW_PERMIT_UART8)							
if(bus==8) {uartSetPinRX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,13),8); //--set pin RX;
            uartSetPinTX(readArrPGM(RT_HW_PGM_PIN_UHS_ID,14),8); //--set pin TX;
return;}
#endif
#if defined(RT_HW_PERMIT_UARTS)
if(bus==8) {uartSetPinRX(readArrPGM(RT_HW_PGM_PIN_USS_ID,0),9);
            uartSetPinTX(readArrPGM(RT_HW_PGM_PIN_USS_ID,1),9);
return;}						
#endif
};
//=================================================================================================
// 		    1.Функции чтения/записи флагов begin,ready, вывод заголовка, 
//            запись/чтение имени встроенного BlueTooth
//=================================================================================================
bool     uartCheckBegin(uint8_t num=0){return bitRead(uart.begin,num);}
bool     uartCheckReady(uint8_t num=0){return bitRead(uart.ready,num);}
uint16_t uartGetBegin(){return uart.begin;}
uint16_t uartGetReady(){return uart.ready;}	
void     uartSetNameBTH(String name){
#if defined(RT_HW_PERMIT_UARTB)		
LinkUARTB.name=name; return;			
#endif		
name=" ";};
String   uartGetNameBTH(){
#if defined(RT_HW_PERMIT_UARTB)		
return LinkUARTB.name;			
#endif		 
return String(F("No Bluetooth"));}
void     uartHeadInterface(uint8_t num=0){								
if(!uartCheckNum(num))			{return;}	//--Проверка наличия шины;
consoleBegin();	if(!console.ok)	{return;} 	//--Настройка консоли с параметрами по умолчанию и ожидание готовности;
mess.alig='R'; mess.blank='G'; mess.modePin='A'; mess.modeAdr='A'; mess.qntBit=8; mess.signBit='1'; 
if(num==9) {uint8_t arr[]={uartGetPinRX(num),uartGetPinTX(num)}; 
	        consoleVar(String(F("SoftwareSerial: "))); 
            consoleVar(String(F(" RX,TX="))); consoleArrVar(arr,'E',0,'P',2);   return;}
if(num==10){consoleVar(String(F("ESP32 Bluetooth name="))); consoleVar(uartGetNameBTH(),'E'); return;} 
//-------------------------------------------------------------------------------------------------
	       {uint8_t arr[]={uartGetPinRX(num),uartGetPinTX(num)}; 
            consoleVar(String(F("UART"))); consoleVar(num,':');
            consoleVar(String(F(" RX,TX="))); consoleArrVar(arr,'E',0,'P',2);   return;}
};
//=================================================================================================
// 			2.Проверка доступности UART, pапись/чтение слова состояния и скорости, конфигурации UART
//=================================================================================================
bool     uartCheckNum (uint8_t num=0){
#if defined(RT_HW_PERMIT_UART0)
if(num==0){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART1)
if(num==1){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART2)
if(num==2){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART3)
if(num==3){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART4)
if(num==4){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART5)
if(num==5){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART6)
if(num==6){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART7)
if(num==7){return 1;}
#endif
#if defined(RT_HW_PERMIT_UART8)
if(num==8){return 1;}
#endif
#if defined(RT_HW_PERMIT_UARTS)
if(num==9){return 1;}
#endif
#if defined(RT_HW_PERMIT_UARTB)
if(num==10){return 1;}
#endif
return 0;};
void     uartSetState (uint8_t state,   uint8_t num=0){
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {LinkUART0.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {LinkUART1.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {LinkUART2.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {LinkUART3.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {LinkUART4.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {LinkUART5.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {LinkUART6.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {LinkUART7.state=state;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {LinkUART8.state=state;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {LinkUARTS.state=state;}
#endif
#if defined(RT_HW_PERMIT_UARTB)	
if(num==10) {LinkUARTB.state=state;}
#endif
};
void     uartSetSpeed (uint32_t speed,  uint8_t num=0){
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {LinkUART0.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {LinkUART1.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {LinkUART2.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {LinkUART3.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {LinkUART4.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {LinkUART5.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {LinkUART6.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {LinkUART7.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {LinkUART8.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {LinkUARTS.speed=speed;}
#endif
#if defined(RT_HW_PERMIT_UARTB)	
if(num==10) {LinkUARTB.speed=speed;}
#endif
};
void     uartSetConfig(uint32_t config, uint8_t num=0){
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {LinkUART0.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {LinkUART1.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {LinkUART2.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {LinkUART3.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {LinkUART4.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {LinkUART5.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {LinkUART6.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {LinkUART7.config=config;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {LinkUART8.config=config;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {LinkUARTS.config=config;}
#endif
#if defined(RT_HW_PERMIT_UARTB)	
#endif
};
uint8_t  uartGetState (uint8_t num=0){
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {return LinkUART0.state;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {return LinkUART1.state;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {return LinkUART2.state;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {return LinkUART3.state;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {return LinkUART4.state;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {return LinkUART5.state;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {return LinkUART6.state;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {return LinkUART7.state;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {return LinkUART8.state;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {return LinkUARTS.state;}
#endif
#if defined(RT_HW_PERMIT_UARTB)	
if(num==10) {return LinkUARTB.state;}
#endif
return 0;};
uint32_t uartGetSpeed (uint8_t num=0){
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {return LinkUART0.speed;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {return LinkUART1.speed;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {return LinkUART2.speed;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {return LinkUART3.speed;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {return LinkUART4.speed;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {return LinkUART5.speed;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {return LinkUART6.speed;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {return LinkUART7.speed;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {return LinkUART8.speed;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {return LinkUARTS.speed;}
#endif
#if defined(RT_HW_PERMIT_UARTB)	
if(num==10) {return (uint32_t)115200;}
#endif
return (uint32_t)115200;};
uint32_t uartGetConfig(uint8_t num=0){	
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {return LinkUART0.config;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {return LinkUART1.config;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {return LinkUART2.config;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {return LinkUART3.config;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {return LinkUART4.config;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {return LinkUART5.config;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {return LinkUART6.config;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {return LinkUART7.config;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {return LinkUART8.config;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {return LinkUARTS.config;}
#endif
#if defined(RT_HW_PERMIT_UARTB)	
if(num==10) {return 0x800001C;}
#endif
return 0x800001C;};
//=================================================================================================
// 			3.Запись/чтение номеров пинов, скорости
//=================================================================================================
void     uartSetPinRX(uint8_t rx, uint8_t num=0){
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {LinkUART0.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {LinkUART1.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {LinkUART2.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {LinkUART3.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {LinkUART4.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {LinkUART5.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {LinkUART6.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {LinkUART7.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {LinkUART8.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {LinkUARTS.rx=rx;}
#endif
#if defined(RT_HW_PERMIT_UARTB)	
#endif
};
void     uartSetPinTX(uint8_t tx, uint8_t num=0){
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {LinkUART0.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {LinkUART1.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {LinkUART2.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {LinkUART3.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {LinkUART4.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {LinkUART5.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {LinkUART6.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {LinkUART7.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {LinkUART8.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {LinkUARTS.tx=tx;}
#endif
#if defined(RT_HW_PERMIT_UARTB)	
#endif
};
uint8_t  uartGetPinRX(uint8_t num=0){
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {return LinkUART0.rx;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {return LinkUART1.rx;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {return LinkUART2.rx;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {return LinkUART3.rx;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {return LinkUART4.rx;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {return LinkUART5.rx;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {return LinkUART6.rx;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {return LinkUART7.rx;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {return LinkUART8.rx;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {return LinkUARTS.rx;}
#endif
#if defined(RT_HW_PERMIT_UARTB)	
#endif
return 255;};
uint8_t  uartGetPinTX(uint8_t num=0){
#if defined(RT_HW_PERMIT_UART0)	
if(num==0) {return LinkUART0.tx;}
#endif
#if defined(RT_HW_PERMIT_UART1)	
if(num==1) {return LinkUART1.tx;}
#endif
#if defined(RT_HW_PERMIT_UART2)	
if(num==2) {return LinkUART2.tx;}
#endif
#if defined(RT_HW_PERMIT_UART3)	
if(num==3) {return LinkUART3.tx;}
#endif
#if defined(RT_HW_PERMIT_UART4)	
if(num==4) {return LinkUART4.tx;}
#endif
#if defined(RT_HW_PERMIT_UART5)	
if(num==5) {return LinkUART5.tx;}
#endif
#if defined(RT_HW_PERMIT_UART6)	
if(num==6) {return LinkUART6.tx;}
#endif
#if defined(RT_HW_PERMIT_UART7)	
if(num==7) {return LinkUART7.tx;}
#endif
#if defined(RT_HW_PERMIT_UART8)	
if(num==8) {return LinkUART8.tx;}
#endif
#if defined(RT_HW_PERMIT_UARTS)	
if(num==9) {return LinkUARTS.tx;}
#endif
#if defined(RT_HW_PERMIT_UARTB)	
#endif
return 255;};
//=================================================================================================
// 			4.Инициализация uartBegin(n)
//void begin(unsigned long baud, uint32_t config=SERIAL_8N1, int8_t rxPin=-1, int8_t txPin=-1, bool invert=false, unsigned long timeout_ms = 20000UL, uint8_t rxfifo_full_thrhd = 112);
//=================================================================================================	
void     uartBegin(uint8_t num=0);

//void begin(unsigned long baud, uint32_t config=SERIAL_8N1, int8_t rxPin=-1, int8_t txPin=-1, bool invert=false, unsigned long timeout_ms = 20000UL, uint8_t rxfifo_full_thrhd = 112);
//=================================================================================================	
/*
void     uartBegin(uint8_t n=0){
#if defined(RT_HW_PERMIT_UART0)	
//- - - - - - - - - - - - - - - - - - - - - - - -
#if defined(RT_HW_CORE_ESP32S) || defined(RT_HW_CORE_ESP32S2) || defined(RT_HW_CORE_ESP32S3)
if(n==0){if(uartGetState(n)==0){uartSetState(1,n); Serial.begin(uartGetSpeed(n),uartGetConfig(n),(int8_t)uartGetPinRX(n),(int8_t)uartGetPinTX(n), false, 20000UL, 112);}                                                                                  return;}
#endif
//- - - - - - - - - - - - - - - - - - - - - - - -
#if defined(RT_HW_CORE_ESP2C3)
if(n==0){if(uartGetState(n)==0){uartSetState(1,n); Serial.begin(uartGetSpeed(n),uartGetConfig(n),(int8_t)uartGetPinRX(n),(int8_t)uartGetPinTX(n), false, 20000UL, 112);}                                                                                  return;}
#endif
//- - - - - - - - - - - - - - - - - - - - - - - -
if(n==0){if(uartGetState(n)==0){uartSetState(1,n); Serial.begin(uartGetSpeed(n));} return;}
#endif	
#if defined(RT_HW_PERMIT_UART1)
//- - - - - - - - - - - - - - - - - - - - - - - -
#if defined(RT_HW_CORE_ESP32S) || defined(RT_HW_CORE_ESP32S2) || defined(RT_HW_CORE_ESP32S3)
if(n==1){if(uartGetState(n)==0){uartSetState(1,n); Serial1.begin(uartGetSpeed(n),uartGetConfig(n),(int8_t)uartGetPinRX(n),(int8_t)uartGetPinTX(n), false, 20000UL, 112);}  return;}
#endif
//- - - - - - - - - - - - - - - - - - - - - - - -
#if defined(RT_HW_CORE_ESP32C3)
if(n==1){if(uartGetState(n)==0){uartSetState(1,n); Serial1.begin(uartGetSpeed(n),uartGetConfig(n),(int8_t)uartGetPinRX(n),(int8_t)uartGetPinTX(n),false, 20000UL, 112);}  return;}
#endif
//- - - - - - - - - - - - - - - - - - - - - - - -
if(n==1){if(uartGetState(n)==0){uartSetState(1,n); Serial1.begin(uartGetSpeed(n));} return;}
#endif	
#if defined(RT_HW_PERMIT_UART2)
#if defined(RT_HW_CORE_ESP32S) || defined(RT_HW_CORE_ESP32S2) || defined(RT_HW_CORE_ESP32S3)
if(n==2){if(uartGetState(n)==0){uartSetState(1,n); Serial2.begin(uartGetSpeed(n),uartGetConfig(n),(int8_t)uartGetPinRX(n),(int8_t)uartGetPinTX(n), false, 20000UL, 112);}  return;}
#endif
if(n==2){if(uartGetState(n)==0){uartSetState(1,n); Serial2.begin(uartGetSpeed(n));} return;}
#endif	
#if	defined(RT_HW_PERMIT_UART3)
if(n==3){if(uartGetState(n)==0){uartSetState(1,n); Serial3.begin(uartGetSpeed(n));}}
#endif
#if defined(RT_HW_PERMIT_UART4)
if(n==4){if(uartGetState(n)==0){uartSetState(1,n); Serial4.begin(uartGetSpeed(n));}}
#endif	
#if defined(RT_HW_PERMIT_UART5)
if(n==5){if(uartGetState(n)==0){uartSetState(1,n); Serial5.begin(uartGetSpeed(n));}}
#endif
#if defined(RT_HW_PERMIT_UART6)
if(n==6){if(uartGetState(n)==0){uartSetState(1,n); Serial6.begin(uartGetSpeed(n));}}
#endif
#if defined(RT_HW_PERMIT_UART7)
if(n==7){if(uartGetState(n)==0){uartSetState(1,n); Serial7.begin(uartGetSpeed(n));}}
#endif
#if defined(RT_HW_PERMIT_UART8)
if(n==8){if(uartGetState(n)==0){uartSetState(1,n); Serial8.begin(uartGetSpeed(n));}}
#endif
#if defined(RT_HW_PERMIT_UARTS)
if(n==9){if(uartGetState(n)==0){uartSetState(1,n); SerialS.begin(uartGetSpeed(n));}}
#endif
#if defined(RT_HW_PERMIT_UARTB)
if(n==10){if(uartGetState(n)==0){uartSetState(1,n); SerialB.begin(LinkUARTB.name);}}
#endif				
}
*/



//=================================================================================================
// 			5.Проверка подключения устройства (Serial)
//=================================================================================================		
bool     uartSerial (uint8_t num=0);
//=================================================================================================
// 			6.Проверка буферов ввода/вывода
//=================================================================================================
uint16_t uartAvailable(uint8_t num=0);	
uint16_t uartAvailableForWrite(uint8_t num=0);
//=================================================================================================
// 			7.Ввод/вывод
//=================================================================================================
uint8_t  uartRead(uint8_t n);
void     uartWrite(uint8_t n, uint8_t val);
void     uartPrint(uint8_t n, String str);
//=================================================================================================

