//=================================================================================================
//                         Шина i2c
//ecoins@mail.ru 01.11.2021г.
//=================================================================================================
// AVR  	- одна шина i2c(Wire),  объединенная библиотека master/slave;
// ESP8266  - одна шина i2c(Wire),  объединенная библиотека master/slave;
// ESP32S   - две  шины i2c master(Wire1,Wire2), библиотека master - Wire.h, 
//            при использовании библиотеки slave - ESP32_I2C_Slave (требуется загрузка доп.библиотеки)
//            две шины i2с slave(WireSlave1, WireSlave2).
//            *** одновременно можно использовать только две шины в разном сочетании master/slave; 
// SAM3     - одна шина i2c(Wire),  объединенная библиотека master/slave;
// Seeeduino- до 6 шин  i2c(Wire,Wire1-5), объединенная библиотека master/slave;
// STM32    - до 3 шин  i2c(Wire определена, 
//                          Wire1 нужно определять TwoWire Wire1(RT_HW_I2C1_SDA,  RT_HW_I2C1_SCL),
//                          Wire2 нужно определять TwoWire Wire2(RT_HW_I2C2_SDA,  RT_HW_I2C2_SCL);
//			  для STM32duino     - объединенная библиотека master/slave;
//			  для STM32master_F1 - библиотека master - Wire.h, библиотека slave - Wire_slave.h (в core STM32master), 
//			  для STM32master_F4 - библиотека master - Wire.h, библиотека slave - отсутствует;
// Raspberry Pi Pico - две шины.
//
//struct RT_HW_STRUCT_I2C_FUNC {
//uint8_t	 adr=255; 		//--Адрес;	
//uint8_t  bus=0;			//--Номер шины;
//uint8_t  stop=1;		//--Cтоповый бит;
//uint8_t  idx;			//--Рабочий индекс;
//uint32_t speed=400000UL; //--Скорость шины;	
//};
//=================================================================================================	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						1.Структура для шины с i2с
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_STRUCT_I2C {
	uint8_t  sda=255; 	//--пин SDA;
	uint8_t  scl=255;	//--пин SCL;  
	uint8_t  adr=255; 	//--Адрес: для Master всегда=0;
	uint8_t  busy=0; 	//--Состояние занятости шины: =10-для i2c0; =11-для i2c1; =12-для i2c2;	
	uint8_t  codeErr=0; //--Код ошибки;
	uint8_t  status=0; 	//--Настройка шины: =0 - не настроена; =1 Master; =2 Slave;
	uint32_t speed=400000UL; //--Скорость шины (по умолчанию);
	uint32_t timeOut;   //--Регистр для тайм-аута при приеме байтов;
	uint32_t cntByte;   //--Счетчик принятых байт;
    uint8_t  ddr=0;
	uint8_t  idx;
    uint16_t buff;
    uint16_t cntScan=0;
	uint8_t  i2cDelay;	//--Задержка для программного i2c(STM32_M)	
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						2.Структура для функций i2с
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_STRUCT_I2C_FUNC {
uint8_t  codeErr=0; 	//--Код ошибки;
uint8_t	 adr=255; 		//--Адрес;	
uint8_t  bus=0;			//--Номер шины;
uint8_t  stop=1;		//--Cтоповый бит;
uint8_t  idx;			//--Рабочий индекс;
uint8_t  dir=0;			//--Этапы внутри функций;
uint32_t speed=400000UL; //--Скорость шины;	
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							1.СОЗДАНИЕ ОБЪЕКТОВ с данными
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_PERMIT_I2C0)								//++++Данные для i2c0------------------			  
RT_HW_STRUCT_I2C 	LinkI2C0;										
#endif	//-----------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C1)								//++++Данные для i2c1------------------
RT_HW_STRUCT_I2C 	LinkI2C1;
#endif	//-----------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C2)								//++++Данные для i2c2------------------
RT_HW_STRUCT_I2C 	LinkI2C2;
#endif	//-----------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2CS)								//++++Данные для i2cS (soft i2c)
RT_HW_STRUCT_I2C 	LinkI2CS;
#endif	//-----------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2CS1)								//++++Данные для i2cS1(soft i2c)
RT_HW_STRUCT_I2C 	LinkI2CS1;										
#endif	//-----------------------------------------------------------------------------------------
//=================================================================================================
//							2.СЕРВИСНЫЕ УТИЛИТЫ
//=================================================================================================
void    i2cInitParam (uint8_t bus){						//++++2.1.Инициализация default параметров
#if defined(RT_HW_PERMIT_I2C0)							
if(bus==0) {LinkI2C0.busy=LinkI2C0.status=LinkI2C0.codeErr=0;	//--clear busy,status,codeErr;
			LinkI2C0.sda =readArrPGM(RT_HW_PGM_PIN_I2C_ID,0);	//--set pin SDA; 
			LinkI2C0.scl =readArrPGM(RT_HW_PGM_PIN_I2C_ID,1);	//--set pin SCL;
			LinkI2C0.speed=RT_HW_I2C_SPEED;            	    //--set speed;
return;}
#endif
#if defined(RT_HW_PERMIT_I2C1) 
if(bus==1){LinkI2C1.busy=LinkI2C1.status=LinkI2C1.codeErr=0;	//--clear busy,status,codeErr;
		   LinkI2C1.sda =readArrPGM(RT_HW_PGM_PIN_I2C_ID,2);	//--set pin SDA; 
		   LinkI2C1.scl =readArrPGM(RT_HW_PGM_PIN_I2C_ID,3);	//--set pin SCL;
		   LinkI2C1.speed=RT_HW_I2C_SPEED;            	    //--set speed;
return;}
#endif
#if defined(RT_HW_PERMIT_I2C2)
if(bus==2){LinkI2C2.busy=LinkI2C2.status=LinkI2C0.codeErr=0;	//--clear busy,status,codeErr;
		   LinkI2C2.sda =readArrPGM(RT_HW_PGM_PIN_I2C_ID,4);	//--set pin SDA; 
		   LinkI2C2.scl =readArrPGM(RT_HW_PGM_PIN_I2C_ID,5);	//--set pin SCL;
		   LinkI2C2.speed=RT_HW_I2C_SPEED;            	    //--set speed; 	
return;}
#endif
#if defined(RT_HW_PERMIT_I2CS)	//--Настройка i2cS  применяется для контроллеров STM32F1_M
if(bus==0){LinkI2CS.busy=LinkI2CS.status=LinkI2CS.codeErr=0;	//--clear busy,status,codeErr;
		   LinkI2CS.sda =readArrPGM(RT_HW_PGM_PIN_ICS_ID,0);	//--set pin SDA; 
		   LinkI2CS.scl =readArrPGM(RT_HW_PGM_PIN_ICS_ID,1);	//--set pin SCL;
		   LinkI2CS.speed=RT_HW_I2C_SPEED;            	    	//--set speed;
return;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	//--Настройка i2cS1 применяется для контроллеров STM32F1_M
if(bus==1){LinkI2CS1.busy=LinkI2CS1.status=LinkI2CS1.codeErr=0;	//--clear busy,status,codeErr;
		   LinkI2CS1.sda =readArrPGM(RT_HW_PGM_PIN_ICS_ID,2);	//--set pin SDA; 
		   LinkI2CS1.scl =readArrPGM(RT_HW_PGM_PIN_ICS_ID,3);	//--set pin SCL;
		   LinkI2CS1.speed=RT_HW_I2C_SPEED;            	    	//--set speed;  	
return;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
};
void    i2cHeadInterface(uint8_t bus=0){			    //++++2.2.Вывод заголовка по интерфейсу i2c						
if(!i2cCheckBus(bus))			{return;}	//--Проверка наличия шины;
consoleBegin();	if(!console.ok)	{return;} 	//--Настройка консоли с параметрами по умолчанию и ожидание готовности;
mess.alig='R'; mess.blank='G'; mess.modePin='A'; mess.modeAdr='A'; mess.qntBit=8; mess.signBit='1'; 
consoleVar(String(F("I2C"))); consoleVar(bus);
consoleVar(String(F(" bus="))); consoleVar(bus,':');
consoleVar(String(F(" SDA,SCL="))); consoleVar(i2cGetPinSDA(bus),',',0,'P'); consoleVar(i2cGetPinSCL(bus),';',0,'P'); consoleBlank(15);
	 if(i2cGetStatus(bus)==0) {consoleVar(String(F("No init")),';');} 
	 if(i2cGetStatus(bus)==1) {consoleVar(String(F("Master ")),';');} 
	 if(i2cGetStatus(bus)==2) {consoleVar(String(F("Slave  ")),';');} 
consoleVar(String(F(" speed(kHz)="))); consoleVar((uint32_t)(i2cGetSpeed(bus)/1000),';');
consoleCR();}							
uint8_t i2cScanBus   (uint8_t bus=0, uint32_t sp=0){    //++++2.3.Сканирование шины i2c============							
static uint8_t codeErr=0;
static uint8_t cnt=0;
static uint8_t idx;
//-------------------------------------------------------------------------------------------------
//if(step==90){return 1;}			//++++Заглушка для действий после  вывода заголовка------------
i2cBegin(codeErr,bus);           
consoleBegin(); if(!console.ok){return 1;}	//--Настройка консоли и ожидание готовности;
i2cBegin(codeErr,bus);
//i2cHeadInterface(bus);
mess.alig='R'; mess.blank='G';  mess.modePin='T'; 
consoleVar(String(F("I2C")));           consoleVar(bus,'~');
consoleVar(String(F(" bus=")));         consoleVar(bus,':'); 
//consoleVar(String(F(" SDA,SCL="))); consoleVar(i2cGetPinSDA(bus),',',0,'P'); consoleVar(i2cGetPinSCL(bus),';',0,'P');  
consoleVar(String(F(" scan->")));
     if(i2cGetStatus(bus)==0) {consoleVar(String(F("No init")),'E'); return 1;} 
//	 if(i2cGetStatus(bus)==1) {consoleVar(String(F(" Master")),'E');} 
	 if(i2cGetStatus(bus)==2) {consoleVar(String(F(" Slave")),'E');  return 1;} 
if(i2cGetStatus(bus)!=1) {consoleVar(String(F(" No Master")),'E');   return 1;}
idx=1;   codeErr=0; cnt=0;
//-------------------------------------------------------------------------------------------------
do{if(idx>127){break;}
   i2cFindAdr(codeErr,idx,bus,sp); 
   if(codeErr==0){if(cnt>0){if((mess.modeAdr=='A') && (cnt==6)) {consoleCR();}
					      else if((mess.modeAdr=='H') && (cnt==10)){consoleCR();}
					      else if((mess.modeAdr=='D') && (cnt==12)){consoleCR();}
						  else                                     {consoleChar(',');}}                                   
   cnt++; consoleVar(idx,'~',0,'A'); }
idx++;}while(1);
if(cnt>0){consoleChar(';');} 
consoleTest(String(F(" qnt")),cnt,'E'); 	
return 1;}

/*
uint8_t i2cScanBus(RT_HW_STRUCT_I2C_FUNC &id, uint8_t &qnt, uint8_t &cnt){    //++++2.3.Сканирование шины i2c============							
static uint8_t codeErr=0;
static uint8_t cnt=0;
static uint8_t idx;
//-------------------------------------------------------------------------------------------------
//if(step==90){return 1;}			//++++Заглушка для действий после  вывода заголовка------------
i2cBegin(codeErr,bus);           
consoleBegin(); if(!console.ok){return 1;}	//--Настройка консоли и ожидание готовности;
i2cBegin(codeErr,bus);
//i2cHeadInterface(bus);
mess.alig='R'; mess.blank='G';  mess.modePin='T'; 
consoleVar(String(F("I2C")));           consoleVar(bus,'~');
consoleVar(String(F(" bus=")));         consoleVar(bus,':'); 
//consoleVar(String(F(" SDA,SCL="))); consoleVar(i2cGetPinSDA(bus),',',0,'P'); consoleVar(i2cGetPinSCL(bus),';',0,'P');  
consoleVar(String(F(" scan->")));
     if(i2cGetStatus(bus)==0) {consoleVar(String(F("No init")),'E'); return 1;} 
//	 if(i2cGetStatus(bus)==1) {consoleVar(String(F(" Master")),'E');} 
	 if(i2cGetStatus(bus)==2) {consoleVar(String(F(" Slave")),'E');  return 1;} 
if(i2cGetStatus(bus)!=1) {consoleVar(String(F(" No Master")),'E');   return 1;}
idx=1;   codeErr=0; cnt=0;
//-------------------------------------------------------------------------------------------------
do{if(idx>127){break;}
   i2cFindAdr(codeErr,idx,bus,sp); 
   if(codeErr==0){if(cnt>0){if((mess.modeAdr=='A') && (cnt==6)) {consoleCR();}
					      else if((mess.modeAdr=='H') && (cnt==10)){consoleCR();}
					      else if((mess.modeAdr=='D') && (cnt==12)){consoleCR();}
						  else                                     {consoleChar(',');}}                                   
   cnt++; consoleVar(idx,'~',0,'A'); }
idx++;}while(1);
if(cnt>0){consoleChar(';');} 
consoleTest(String(F(" qnt")),cnt,'E'); 	
return 1;}
*/

uint8_t i2cInitDevice(RT_HW_STRUCT_I2C_FUNC &id){		//++++Проверка bus,adr, инициализация i2c: ok=1; err=0->codeErr=60,61,62;
id.codeErr=0;
               if(!i2cCheckBus(id.bus))	    {id.codeErr=60;}	//--Check bus;
if(id.codeErr==0){if(id.adr>254) 					{id.codeErr=61;}}   //--Check adr;
if(id.codeErr==0){if(!i2cBeginMaster(id.codeErr,id.bus)){return 0;}} 	//--Init  i2c;
if(id.codeErr==0){if(i2cGetStatus(id.bus)!=1)		{id.codeErr=62;}}	//--Check i2c as Master;
return  1;	
};	//++++spiInitDevice()--------------------------------------------------------------------------
uint8_t i2cFindAdr   (RT_HW_STRUCT_I2C_FUNC &id){       //++++Поиск адреса на шине----
if(i2cSetBusy(id.bus)==0){return 0;}; if(id.speed!=0){i2cSetClock(id.speed,id.bus);}	//--Захват шины и установка скорости шины;
i2cBeginTransmission(id.adr,id.bus); 							//--Начало транзакции;	
id.codeErr=i2cEndTransmission(id.bus);							//--Завершение транзакции с сохранением кода ошибки0;
i2cSetClock(i2cGetSpeed(id.bus),id.bus); i2cClearBusy(id.bus); 	//--Восстановление скорости и освобождение шины;	
return 1;														//--Выход по завершению отправки байтов;
};	//++++++END i2cFindAdr()=======================================================================
uint8_t i2cBegin     (RT_HW_STRUCT_I2C_FUNC &id, uint8_t pinSDA=255, uint8_t pinSCL=255){ //++++Настройка i2c.Master настройкой пинов	
if((pinSDA!=255) && (pinSCL!=255)) {i2cSetPins(pinSDA,pinSCL,id.bus);}  //--Установка номеров пинов (при RT_HW_I2C_BLOCK_USER_PINS=0);
if(!i2cBeginMaster(id.codeErr,id.bus)) {return 0;}; 					//--Настройка шины;
return 1;};	//++++++END i2cBegin()=================================================================




//===================================================================================================
uint8_t i2cInitDevice(uint8_t &codeErr, uint8_t adr,uint8_t bus){			//++++Проверка bus,adr, инициализация i2c: ok=1; err=0->codeErr=60,61,62;
codeErr=0;
               if(!i2cCheckBus(bus))	    {codeErr=60;}	//--Check bus;
if(codeErr==0){if(adr>254) 					{codeErr=61;}}  //--Check adr;
if(codeErr==0){if(!i2cBeginMaster(codeErr,bus)){return 0;}} 				//--Init  i2c;
if(codeErr==0){if(i2cGetStatus(bus)!=1)		{codeErr=62;}}	//--Check i2c as Master;
return  1;	
};	//++++spiInitDevice()----------------------------------------------------------------------------
uint8_t i2cFindAdr   (uint8_t &codeErr, uint8_t adr,uint8_t bus=0, uint32_t sp=0){ //++++Поиск адреса на шине----
if(i2cSetBusy(bus)==0){return 0;}; if(sp!=0){i2cSetClock(sp,bus);}	//--Захват шины и установка скорости шины;
i2cBeginTransmission(adr,bus); 										//--Начало транзакции;	
codeErr=i2cEndTransmission(bus);									//--Завершение транзакции с сохранением кода ошибки0;
i2cSetClock(i2cGetSpeed(bus),bus);	i2cClearBusy(bus); 				//--Восстановление скорости и освобождение шины;	
return 1;															//--Выход по завершению отправки байтов;
};	//++++++END i2cFindAdr()=======================================================================
//=================================================================================================
//							3.ИНИЦИАЛИЗАЦИЯ ШИНЫ
//=================================================================================================
uint8_t i2cBegin     (uint8_t &codeErr, uint8_t bus){	//++++Настройка i2c.Master=====================	
if(!i2cBeginMaster(codeErr,bus)) {return 0;}; 				//--Настройка шины;
return 1;};	//++++++END i2cBegin()=================================================================
uint8_t i2cBegin     (uint8_t &codeErr, uint8_t pinSDA, uint8_t pinSCL, uint8_t bus){ //++++Настройка i2c.Master настройкой пинов	
i2cSetPins(pinSDA,pinSCL,bus);		//--Установка номеров пинов (при RT_HW_I2C_BLOCK_USER_PINS=0);
if(!i2cBeginMaster(codeErr,bus)) {return 0;}; 				//--Настройка шины;
return 1;};	//++++++END i2cBegin()=================================================================
//=================================================================================================
//							4.ОТПРАВКА БАЙТА(БАЙТОВ)
//=================================================================================================
uint8_t i2cSendByte  (uint8_t &codeErr, uint8_t var, uint8_t adr, uint8_t bus=0, uint32_t sp=0){//++++Запись байта	
//-------------------------------------------------------------------------------------------------
if(i2cGetStatus(bus)!=1){return 1;}						//--Выход, если i2c не Master;
if(i2cSetBusy(bus)==0){return 0;}; 						//--Захват шины;
if(sp!=0){i2cSetClock(sp,bus);}	else {i2cSetClock(i2cGetSpeed(bus),bus);}	//--Установка скорости шины;	
#if !defined(RT_HW_CORE_STM32_M_F1)
#endif
//-------------------------------------------------------------------------------------------------
i2cBeginTransmission(adr,bus); 							//--Начало транзакции;	
i2cWrite(var,bus); 										//--Запись байта;
codeErr=i2cEndTransmission(bus);						//--Завершение транзакции с сохранением кода ошибки,ok->codeErr=0;
//-------------------------------------------------------------------------------------------------
i2cSetClock(i2cGetSpeed(bus),bus);						//--Восстановление скорости шины;
#if !defined(RT_HW_CORE_STM32_M_F1)	
#endif
i2cClearBusy(bus); 										//--Освобождение шины;	
//-------------------------------------------------------------------------------------------------
return 1;												//--Выход по завершению отправки байта;
};	//++++END i2cSendByte()========================================================================
uint8_t i2cSendBytes (uint8_t &codeErr, const uint8_t *buff, uint8_t qnt, uint8_t adr, uint8_t bus=0, uint32_t sp=0){//++++Запись qnt байт из arr[];
//-------------------------------------------------------------------------------------------------
if(i2cGetStatus(bus)!=1){return 1;}						//--Выход, если i2c не Master;
if(i2cSetBusy(bus)==0)  {return 0;}; 					//--Захват шины;
if(sp!=0){i2cSetClock(sp,bus);}							//--Установка скорости шины;	
//-------------------------------------------------------------------------------------------------
if(qnt>32) {qnt=32;};									//--Ограничение кол-ва отправляемых байт;
i2cBeginTransmission(adr,bus); 							//--Начало транзакции;	
for(i2cClearIndex(bus); i2cGetIndex(bus)<qnt; i2cIncIndex(bus)){i2cWrite(buff[i2cGetIndex(bus)],bus);} //--Побайтная запись байтов;
codeErr=i2cEndTransmission(bus);				//--Завершение транзакции,ok->codeErr=0;
//-------------------------------------------------------------------------------------------------
i2cSetClock(i2cGetSpeed(bus),bus);	 					//--Восстановление скорости;
i2cClearBusy(bus); 										//--Освобождение шины;
//-------------------------------------------------------------------------------------------------
return 1;												//--Выход по завершению отправки байтов;
};	//++++++END i2cFindAdr()=======================================================================
//=================================================================================================
//							5.ПОЛУЧЕНИЕ БАЙТА(БАЙТОВ)
//=================================================================================================
uint8_t i2cGetBytes  (uint8_t &codeErr, uint8_t *buff, uint8_t qnt, uint8_t adr, uint8_t bus=0, uint32_t sp=0, uint8_t sendStop=1){//++++Чтение байтов в массив buff;
codeErr=0;
//-------------------------------------------------------------------------------------------------
if(i2cGetStatus(bus)!=1){return 1;}						//--Выход, если i2c не Master;
if(i2cSetBusy(bus)==0){return 0;}; 						//--Захват шины;
if(sp!=0){i2cSetClock(sp,bus);}							//--Установка скорости шины;	
//-------------------------------------------------------------------------------------------------
if(qnt>32) {qnt=32;};									//--Ограничение кол-ва отправляемых байт;
i2cSetCntByte(i2cRequestFrom(qnt,adr,bus,sendStop),bus);//--Запрос qnt байтов в буфер; 	
if(i2cGetCntByte(bus)!=qnt){codeErr=5;} 		//--Если кол-во не совпадает, то код ошибки=5;
//-------------------------------------------------------------------------------------------------
if(codeErr==0){								//--Начало ожидания прихода запрошенных байт;
i2cSetTimeOut((micros()+(150*qnt)),bus);	 		 	//--Установка тайм-аут на получение запрошенных данных;
do{i2cSetCntByte(i2cAvailable(bus),bus); 			 	//--Чтение кол-ва байт в буфере;
   if(i2cGetCntByte(bus)>=qnt){break;}		    	 	//--Завершение по заполнению буфера;
   if(micros()>=i2cGetTimeOut(bus)){if((micros()-i2cGetTimeOut(bus))>(150*qnt)){codeErr=6;} break;} //--Тайм-аут прихода байт;
   else                 {if((0xFFFFFFFF-i2cGetTimeOut(bus)+micros())>(150*qnt)){codeErr=6;} break;}}while(1);}
//-------------------------------------------------------------------------------------------------
if(codeErr==0){for(i2cClearIndex(bus); i2cGetIndex(bus)<qnt; i2cIncIndex(bus)){buff[i2cGetIndex(bus)]=i2cRead(bus);}} //--Побайтное чтение байтов;	
//if(i2cGetCodeErr(bus)==0){for(uint8_t i=0; i<qnt; i++){buff[i]=i2cRead(bus);}}
//-------------------------------------------------------------------------------------------------
i2cSetClock(i2cGetSpeed(bus),bus);	 					//--Восстановление скорости;
i2cClearBusy(bus); 										//--Освобождение шины;
//-------------------------------------------------------------------------------------------------
return 1;												//--Выход по завершению получения байтов;
};	//++++END i2cGetBytes()========================================================================
uint8_t i2cGetByte   (uint8_t &codeErr, uint8_t &buff,              uint8_t adr, uint8_t bus=0, uint32_t sp=0, uint8_t sendStop=1){//++++Чтение байта  в переменную buff; 
//-------------------------------------------------------------------------------------------------
if(i2cGetStatus(bus)!=1){return 1;}						//--Выход, если i2c не Master;
if(i2cSetBusy(bus)==0){return 0;}; 						//--Захват шины;
if(sp!=0){i2cSetClock(sp,bus);}							//--Установка скорости шины;	
//-------------------------------------------------------------------------------------------------
i2cSetCntByte(i2cRequestFrom(1,adr,bus,sendStop),bus);	//--Запрос qnt байтов в буфер; 	
if(i2cGetCntByte(bus)!=1){codeErr=5;} 					//--Если кол-во не совпадает, то код ошибки=5;
else			         {codeErr=0;} 					//--Если кол-во совпадает, то очистка
//-------------------------------------------------------------------------------------------------
if(codeErr==0){
i2cSetTimeOut((micros()+150),bus);				//--Установка тайм-аут на получение запрошенных данных;
do{i2cSetCntByte(i2cAvailable(bus),bus);  		//--Проверка наличие байта в буфере;
   if(i2cGetCntByte(bus)>=1){break;}			//--Завершение по заполнению буфера;
   if(micros()>=i2cGetTimeOut(bus)){if((micros()-i2cGetTimeOut(bus))>(150)){codeErr=6;} break;} //--Тайм-аут прихода байт;
   else                 {if((0xFFFFFFFF-i2cGetTimeOut(bus)+micros())>(150)){codeErr=6;} break;}}while(1);}
//-------------------------------------------------------------------------------------------------
if(codeErr==0){buff=i2cRead(bus);}						//--Чтение байта из буфера i2с;
//-------------------------------------------------------------------------------------------------
i2cSetClock(i2cGetSpeed(bus),bus);	 					//--Восстановление скорости;
i2cClearBusy(bus); 										//--Освобождение шины;
//-------------------------------------------------------------------------------------------------
return 1;	  			//--End   device(adr);
};	//++++END i2cGetByte()========================================================================
//=================================================================================================
//							6.ПОЛУЧЕНИЕ БАЙТА(БАЙТОВ) С УКАЗАННОГО РЕГИСТРА
//=================================================================================================
uint8_t i2cReadRegs  (uint8_t &codeErr, uint8_t reg, uint8_t *buff, uint8_t qnt, uint8_t adr, uint8_t bus=0, uint32_t sp=0, uint8_t sendStop=1){//++++Чтение байтов c регистра reg в массив buff;
//-------------------------------------------------------------------------------------------------
if(i2cGetStatus(bus)!=1){return 1;}						//--Выход, если i2c не Master;
if(i2cSetBusy(bus)==0){return 0;}; 						//--Захват шины;
if(sp!=0){i2cSetClock(sp,bus);}							//--Установка скорости шины;	
i2cBeginTransmission(adr,bus); 							//--Начало транзакции;	
i2cWrite(reg,bus); 										//--Запись номера регистра;
codeErr=i2cEndTransmission(bus);						//--Завершение транзакции;
//------------------------------------------------------------------------------------------------
if(codeErr==0){											//--Начало запроса байтов;
if(qnt>32) {qnt=32;}; 									//--Ограничение кол-ва принимаемых байт;
i2cSetCntByte(i2cRequestFrom(qnt,adr,bus,sendStop),bus);//--Запрос qnt байтов в буфер; 									
if(i2cGetCntByte(bus)!=qnt){codeErr=5;}}		//--Если кол-во не совпадает, то код ошибки=5;
//-------------------------------------------------------------------------------------------------
if(codeErr==0){											//--Начало ожидания прихода запрошенных байт;
i2cSetTimeOut((micros()+(150*qnt)),bus);	 		 	//--Установка тайм-аут на получение запрошенных данных;
do{i2cSetCntByte(i2cAvailable(bus),bus); 			 	//--Чтение кол-ва байт в буфере;
   if(i2cGetCntByte(bus)>=qnt){break;}		    	 	//--Завершение по заполнению буфера;
   if(micros()>=i2cGetTimeOut(bus)){if((micros()-i2cGetTimeOut(bus))>(150*qnt)){codeErr=6;} break;} //--Тайм-аут прихода байт;
   else                 {if((0xFFFFFFFF-i2cGetTimeOut(bus)+micros())>(150*qnt)){codeErr=6;} break;}}while(1);}
//-------------------------------------------------------------------------------------------------   
if(codeErr==0){for(i2cClearIndex(bus); i2cGetIndex(bus)<qnt; i2cIncIndex(bus)){buff[i2cGetIndex(bus)]=i2cRead(bus);}} //--Побайтное чтение байтов;	
//-------------------------------------------------------------------------------------------------
i2cSetClock(i2cGetSpeed(bus),bus);	i2cClearBusy(bus); 			//--Восстановление скорости и освобождение шины;				 
return 1;	  			//--End   device(adr);
};	//++++END i2cReadRegs(reg,*buff,qnt...)========================================================
uint8_t i2cReadReg   (uint8_t &codeErr, uint8_t reg, uint8_t &buff,              uint8_t adr, uint8_t bus=0, uint32_t sp=0, uint8_t sendStop=1){//++++Чтение байта  c регистра reg в массив buff;
//------------------------------------------------------------------------------------------------
if(i2cGetStatus(bus)!=1){return 1;}						//--Выход, если i2c не Master;
if(i2cSetBusy(bus)==0){return 0;}; 						//--Захват шины;
if(sp!=0){i2cSetClock(sp,bus);}							//--Установка скорости шины;	
i2cBeginTransmission(adr,bus); 							//--Начало транзакции;	
i2cWrite(reg,bus); 										//--Запись номера регистра;
codeErr=i2cEndTransmission(bus);						//--Завершение транзакции;
//------------------------------------------------------------------------------------------------
if(codeErr==0){											//--Начало запроса байтов;
i2cSetCntByte(i2cRequestFrom(1,adr,bus,sendStop),bus);	//--Запрос qnt байтов в буфер; 									
if(i2cGetCntByte(bus)!=1){codeErr=5;}}		//--Если кол-во не совпадает, то код ошибки=5;
//-------------------------------------------------------------------------------------------------
if(i2cGetCodeErr(bus)==0){								//--Начало ожидания прихода запрошенных байт;
i2cSetTimeOut((micros()+(150)),bus);	 		 		//--Установка тайм-аут на получение запрошенных данных;
do{i2cSetCntByte(i2cAvailable(bus),bus); 			 	//--Чтение кол-ва байт в буфере;
   if(i2cGetCntByte(bus)>=1){break;}		    	 	//--Завершение по заполнению буфера;
   if(micros()>=i2cGetTimeOut(bus)){if((micros()-i2cGetTimeOut(bus))>(150)){codeErr=6;} break;} //--Тайм-аут прихода байт;
   else                 {if((0xFFFFFFFF-i2cGetTimeOut(bus)+micros())>(150)){codeErr=6;} break;}}while(1);}
//-------------------------------------------------------------------------------------------------   
if(codeErr==0){buff=i2cRead(bus);}						//--Чтение байта;
i2cSetClock(i2cGetSpeed(bus),bus);	i2cClearBusy(bus); 	//--Восстановление скорости и освобождение шины;				 
//-------------------------------------------------------------------------------------------------
return 1;												//--Выход по завершению получения байтов;			
};	//++++END i2cReadRegs(reg,*buff,...)===========================================================

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// 								ВНУТРЕННИЕ ФУНКЦИИ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 		
//=================================================================================================
//							7.СЕРВИСНЫЕ УТИЛИТЫ
//=================================================================================================
bool    i2cCheckBus  (uint8_t bus){					//++++7.1.Проверка шины========================
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {return 1;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {return 1;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {return 1;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0)  {return 1;}
#endif	
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {return 1;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
return 0;};
uint8_t i2cCheckBusy (uint8_t bus){					//++++7.2.Проверка занятости шины==============	
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {if(LinkI2C0.busy){return 1;} return 0;} 
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {if(LinkI2C1.busy){return 1;} return 0;} 
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {if(LinkI2C2.busy){return 1;} return 0;} 
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {if(LinkI2CS.busy){return 1;} return 0;} 
#endif	
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {if(LinkI2CS1.busy){return 1;} return 0;} 
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
return 0;};
uint8_t i2cSetBusy   (uint8_t bus){					//++++7.3.Захват шины==========================
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {if(LinkI2C0.busy==0) {LinkI2C0.busy=1;}  if(LinkI2C0.busy) {return 1;}; return 0;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {if(LinkI2C1.busy==0) {LinkI2C1.busy=1;}  if(LinkI2C1.busy) {return 1;}; return 0;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {if(LinkI2C2.busy==0) {LinkI2C2.busy=1;}  if(LinkI2C2.busy) {return 1;}; return 0;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {if(LinkI2CS.busy==0) {LinkI2CS.busy=1;}  if(LinkI2CS.busy) {return 1;}; return 0;}
#endif	
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {if(LinkI2CS1.busy==0){LinkI2CS1.busy=1;} if(LinkI2CS1.busy){return 1;}; return 0;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
return 0;};
void    i2cClearBusy (uint8_t bus){					//++++7.4.Освобождение шины====================
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {LinkI2C0.busy=0; return;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {LinkI2C1.busy=0; return;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {LinkI2C2.busy=0; return;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {LinkI2CS.busy=0; return;}
#endif	
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {LinkI2CS1.busy=0; return;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
return;};
void    i2cSetIndex  (uint8_t var, uint8_t bus){	//++++7.5.Запись индекса=======================
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {LinkI2C0.idx=var; return;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {LinkI2C1.idx=var; return;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {LinkI2C2.idx=var; return;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {LinkI2CS.idx=var; return;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {LinkI2CS1.idx=var; return;}
#endif
buff.vu8=var; //--Для предотвращения warning C++;
buff.bf8=bus; //--Для предотвращения warning C++;
};
uint8_t i2cGetIndex  (			   uint8_t bus){	//++++7.6.Получение индекса====================	
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {return LinkI2C0.idx;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {return LinkI2C1.idx;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {return LinkI2C2.idx;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {return LinkI2CS.idx;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {return LinkI2CS1.idx;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
return 0;};
void    i2cIncIndex  (			   uint8_t bus){	//++++7.7.Инкремент индекса====================	
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {LinkI2C0.idx++; return;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {LinkI2C1.idx++; return;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {LinkI2C2.idx++; return;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {LinkI2CS.idx++; return;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {LinkI2CS1.idx++; return;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
};
void    i2cClearIndex(			   uint8_t bus){	//++++7.8.Очистка индекса======================	
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {LinkI2C0.idx=0; return;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {LinkI2C1.idx=0; return;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {LinkI2C2.idx=0; return;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {LinkI2CS.idx=0; return;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {LinkI2CS1.idx=0; return;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
};

//=================================================================================================
// 							8.Функции управления статусом,кода ошибок,тайм-аутом,счетчика байтов                        
//=================================================================================================
void    i2cSetStatus (uint8_t var,  uint8_t bus){	//++++8.1.Запись статуса===========
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {LinkI2C0.status=var; return;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {LinkI2C1.status=var; return;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {LinkI2C2.status=var; return;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {LinkI2CS.status=var; return;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {LinkI2CS1.status=var; return;}
#endif
buff.vu8=var; //--Для предотвращения warning C++;
buff.bf8=bus; //--Для предотвращения warning C++;
};
uint8_t i2cGetStatus (				uint8_t bus){	//++++8.2.Чтение статуса===========
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {return LinkI2C0.status;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {return LinkI2C1.status;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {return LinkI2C2.status;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {return LinkI2CS.status;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {return LinkI2CS1.status;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
return 0;};
void 	i2cSetCodeErr(uint8_t var,  uint8_t bus){	//++++8.3.Запись кода ошибки=======
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {LinkI2C0.codeErr=var; return;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {LinkI2C1.codeErr=var; return;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {LinkI2C2.codeErr=var; return;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {LinkI2CS.codeErr=var; return;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {LinkI2CS1.codeErr=var; return;}
#endif
buff.vu8=var; //--Для предотвращения warning C++;
buff.bf8=bus; //--Для предотвращения warning C++;
};
uint8_t i2cGetCodeErr(				uint8_t bus){	//++++8.4.Чтение кода ошибки=======
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {return LinkI2C0.codeErr;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {return LinkI2C1.codeErr;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {return LinkI2C2.status;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {return LinkI2CS.status;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {return LinkI2CS1.status;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
return 0;};
void    i2cSetTimeOut(uint32_t vTm, uint8_t bus){	//++++8.5.Запись начального времени	
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {LinkI2C0.timeOut=vTm; return;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {LinkI2C1.timeOut=vTm; return;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {LinkI2C2.timеOut=vTm; return;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {LinkI2CS.timeOut=vTm; return;}
#endif	
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {LinkI2CS1.timeOut=vTm; return;}
#endif
buff.vu32=vTm;//--Для предотвращения warning C++;
buff.bf8=bus; //--Для предотвращения warning C++;
};
uint32_t i2cGetTimeOut(     		uint8_t bus){	//++++8.6.Чтение начального времени
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {return LinkI2C0.timeOut;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {return LinkI2C1.timeOut;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {return LinkI2C2.timeOut;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {return LinkI2CS.timeOut;}
#endif	
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {return LinkI2CS1.timeOut;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
return 0;};
void    i2cSetCntByte(uint8_t  cnt, uint8_t bus){	//++++3.7.Запись кол-ва байт=======
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {LinkI2C0.cntByte=cnt; return;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {LinkI2C1.cntByte=cnt; return;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {LinkI2C2.cntByte=cnt; return;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {LinkI2CS.cntByte=cnt; return;}
#endif	
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {LinkI2CS1.cntByte=cnt; return;}
#endif
buff.vu8=cnt; //--Для предотвращения warning C++;
buff.bf8=bus; //--Для предотвращения warning C++;
};
uint8_t i2cGetCntByte(       		uint8_t bus){	//++++3.8.Чтение кол-ва байт=======
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {return LinkI2C0.cntByte;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {return LinkI2C1.cntByte;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {return LinkI2C2.cntByte;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {return LinkI2CS.cntByte;}
#endif	
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {return LinkI2CS1.cntByte;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
return 0;};



//=================================================================================================
// 					3.Функции управления статусом,кода ошибок,тайм-аутом,счетчика байтов                        
//=================================================================================================
//void 	 i2cSetStatus (uint8_t var,  uint8_t bus);		//++++3.1.Запись статуса===================
//uint8_t  i2cGetStatus (				 uint8_t bus);		//++++3.2.Чтение статуса===================
//void 	 i2cSetCodeErr(uint8_t var,  uint8_t bus);		//++++3.3.Запись кода ошибки===============
//uint8_t  i2cGetCodeErr(				 uint8_t bus);		//++++3.4.Чтение кода ошибки===============
//void     i2cSetTimeOut(uint32_t vTm, uint8_t bus);		//++++3.5.Запись начального времени========		
//uint32_t i2cGetTimeOut(     		 uint8_t bus);		//++++3.6.Чтение начального времени========
//void     i2cSetCntByte(uint8_t  cnt, uint8_t bus);		//++++3.7.Запись кол-ва байт===============
//uint8_t  i2cGetCntByte(       		 uint8_t bus);		//++++3.8.Чтение кол-ва байт===============	
//=================================================================================================
// 					5.Запись/чтение номеров пинов, Slave-адреса, скорости
//=================================================================================================
void	 i2cSetPins(uint8_t pinSDA,  uint8_t pinSCL, uint8_t bus){ //++++Запись в вектор данных номеров пинов SCL,SDA;
#if defined(RT_HW_I2C_BLOCK_USER_PINS)
buff.vu8=pinSDA; //--for block warning C++;
buff.bf8=pinSCL; //--for block warning C++;
buff.vu16=bus; 	 //--for block warning C++;
return;
#endif
i2cSetPinSDA(pinSDA,bus); i2cSetPinSCL(pinSCL,bus); return;
};	//++++END i2cSetPins()=========================================================================
void     i2cSetPinSCL(uint8_t scl,	 uint8_t bus){	//++++Запись номера пина SCL-------------------
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {LinkI2C0.scl=scl;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {LinkI2C1.scl=scl;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {LinkI2C2.scl=scl;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {LinkI2CS.scl=scl;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {LinkI2CS1.scl=scl;}
#endif
buff.vu8=scl; //--Для предотвращения warning C++;
buff.bf8=bus; //--Для предотвращения warning C++;
};
void     i2cSetPinSDA(uint8_t sda,	 uint8_t bus){	//++++Запись номера пина SDA-------------------
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {LinkI2C0.sda=sda;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {LinkI2C1.sda=sda;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {LinkI2C2.sda=sda;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {LinkI2CS.sda=sda;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {LinkI2CS1.sda=sda;}
#endif
buff.vu8=sda; //--Для предотвращения warning C++;
buff.bf8=bus; //--Для предотвращения warning C++;
};
uint8_t  i2cGetPinSCL(            	 uint8_t bus){	//++++Чтение номера пина SCL-------------------
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {return LinkI2C0.scl;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {return LinkI2C1.scl;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {return LinkI2C2.scl;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {return LinkI2CS.scl;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1){return LinkI2CS1.scl;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
return 0;};
uint8_t  i2cGetPinSDA(            	 uint8_t bus){	//++++Чтение номера пина SDA-------------------
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {return LinkI2C0.sda;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {return LinkI2C1.sda;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {return LinkI2C2.sda;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {return LinkI2CS.sda;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {return LinkI2CS1.sda;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
return 0;};
void     i2cSetAdr   (uint8_t adr,	 uint8_t bus){	//++++Запись Slave-адреса(=0:Master; >0:Slave)-
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {LinkI2C0.adr=adr; return;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {LinkI2C1.adr=adr; return;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {LinkI2C2.adr=adr; return;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {LinkI2CS.adr=adr; return;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1) {LinkI2CS1.adr=adr; return;}
#endif
buff.vu8=adr; //--Для предотвращения warning C++;
buff.bf8=bus; //--Для предотвращения warning C++;
};
uint8_t  i2cGetAdr   (				 uint8_t bus){	//++++Чтение Slave-адреса(=0:Master; >0:Slave)-
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {return LinkI2C0.adr;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {return LinkI2C1.adr;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {return LinkI2C2.adr;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {return LinkI2CS.adr;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1){return LinkI2CS1.adr;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
return 0;};
void     i2cSetSpeed (               uint8_t bus, uint32_t sp=0){ //++++Запись скорости(кГц) в блок  данных шины i2c;
if(sp==0){sp=i2cGetSpeed(bus);}
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {LinkI2C0.speed=sp;  return;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {LinkI2C1.speed=sp;  return;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {LinkI2C2.speed=sp;  return;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2C2S)	
if(bus==0) {LinkI2CS.speed=sp;  return;}
#endif
#if defined(RT_HW_PERMIT_I2C2S1)	
if(bus==1) {LinkI2CS1.speed=sp; return;}
#endif
buff.vu32=sp; //--Для предотвращения warning C++;
buff.bf8=bus; //--Для предотвращения warning C++;
};
uint32_t i2cGetSpeed (               uint8_t bus){	//++++Чтение скорости(кГц)из блока данных шины i2c
#if defined(RT_HW_PERMIT_I2C0)	
if(bus==0) {return LinkI2C0.speed;}
#endif
#if defined(RT_HW_PERMIT_I2C1)	
if(bus==1) {return LinkI2C1.speed;}
#endif
#if defined(RT_HW_PERMIT_I2C2)	
if(bus==2) {return LinkI2C2.speed;}
#endif
//-------------------------------------
#if defined(RT_HW_PERMIT_I2CS)	
if(bus==0) {return LinkI2CS.speed;}
#endif
#if defined(RT_HW_PERMIT_I2CS1)	
if(bus==1){return LinkI2CS1.speed;}
#endif
buff.bf8=bus; //--Для предотвращения warning C++;
return 0;};
void     i2cSetClock (uint32_t speed,uint8_t bus);	//++++Установка скорости(аппаратная)-----------
//=================================================================================================
//									6.Функции обмена
// Коды codeErr: 0-ok,1-слишком много данных; 2-Nack на передачу адреса; 3-Nack на передачу данных; 4-другая ошибка i2c; 
// 5-недопустимый контроллер; 6-недопустимая шина; 7-не допустимый адрес(>=254),8-шина не настроена как master,8 другие причины; 
//=================================================================================================
void     i2cBeginTransmission(uint8_t adr, uint8_t bus);//++++Начало аппаратной транзакции=========
uint8_t  i2cEndTransmission  (uint8_t bus);				//++++Завершение аппаратной транзакции=====
uint8_t  i2cFlush            (uint8_t bus);				//++++Flush()==============================
uint8_t  i2cAvailable        (uint8_t bus);				//++++Проверка кол-ва принятых байт в буфере i2c
uint8_t  i2cRead             (uint8_t bus);				//++++Чтение байта из буфера i2c===========
void     i2cWrite(uint8_t var,uint8_t bus);				//++++Запись байта=========================
void     i2cWrite(const uint8_t *arr, uint8_t qnt, uint8_t bus);//++++Запись qnt байт==============
void     i2cWrite(const char    *arr, uint8_t qnt, uint8_t bus);//++++Запись qnt символов==========
uint8_t  i2cRequestFrom(uint8_t qnt,  uint8_t adr, uint8_t bus, uint8_t sendStop=1);//++++Чтение qnt байт в буфер i2c
//=================================================================================================
//						        7.НАСТРОЙКА ШИНЫ i2c.Master
//=================================================================================================
uint8_t  i2cBeginMaster(uint8_t &codeErr, uint8_t bus);
//=================================================================================================
//=================================================================================================
//								Настройка шины как Slave
//=================================================================================================
void    i2cBeginSlave(uint8_t bus);







/*
uint8_t  i2cBeginMaster(uint8_t bus){
if(!i2cCheckBus(bus)) {return 0;}
if((i2cGetStatus(bus)!=0) && (i2cGetStatus(bus)!=1)){i2cSetCodeErr(8,bus); return 0;} //--Выход, если шину уже Master;
i2cSetCodeErr(0,bus);					//--Очистка ошибок;
i2cSetSpeed(bus);						//--Установка параметра частоты шины i2c по умолчанию;
#if defined(RT_HW_PERMIT_I2C0)
if(bus==0){									
#if defined(RT_HW_CORE_AVR) || defined(RT_HW_CORE_SAM) || defined(RT_HW_CORE_SAMD) || defined(RT_HW_CORE_STM32_M)
Wire.begin();							//--Инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;
i2cSetStatus(1,bus); return 1;			//--Установка статуса Master и успешный выход;
#endif		//-------------------------------------------------------------------------------------
#if defined(RT_HW_CORE_ESP8266)
Wire.begin(i2cGetPinSDA(bus),i2cGetPinSCL(bus));//--Установка номеров пинов SDA,SCL и инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
Wire.setClockStretchLimit(260);			//--Ограничение лимита по времени;
i2cSetStatus(1,bus); return 1;			//--Установка статуса Master и успешный выход;
#endif		//-------------------------------------------------------------------------------------
#if defined(RT_HW_CORE_ESP32)			 													
Wire.setPins(i2cGetPinSDA(bus),i2cGetPinSCL(bus));//--Настройка пинов SDA,SCL;
Wire.begin();							//--Инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;		
Wire.setTimeOut(50); 					//--По умолчанию  тайм-аут на транзакцию =50ms;	
i2cSetStatus(1,bus); return 1;			//--Установка статуса Master и успешный выход;
#endif		//-------------------------------------------------------------------------------------
#if defined(RT_HW_CORE_STM32_D) || defined(RT_HW_CORE_RP_PICO) || defined(RT_HW_CORE_SEEED) 
Wire.setSDA(i2cGetPinSDA(bus));			//--Установка номера пина SDA;	
Wire.setSCL(i2cGetPinSCL(bus));			//--Установка номера пина SCL;		
Wire.begin();							//--Инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus); return 1;			//--Установка статуса Master и успешный выход;
#endif		//-------------------------------------------------------------------------------------
#if defined(RT_HW_CORE_ANON)
Wire.begin();							//--Инициализация как Master;
i2cSetStatus(1,bus); return 1;			//--Установка статуса Master и успешный выход;
#endif		//-------------------------------------------------------------------------------------
i2cSetStatus(5,bus); i2cSetCodeErr(5,bus); return 0;}	//--Выход с установкой неопределенного статуса;
#endif	//--END #if defined(RT_HW_PERMIT_I2C0)
//=================================================================================================
#if defined(RT_HW_PERMIT_I2C1)
if(bus==1){									
#if defined(RT_HW_CORE_SAM) || defined(RT_HW_CORE_STM32_M)
Wire1.begin();							//--Инициализация как Master;
Wire1.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus); return 1;			//--Установка статуса Master и успешный выход;
#endif		//-------------------------------------------------------------------------------------
#if defined(RT_HW_CORE_ESP32)			 													
Wire1.setPins(i2cGetPinSDA(bus),i2cGetPinSCL(bus));//--Настройка пинов SDA,SCL;
Wire1.begin();							//--Инициализация как Master;
Wire1.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;			
Wire1.setTimeOut(50); 					//--По умолчанию  тайм-аут на транзакцию =50ms;	
i2cSetStatus(1,bus); return 1;			//--Установка статуса Master и успешный выход;
#endif		//-------------------------------------------------------------------------------------
#if defined(RT_HW_CORE_STM32_D) || defined(RT_HW_CORE_RP_PICO) || defined(RT_HW_CORE_SEEED) 
Wire1.setSDA(i2cGetPinSDA(bus));		//--Установка номера пина SDA;	
Wire1.setSCL(i2cGetPinSCL(bus));		//--Установка номера пина SCL;			
Wire1.begin();							//--Инициализация как Master;
Wire1.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus); return 1;			//--Установка статуса Master и успешный выход;
#endif		//--------------------------------------------------------------------------------------
LinkI2C1.status=5; i2cSetCodeErr(5,bus); return 0;}	//--Выход с установкой неопределенного статуса;
#endif	//--END #if defined(RT_HW_PERMIT_I2C1)
#if defined(RT_HW_PERMIT_I2C2)
if(bus==2){									
#if defined(RT_HW_CORE_STM32_M)
Wire2.begin();							//--Инициализация как Master;
LinkI2C2.status=1; return 0;			//--Установка статуса Master и успешный выход;
#endif		//-------------------------------------------------------------------------------------
#if defined(RT_HW_CORE_STM32_D) || defined(RT_HW_CORE_RP_PICO) || defined(RT_HW_CORE_SEEED) 
Wire2.setSDA(i2cGetPinSDA(bus));		//--Установка номера пина SDA;	
Wire2.setSCL(i2cGetPinSCL(bus));		//--Установка номера пина SCL;			
Wire2.begin();							//--Инициализация как Master;
Wire2.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus); return 1;			//--Установка статуса Master и успешный выход;
#endif		//--------------------------------------------------------------------------------------
LinkI2C2.status=5; i2cSetCodeErr(5,bus); return 0;}	//--Выход с установкой неопределенного статуса;
//====END if(bus==1)---------------------------------------------------

#endif	//--END #if defined(RT_HW_PERMIT_I2C2)
#if defined(RT_HW_PERMIT_I2CS)
if(bus==0){									
#if defined(RT_HW_CORE_STM32_M)
WireS.begin();							//--Инициализация как Master;
WireS.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus); return 1;			//--Установка статуса Master и успешный выход;
#endif		//-------------------------------------------------------------------------------------
LinkI2CS.status=5; i2cSetCodeErr(5,bus); return 0;}	//--Выход с установкой неопределенного статуса;
//====END if(I2S bus=0)----------------------------------------------------------------------------
#endif	//--END #if defined(RT_HW_PERMIT_I2CS)
#if defined(RT_HW_PERMIT_I2CS1)
if(bus==1){									
#if defined(RT_HW_CORE_STM32_M)
WireS1.begin();							//--Инициализация как Master;
WireS1.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus); return 1;			//--Установка статуса Master и успешный выход;
#endif		//-------------------------------------------------------------------------------------
i2cSetStatus(5,bus);i2cSetCodeErr(5,bus); return 0;}	//--Выход с установкой неопределенного статуса;
//====END if(I2S1 bus=1)---------------------------------------------------------------------------
#endif	//--END #if defined(RT_HW_PERMIT_I2CS1)
return 0;}; //--END i2cBegin()
//=================================================================================================

*/
//=================================================================================================
//							3.ИНИЦИАЛИЗАЦИЯ ШИНЫ
//=================================================================================================
/*
uint8_t i2cBegin(uint8_t bus){							      //++++Настройка i2c.Master------	
if(i2cCheckBus (bus)==0){return 0;}	//--Выход, если недопустимый номер шины;
if(i2cGetStatus(bus)==1){return 1;}	//--Выход, если шина настроена как Master;
if(i2cGetStatus(bus)==2){return 0;}	//--Выход, если шина настроена как Slave;
if(i2cGetStatus(bus) >5){return 0;}	//--Выход, если шина настроена с неопределенным статусом;
if(i2cSetBusy(bus)==0)  {return 0;} //--Захват шины. Выход если шина занята;
i2cBeginMaster(bus); 				//--Настройка шины;
i2cClearBusy(bus);					//--Освобождение шины;
if(i2cGetStatus(bus)==1){return 1;}	//--Выход(1),если устройство    Master;
else{i2cSetCodeErr(8,bus);return 0;}//--Выход(0),если устройство не Master;
};	//++++++END uint8_t i2cBegin()=================================================================
uint8_t i2cBegin(uint8_t pinSDA, uint8_t pinSCL,uint8_t bus){   //++++Настройка i2c.Master.[pins]	
if(i2cCheckBus (bus)!=0){return 0;}	//--Выход, если недопустимый номер шины;
if(i2cGetStatus(bus)==1){return 1;}	//--Выход, если шина настроена как Master;
if(i2cGetStatus(bus)==2){return 0;}	//--Выход, если шина настроена как Slave;
if(i2cGetStatus(bus) >5){return 0;}	//--Выход, если шина настроена с неопределенным статусом;
if(i2cSetBusy(bus)==0)  {return 0;} //--Захват шины. Выход если шина занята;
i2cSetPins(pinSDA,pinSCL,bus);		//--Установка номеров пинов (при RT_HW_I2C_BLOCK_USER_PINS=0);
i2cBeginMaster(bus); 				//--Настройка шины;
i2cClearBusy(bus);					//--Освобождение шины;
if(i2cGetStatus(bus)==1){return 1;}	//--Выход(1),если устройство    Master;
else{i2cSetCodeErr(8,bus);return 0;}//--Выход(0),если устройство не Master;
};	//+++END uint8_t i2cBegini(pins)===============================================================
*/

//=================================================================================================
//							2.1.СЕРВИСНЫЕ УТИЛИТЫ
//=================================================================================================
//==================================================================================================
/*
uint8_t i2cInitDevice   (uint8_t adr,uint8_t bus){			//++++Проверка bus,adr, инициализация SPI: ok=1; err=0->codeErr=60,61,62;
i2cSetCodeErr(0,bus);
if(!i2cCheckBus(bus))						{i2cSetCodeErr(60,bus); return 1;}	//--Check bus;
if(adr>254) 								{i2cSetCodeErr(61,bus); return 1;}  //--Check adr;
if(!i2cBegin(bus)){return 0;}; 													//--Init  i2c;
if(i2cGetStatus(bus)!=1)					{spiSetCodeErr(62,bus); return 1;}	//--Check i2c as Master;
return  1;	
};	//++++spiInitDevice()----------------------------------------------------------------------------
uint8_t i2cFindAdr      (uint8_t adr,uint8_t bus=0, uint32_t sp=0){ //++++Поиск адреса на шине----
//if(i2cFuncBegin(adr,bus,sp)==0){return 0;} 		//--Begin device(adr); if(i2c=busy){return 0;}		
//i2cFuncEnd(bus); 				return 1;		//--End   device(adr); 
if(i2cSetBusy(bus)==0){return 0;}; if(sp!=0){i2cSetClock(sp,bus);}	//--Захват шины и установка скорости шины;
i2cBeginTransmission(adr,bus); 										//--Начало транзакции;	
i2cSetCodeErr(i2cEndTransmission(bus),bus);							//--Завершение транзакции с сохранением кода ошибки,ok->codeErr=0;
i2cSetClock(i2cGetSpeed(bus),bus);	i2cClearBusy(bus); 				//--Восстановление скорости и освобождение шины;	
return 1;															//--Выход по завершению отправки байтов;
};	//++++++END i2cFindAdr()=======================================================================
*/
//=================================================================================================
//							4.1.ОТПРАВКА БАЙТА(БАЙТОВ)
//=================================================================================================
/*
uint8_t i2cSendByte (      uint8_t var,                uint8_t adr, uint8_t bus=0, uint32_t sp=0){//++++Запись байта	
//-------------------------------------------------------------------------------------------------
if(i2cSetBusy(bus)==0){return 0;}; 						//--Захват шины;
if(sp!=0){i2cSetClock(sp,bus);}							//--Установка скорости шины;	
//-------------------------------------------------------------------------------------------------
i2cBeginTransmission(adr,bus); 							//--Начало транзакции;	
i2cWrite(var,bus); 										//--Запись байта;
i2cSetCodeErr(i2cEndTransmission(bus),bus);				//--Завершение транзакции с сохранением кода ошибки,ok->codeErr=0;
//-------------------------------------------------------------------------------------------------
i2cSetClock(i2cGetSpeed(bus),bus);						//--Восстановление скорости шины;	
i2cClearBusy(bus); 										//--Освобождение шины;	
//-------------------------------------------------------------------------------------------------
return 1;												//--Выход по завершению отправки байта;
};	//++++END i2cSendByte()========================================================================
uint8_t i2cSendBytes(const uint8_t *buff, uint8_t qnt, uint8_t adr, uint8_t bus=0, uint32_t sp=0){//++++Запись qnt байт из arr[];
//-------------------------------------------------------------------------------------------------
if(i2cSetBusy(bus)==0){return 0;}; 						//--Захват шины;
if(sp!=0){i2cSetClock(sp,bus);}							//--Установка скорости шины;	
//-------------------------------------------------------------------------------------------------
if(qnt>32) {qnt=32;};									//--Ограничение кол-ва отправляемых байт;
i2cBeginTransmission(adr,bus); 							//--Начало транзакции;	
for(i2cClearIndex(bus); i2cGetIndex(bus)<qnt; i2cIncIndex(bus)){i2cWrite(buff[i2cGetIndex(bus)],bus);} //--Побайтная запись байтов;
i2cSetCodeErr(i2cEndTransmission(bus),bus);				//--Завершение транзакции,ok->codeErr=0;
//-------------------------------------------------------------------------------------------------
i2cSetClock(i2cGetSpeed(bus),bus);	 					//--Восстановление скорости;
i2cClearBusy(bus); 										//--Освобождение шины;
//-------------------------------------------------------------------------------------------------
return 1;												//--Выход по завершению отправки байтов;
};	//++++++END i2cFindAdr()=======================================================================
*/
//=================================================================================================
//							5.1.ПОЛУЧЕНИЕ БАЙТА(БАЙТОВ)
//=================================================================================================
/*
uint8_t i2cGetBytes(uint8_t *buff, uint8_t qnt, uint8_t adr, uint8_t bus=0, uint32_t sp=0, uint8_t sendStop=1){//++++Чтение байтов в массив buff;
//-------------------------------------------------------------------------------------------------
if(i2cSetBusy(bus)==0){return 0;}; 						//--Захват шины;
if(sp!=0){i2cSetClock(sp,bus);}							//--Установка скорости шины;	
//-------------------------------------------------------------------------------------------------
if(qnt>32) {qnt=32;};									//--Ограничение кол-ва отправляемых байт;
i2cSetCntByte(i2cRequestFrom(qnt,adr,bus,sendStop),bus);//--Запрос qnt байтов в буфер; 	
if(i2cGetCntByte(bus)!=qnt){i2cSetCodeErr(5,bus);} 		//--Если кол-во не совпадает, то код ошибки=5;
else			           {i2cSetCodeErr(0,bus);} 		//--Если кол-во совпадает, то очистка
//-------------------------------------------------------------------------------------------------
if(i2cGetCodeErr(bus)==0){								//--Начало ожидания прихода запрошенных байт;
i2cSetTimeOut((micros()+(150*qnt)),bus);	 		 	//--Установка тайм-аут на получение запрошенных данных;
do{i2cSetCntByte(i2cAvailable(bus),bus); 			 	//--Чтение кол-ва байт в буфере;
   if(i2cGetCntByte(bus)>=qnt){break;}		    	 	//--Завершение по заполнению буфера;
   if(micros()>=i2cGetTimeOut(bus)){if((micros()-i2cGetTimeOut(bus))>(150*qnt)){i2cSetCodeErr(6,bus);} break;} //--Тайм-аут прихода байт;
   else                 {if((0xFFFFFFFF-i2cGetTimeOut(bus)+micros())>(150*qnt)){i2cSetCodeErr(6,bus);} break;}}while(1);}
//-------------------------------------------------------------------------------------------------
if(i2cGetCodeErr(bus)==0){for(i2cClearIndex(bus); i2cGetIndex(bus)<qnt; i2cIncIndex(bus)){buff[i2cGetIndex(bus)]=i2cRead(bus);}} //--Побайтное чтение байтов;	
//if(i2cGetCodeErr(bus)==0){for(uint8_t i=0; i<qnt; i++){buff[i]=i2cRead(bus);}}
//-------------------------------------------------------------------------------------------------
i2cSetClock(i2cGetSpeed(bus),bus);	 					//--Восстановление скорости;
i2cClearBusy(bus); 										//--Освобождение шины;
//-------------------------------------------------------------------------------------------------
return 1;												//--Выход по завершению получения байтов;
};	//++++END i2cGetBytes()========================================================================
uint8_t i2cGetByte (uint8_t &buff,              uint8_t adr, uint8_t bus=0, uint32_t sp=0, uint8_t sendStop=1){//++++Чтение байта  в переменную buff; 
//-------------------------------------------------------------------------------------------------
if(i2cSetBusy(bus)==0){return 0;}; 						//--Захват шины;
if(sp!=0){i2cSetClock(sp,bus);}							//--Установка скорости шины;	
//-------------------------------------------------------------------------------------------------
i2cSetCntByte(i2cRequestFrom(1,adr,bus,sendStop),bus);	//--Запрос qnt байтов в буфер; 	
if(i2cGetCntByte(bus)!=1){i2cSetCodeErr(5,bus);} 		//--Если кол-во не совпадает, то код ошибки=5;
else			         {i2cSetCodeErr(0,bus);} 		//--Если кол-во совпадает, то очистка
//-------------------------------------------------------------------------------------------------
if(i2cGetCodeErr(bus)==0){
i2cSetTimeOut((micros()+150),bus);						//--Установка тайм-аут на получение запрошенных данных;
do{i2cSetCntByte(i2cAvailable(bus),bus);  			 	//--Проверка наличие байта в буфере;
   if(i2cGetCntByte(bus)>=1){break;}					//--Завершение по заполнению буфера;
   if(micros()>=i2cGetTimeOut(bus)){if((micros()-i2cGetTimeOut(bus))>(150)){i2cSetCodeErr(6,bus);} break;} //--Тайм-аут прихода байт;
   else                 {if((0xFFFFFFFF-i2cGetTimeOut(bus)+micros())>(150)){i2cSetCodeErr(6,bus);} break;}}while(1);}
//-------------------------------------------------------------------------------------------------
if(!i2cGetCodeErr(bus)){buff=i2cRead(bus);}				//--Чтение байта из буфера i2с;
//-------------------------------------------------------------------------------------------------
i2cSetClock(i2cGetSpeed(bus),bus);	 					//--Восстановление скорости;
i2cClearBusy(bus); 										//--Освобождение шины;
//-------------------------------------------------------------------------------------------------
return 1;	  			//--End   device(adr);
};	//++++END i2cGetByte()========================================================================
*/
//=================================================================================================
//							6.1.ПОЛУЧЕНИЕ БАЙТА(БАЙТОВ) С УКАЗАННОГО РЕГИСТРА
//=================================================================================================
/*
uint8_t i2cReadRegs(uint8_t reg, uint8_t *buff, uint8_t qnt, uint8_t adr, uint8_t bus=0, uint32_t sp=0, uint8_t sendStop=1){//++++Чтение байтов c регистра reg в массив buff;
//-------------------------------------------------------------------------------------------------
if(i2cSetBusy(bus)==0){return 0;}; 						//--Захват шины;
if(sp!=0){i2cSetClock(sp,bus);}							//--Установка скорости шины;	
i2cBeginTransmission(adr,bus); 							//--Начало транзакции;	
i2cWrite(reg,bus); 										//--Запись номера регистра;
i2cSetCodeErr(i2cEndTransmission(bus),bus);				//--Завершение транзакции;
//------------------------------------------------------------------------------------------------
if(i2cGetCodeErr(bus)==0){								//--Начало запроса байтов;
if(qnt>32) {qnt=32;}; 									//--Ограничение кол-ва принимаемых байт;
i2cSetCntByte(i2cRequestFrom(qnt,adr,bus,sendStop),bus);//--Запрос qnt байтов в буфер; 									
if(i2cGetCntByte(bus)!=qnt){i2cSetCodeErr(5,bus);}}		//--Если кол-во не совпадает, то код ошибки=5;
//-------------------------------------------------------------------------------------------------
if(i2cGetCodeErr(bus)==0){								//--Начало ожидания прихода запрошенных байт;
i2cSetTimeOut((micros()+(150*qnt)),bus);	 		 	//--Установка тайм-аут на получение запрошенных данных;
do{i2cSetCntByte(i2cAvailable(bus),bus); 			 	//--Чтение кол-ва байт в буфере;
   if(i2cGetCntByte(bus)>=qnt){break;}		    	 	//--Завершение по заполнению буфера;
   if(micros()>=i2cGetTimeOut(bus)){if((micros()-i2cGetTimeOut(bus))>(150*qnt)){i2cSetCodeErr(6,bus);} break;} //--Тайм-аут прихода байт;
   else                 {if((0xFFFFFFFF-i2cGetTimeOut(bus)+micros())>(150*qnt)){i2cSetCodeErr(6,bus);} break;}}while(1);}
//-------------------------------------------------------------------------------------------------   
if(i2cGetCodeErr(bus)==0){for(i2cClearIndex(bus); i2cGetIndex(bus)<qnt; i2cIncIndex(bus)){buff[i2cGetIndex(bus)]=i2cRead(bus);}} //--Побайтное чтение байтов;	
//if(i2cGetCodeErr(bus)==0){for(uint8_t i=0; i<qnt; i++){buff[i]=i2cRead(bus);}}
//-------------------------------------------------------------------------------------------------
i2cSetClock(i2cGetSpeed(bus),bus);	i2cClearBusy(bus); 			//--Восстановление скорости и освобождение шины;				 
return 1;	  			//--End   device(adr);
};	//++++END i2cReadRegs(reg,*buff,qnt...)========================================================
uint8_t i2cReadReg (uint8_t reg, uint8_t &buff,              uint8_t adr, uint8_t bus=0, uint32_t sp=0, uint8_t sendStop=1){//++++Чтение байта  c регистра reg в массив buff;
//------------------------------------------------------------------------------------------------
if(i2cSetBusy(bus)==0){return 0;}; 						//--Захват шины;
if(sp!=0){i2cSetClock(sp,bus);}							//--Установка скорости шины;	
i2cBeginTransmission(adr,bus); 							//--Начало транзакции;	
i2cWrite(reg,bus); 										//--Запись номера регистра;
i2cSetCodeErr(i2cEndTransmission(bus),bus);				//--Завершение транзакции;
//------------------------------------------------------------------------------------------------
if(i2cGetCodeErr(bus)==0){								//--Начало запроса байтов;
i2cSetCntByte(i2cRequestFrom(1,adr,bus,sendStop),bus);	//--Запрос qnt байтов в буфер; 									
if(i2cGetCntByte(bus)!=1){i2cSetCodeErr(5,bus);}}		//--Если кол-во не совпадает, то код ошибки=5;
//-------------------------------------------------------------------------------------------------
if(i2cGetCodeErr(bus)==0){								//--Начало ожидания прихода запрошенных байт;
i2cSetTimeOut((micros()+(150)),bus);	 		 		//--Установка тайм-аут на получение запрошенных данных;
do{i2cSetCntByte(i2cAvailable(bus),bus); 			 	//--Чтение кол-ва байт в буфере;
   if(i2cGetCntByte(bus)>=1){break;}		    	 	//--Завершение по заполнению буфера;
   if(micros()>=i2cGetTimeOut(bus)){if((micros()-i2cGetTimeOut(bus))>(150)){i2cSetCodeErr(6,bus);} break;} //--Тайм-аут прихода байт;
   else                 {if((0xFFFFFFFF-i2cGetTimeOut(bus)+micros())>(150)){i2cSetCodeErr(6,bus);} break;}}while(1);}
//-------------------------------------------------------------------------------------------------   
if(i2cGetCodeErr(bus)==0){buff=i2cRead(bus);}			//--Чтение байта;
i2cSetClock(i2cGetSpeed(bus),bus);	i2cClearBusy(bus); 	//--Восстановление скорости и освобождение шины;				 
//-------------------------------------------------------------------------------------------------
return 1;												//--Выход по завершению получения байтов;			
};	//++++END i2cReadRegs(reg,*buff,...)===========================================================
*/



/*
uint8_t i2cGetBytes(                      uint8_t qnt, uint8_t adr, uint8_t bus=0, uint32_t sp=0, uint8_t sendStop=1){//++++Чтение байтов в буфер i2c;
//-------------------------------------------------------------------------------------------------
if(i2cSetBusy(bus)==0){return 0;}; 						//--Захват шины;
if(sp!=0){i2cSetClock(sp,bus);}							//--Установка скорости шины;	
//-------------------------------------------------------------------------------------------------
if(qnt>32) {qnt=32;};									//-Ограничение кол-ва запрашиваемых байт;	 
i2cSetCntByte(i2cRequestFrom(qnt,adr,bus,sendStop),bus);//--Запрос qnt байтов в буфер; 	
if(i2cGetCntByte(bus)!=qnt){i2cSetCodeErr(5,bus);} 		//--Если кол-во не совпадает, то код ошибки=5;
else			           {i2cSetCodeErr(0,bus);} 		//--Если кол-во совпадает, то очистка
//-------------------------------------------------------------------------------------------------
if(i2cGetCodeErr(bus)==0){								//--Начало ожидания прихода запрошенных байт;
i2cSetTimeOut((micros()+(150*qnt)),bus);	 		 	//--Установка тайм-аут на получение запрошенных данных;
do{i2cSetCntByte(i2cAvailable(bus),bus); 			 	//--Чтение кол-ва байт в буфере;
   if(i2cGetCntByte(bus)>=qnt){break;}		    	 	//--Завершение по заполнению буфера;
   if(micros()>=i2cGetTimeOut(bus)){if((micros()-i2cGetTimeOut(bus))>(150*qnt)){i2cSetCodeErr(6,bus);} break;} //--Тайм-аут прихода байт;
   else                 {if((0xFFFFFFFF-i2cGetTimeOut(bus)+micros())>(150*qnt)){i2cSetCodeErr(6,bus);} break;}}while(1);}
//-------------------------------------------------------------------------------------------------
i2cSetClock(i2cGetSpeed(bus),bus);	 					//--Восстановление скорости;
i2cClearBusy(bus); 										//--Освобождение шины;
//-------------------------------------------------------------------------------------------------
return 1;												//--Выход по завершению отправки байтов;
};	//++++END i2cGetBytes()========================================================================
uint8_t i2cGetByte (                                   uint8_t adr, uint8_t bus=0, uint32_t sp=0, uint8_t sendStop=1){//++++Чтение байта  в буфер i2c;
return i2cGetBytes(1,adr, bus,sp,1);
};	//++++END i2cGetByte===========================================================================
*/
/*
uint8_t i2cReadRegs(uint8_t reg,               uint8_t qnt, uint8_t adr, uint8_t bus=0, uint32_t sp=0){                    //++++Чтение qnt байт c регистра=reg в буфер i2c;
if(!i2cSendByte(reg,adr,bus,sp)){i2cSetCodeErr(10,bus); return 0;}				
i2cSetCodeErr(i2cGetBytes(qnt,adr,bus,sp,1),bus); 	//--Чтение qnt байт с установкой кода ошибки; 
return i2cGetCodeErr(bus);							//--Выход, возвращает код ошибки (ok=0);
};
*/


//uint8_t  i2cFuncBegin (uint8_t adr,uint8_t bus,uint32_t sp=0){//+++Begin(1) device(adr)-------------
//if(i2cSetBusy(bus)==0){return 0;}; 	//--Захват шины. Выход если шина занята;	 
//if(sp!=0){i2cSetClock(sp,bus);}		//--Установка скорости шины;
//i2cBeginTransmission(adr,bus); 			//--Начало транзакции;	
//return 1;};	//++++END i2cFuncBegin()======================================================================= 
//void     i2cFuncEnd   (uint8_t bus){                      	//++++End device(adr)------------------
//i2cSetCodeErr(i2cEndTransmission(bus),bus);	//--Завершение транзакции,ok->codeErr=0;	
//i2cSetClock(i2cGetSpeed(bus),bus);			//--Восстановление скорости по умолчанию;
//i2cClearBusy(bus); 							//--Освобождение шины;	
//};	//+++END i2cFuncEnd()==========================================================================