 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			СЕНСОР MAX6675
/*
 Сенсор MAX6675 410 / 5000
 MAX6675 выполняет компенсацию холодного спая и оцифровывает сигнал термопары типа К.
 Данные выводятся с 12-битным разрешением, совместимым с SPI, используется только линия чтения miso.
 Точность измерения до 0,25 ° C, позволяет измерять температуру в диапазоне 0 + 1023.75°C.
 Точность с 12-битным разрешением обеспечивается в диапазоне от 0 ° C до + 700°C, 
 за границами этого дипазона точность в последнем разряде не гарантируется.
 
 MAX6675:
	1.это сложная термопара, преобразованная в цифровой преобразователь со встроенным 12-битным АЦП;
	2.имеет холодный спай, компенсацию измерения и коррекции, цифровой контроллер.
	3.имеет интерфейс, совместимый с SPI, и связанную с ним логику управления.
	4.разработан для работы с внешним микроконтроллером).
 
 Преобразование температуры включает в себя оборудование формирования сигнала для
 преобразоваybz сигнал термопары в напряжение совместимое со встроенным АЦП. 
 Т+(хромовый вывод термопары) и Т-(алюминевый вывод теромопары) подключаются к внутренней схеме MAX6675.
 Перед преобразованием термоэлектрических напряжений в эквивалентные значения температуры, необходимо компенсировать
 разницу  между стороной холодного спая термопары(температура окружающей среды чипа MAX6675) и виртуальным эталон 0°C.
 
 Для термопары типа K изменение напряжения равно 41 мкВ/°C, 
 что приблизительно соответствует характеристике термопары со следующей линейным уравнением:
		VOUT = (41 мкВ /°C) ✕ (TR - TAMB), 
 где TAMB - температура окружающей среды чипа MAX6675.
 Чип MAX6675 измеряет температуру окружающей среды на встроенноим диоде.
 Наиболее точные показания получается достигаются когда холодный спай термопары и корпус чипа MAX6675 имеют
 одинаковую температуру.
 К оцифровке АЦП чип добавляет измерение температуры на встроенном диоде (холодный спай) и 
 считывает 12-разрядный результат на вывод SO.
 Следует избегать размещения тепловыделяющих устройств или компонентов рядом с чипом, 
 поскольку это может привести к ошибкам из-за температуры холодного спая.
 Последовательность всех 0, означает, что температура спая равна 0°C, последовательность всех единиц соответствует 1023.75°C.
 Библиотека обеспечивает отфильтровывание ошибочных измерений (от 0 до 7, по умолчанию =3); 
 
 Коды ошибок:
 0 - нет ошибок;
 60 - недопустимый номер шины    (для SPI);
 61 - недопустимыЙ номер пина cs (для SPI); 
 62 - SPI не настроен            (для SPI);
 65 - недопустимый номер пина sck(для SPN);
 67 - недопустимый номер пина so (для SPN); 
 68 - недопустимый номер пина cs (для SPN); 
 71 - сенсор не подключен;
 72 - ошибочное измерение;
 73 - обрыв термопары;
*/ 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			СЕНСОР MAX6675
//  ecoins (ecoins@mail.ru) 
//  22.10.2022: Begin version
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-------------------------------------------------------------------------------------------------
#ifndef RT_HW_MAX6675_h
#define RT_HW_MAX6675_h
#define RT_HW_MAX6675_VER 			10		//--Номер версии библиотеки;
#include "RT_HW_BASE.h"						//--Подключение базовой библиотеки;
//-------------------------------------------------------------------------------------------------
#define RT_HW_MAX6675_RESOLUTION    0.25   	//--Разрешение измерения в градусах С;
#define RT_HW_MAX6675_DELAY_ERR    	50   	//--Задержка при однократной ошибке;
#define RT_HW_MAX6675_DELAY_ERR_MIN	250		//--Минимальный период при ошибках;
#define RT_HW_MAX6675_DELAY_ERR_MAX	2500	//--Максимальный период при ошибках;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							Структура для MAX6675
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
struct RT_HW_STRUCT_MAX6675_ID{ 
//-----------------Основные параметры управления---------------------------------------------------
uint8_t  custom=0;				//--Состояние настройки ID;  
//----------------Перенастраиваемые параметры пользователем----------------------------------------
uint8_t  cs=255;				//--Номер пина cs  (CS);
uint8_t  bus=0;					//--Номер шины;
RT_HW_STRUCT_SPN spn;
uint32_t speed=RT_HW_SPI_SPEED;	//--Скорость шины SSPI;
uint8_t  time= RT_HW_SPN_DELAY;	//--Скорость (0-10);
//----------------Основные переменные управления измерением-----------------------------------------
uint8_t  run=0,cmdRun=0;       		//--Флаги для запуска измерений;
uint8_t  ddr=0;					//--Этапы основной программы управления устройствами;
uint8_t  step;					//--Управление этапами управления в функциях измерения;
//----------------Выходные параметры-----------------------------------------------------------------
uint8_t  fresh=1;				//--Флаг "свежих" данных;
uint8_t  ok=0;					//--Флаг "Устройство в норме;
uint8_t  codeErr, err;			//--Код ошибки при обмене;
//----------------Дополнительные переменные управления измерением-----------------------------------
uint8_t  initDevice=0, action;	//--Рабочие флаги;
uint8_t  cntErr;				//--Счетчик повторных ошибок;
uint8_t  cntFind;				//--Счетчик поиска устройств;
//----------------Параметры для функци timeDelay()[и также используются функцией i2cFindAdr()]------
uint32_t startDelay;		    //--Рабочий регистр;
uint8_t  workDelay=0;			//--Рабочий регистр; 
uint8_t  pointDelay;			//--Точка возврата из функции;
uint16_t sizeDelay;				//--Задержка для функции();
//--------------------------------------------------------------------------------------------------
char     mi='S';				//--Тип интерфейса: S-SPI, s-SSPI;
uint16_t rawData;				//--Регистр для чтения данных;
float 	 vTemp;					//--Измеренная температура;
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                     КЛАСС УПРАВЛЕНИЯ MAX6675
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class RT_HW_MAX6675{
public:
//=================================================================================================
void setParamSPI(RT_HW_STRUCT_MAX6675_ID &z, uint8_t vCS,  uint8_t vBus, uint32_t vSpeed=0){ 				//++++НАСТРОЙКА параметров SPI
if(z.custom==0){	//++++Однократная настройка параметров---------------------------------------------
z.mi='S'; z.cs=vCS;  z.bus=vBus; z.speed=vSpeed; z.custom=1;}	//--Настройка cs,номера шины,скорости шины;
};	//+++++END setParam()==========================================================================
void setParamSPN(RT_HW_STRUCT_MAX6675_ID &z, uint8_t vSCK, uint8_t vMISO, uint8_t vMOSI, uint8_t vCS, uint8_t vSpeed=0){	//++++НАСТРОЙКА параметров SPN4
if(z.custom==0){z.mi='N'; z.spn.idSCK.pin=vSCK; z.spn.idMISO.pin=vMISO; z.spn.idMOSI.pin=vMOSI; z.spn.idCS.pin=vCS; z.speed=vSpeed; z.custom=1;}
};	//+++++END setParam()==========================================================================
//=================================================================================================
void direct(RT_HW_STRUCT_MAX6675_ID &z){	//++++УПРАВЛЕНИЕ УСТРОЙСТВОМ===========================
//-------------------------------------------------------------------------------------------------
if(z.run){z.cmdRun=1;} 
//-------------------------------------------------------------------------------------------------
if(z.ddr==0) {	//++++Начальная настройка----------------------------------------------------------	
z.err=z.codeErr=z.cntErr=z.cntFind=z.initDevice=0; 			//--Инициализация рабочих параметров;
z.fresh=1; z.ok=0; z.vTemp=0.1;								//--Инициализация выходных параметров; 
z.workDelay=0;z.ddr=90; z.sizeDelay=50; z.pointDelay=10;	//--Переход на продолжение через 200ms;
}	//---------------------------------------------------------------------------------------------
if(z.ddr==10){	//++++Ожидание команды cmdRun(запуск измерения)------------------------------------
if(!z.cmdRun){return;} z.cmdRun=0; z.step=0;				//--Ожидание команды run;
if(!z.initDevice){z.ddr=20;} else{z.ddr=30;}				//--Переход инициализацию или измерение;	
}	//---------------------------------------------------------------------------------------------
if(z.ddr==20){	//++++Инициализация устройства-----------------------------------------------------
init(z); 	  												//--Инициализация устройства; 		  
if(z.err)   {z.ddr=90; z.sizeDelay=2000; z.pointDelay=0;}	//--При ошибке переход ddr=0 через задержку 2000ms;
if(z.action){return;}   									//--Ожидание завершения измерения;
z.initDevice=1; z.step=0; z.ddr=30;							//--Переход на измерение;
}	//---------------------------------------------------------------------------------------------	
if(z.ddr==30){	//++++Измерение--------------------------------------------------------------------
meter(z); if(z.err){z.ddr=0;  return;} 	//==Пошаговое измерение и переход по фатальной ошибке;
if(z.action)      			 {return;} 	//--Ожидание завершения измерения;
z.ddr=10;                     return;	//--Переход на основной цикл;		
}	//---------------------------------------------------------------------------------------------
if(z.ddr==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(z.startDelay,z.workDelay,z.sizeDelay)){return;} z.ddr=z.pointDelay;
}	//---------------------------------------------------------------------------------------------
};	//+++++END direct()============================================================================

void init  (RT_HW_STRUCT_MAX6675_ID &z){	//++++ИНИЦИАЛИЗАЦИЯ устройства=========================
if(z.step==0) {	//++++Подготовка внутренних параметров функции-------------------------------------           
z.workDelay=z.err=z.codeErr=0; z.action=1; 					//--Очистка параметров;
  z.cntErr=0; z.step=90; z.sizeDelay=50; z.pointDelay=10;	//--Переход на продолжение через задержку 50мс;     
}	//---------------------------------------------------------------------------------------------
if(z.step==10){	//++++Выбор метода инициализация шины----------------------------------------------
     if(z.mi=='S'){z.step=20;}								//--Интерфейс SPI;
else if(z.mi=='N'){z.step=30;}								//--Интерфейс SPN;
else              {z.err=1; z.codeErr=60;	return;}		//--Неопределенный интерфейс;
}	//---------------------------------------------------------------------------------------------
if(z.step==20){	//++++Инициализация шины SPI-------------------------------------------------------
if(!RT_HW_Base.SPIinitDevice(z.codeErr,z.cs,z.bus)){return;}	//--Инициализация SPI;
if(z.codeErr){z.err=1;            	return;}				//--Выход по ошибке инициализации шины; 
else         {z.action=0;         	return;}				//--Безусловный переход на продолжение;}	
}	//---------------------------------------------------------------------------------------------
if(z.step==30){	//++++Инициализация шины SPN-------------------------------------------------------
RT_HW_Base.SPNinitDevice(z.spn,z.codeErr);
if(z.spn.status==1){z.codeErr=0; z.action=0;} else {z.codeErr=70; z.err=1;}
return;	//--Инициализация SPN; 
}	//---------------------------------------------------------------------------------------------
if(z.step==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(z.startDelay,z.workDelay,z.sizeDelay)){return;} z.step=z.pointDelay;
}	//---------------------------------------------------------------------------------------------
};	//====END init()===============================================================================

void meter(RT_HW_STRUCT_MAX6675_ID &z){		//++++Измерение========================================
if(z.step==0) {	//++++Подготовка внутренних параметров функции-------------------------------------           
z.workDelay=z.err=z.codeErr=0; z.action=1; 					//--Очистка параметров;
  z.cntErr=0; z.step=10;								//--Переход на продолжение;     
}	//---------------------------------------------------------------------------------------------
if(z.step==10){	//++++Измерение--------------------------------------------------------------------
if(z.mi=='S'){RT_HW_Base.SPIread16(1,0,z.rawData,z.cs,z.bus,z.speed);}
if(z.mi=='N'){RT_HW_Base.SPNread16(z.spn,0,z.rawData);}
	 if(z.rawData==0xffff)   {z.codeErr=71;}			//--Датчик не подключен;
else if(z.rawData==0) 	     {z.codeErr=72;}			//--Ошибочное измерение;
else if(bitRead(z.rawData,2)){z.codeErr=73;}			//--Обрыв термопары;														 
else   					     {z.codeErr= 0;} 			//--Измерение без ошибок;
if(z.codeErr){z.step=85;   z.pointDelay=10;}			//--Переход на обработку ошибки;
else         {z.vTemp=(float)(z.rawData>>3)*RT_HW_MAX6675_RESOLUTION; z.action=0; z.cntErr=0; z.ok=z.fresh=1;}
}	//---------------------------------------------------------------------------------------------
if(z.step==85){	//++++Динамическая задержка с выходом при многократной ошибке----------------------
if(++z.cntErr>3){z.cntErr=3; z.codeErr=84; z.err=1;} z.sizeDelay=z.cntErr*z.cntErr*RT_HW_DEVICE_DELAY; z.step=90;
}	//---------------------------------------------------------------------------------------------
if(z.step==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(z.startDelay,z.workDelay,z.sizeDelay)){return;} z.step=z.pointDelay;
}	//---------------------------------------------------------------------------------------------
};	//++++END meter()==============================================================================
};	//++++END Class MAX6675========================================================================
//=================================================================================================
extern RT_HW_MAX6675 	RT_HW_max6675; 						//--Создание объекта RT_HW_MAX6675;
//=================================================================================================
#endif
//=================================================================================================