//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			СЕНСОР BH1750 (измерение освещенности)
//  ecoins (ecoins@mail.ru) 
//  22.10.2022: Begin version
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-------------------------------------------------------------------------------------------------
#ifndef RT_HW_BH1750_h
#define RT_HW_BH1750_h
#define RT_HW_BH1750_VER 		10		//--Номер версии библиотеки;
#include "RT_HW_BASE.h"					//--Подключение базовой библиотеки;
//=================================================================================================
//                            КЛАСС СЕНСОРА BH1750
//=================================================================================================
class RT_HW_BH1750{
public:
//-----------------Основные параметры управления---------------------------------------------------
uint8_t  custom=0;				//--Состояние настройки ID;  
//----------------Перенастраиваемые параметры пользователем----------------------------------------
uint8_t  bus=0;					//--Номер шины;
uint8_t  adr=0x23;				//--Адрес i2c по умолчанию;
uint32_t speed=RT_HW_I2C_SPEED;		//--Скорость шины;
//----------------Основные переменные управления измерением-----------------------------------------
uint8_t  run=0, cmdRun=0;       //--Флаг для внешнего запуска измерений;
uint8_t  ddr=0;					//--Этапы основной программы управления устройствами;
uint8_t  step;					//--Управление этапами управления в функциях измерения;
//----------------Выходные параметры-----------------------------------------------------------------
uint8_t  fresh=1;				//--Флаг "свежих" данных;
uint8_t  ok=0;					//--Флаг "Устройство в норме;
uint8_t  codeErr, err;			//--Код ошибки при обмене;
//----------------Дополнительные переменные управления измерением-----------------------------------
uint8_t  initDevice, action;	//--Рабочие флаги;
uint8_t  cntErr;				//--Счетчик повторных ошибок;
uint8_t  cntFind;				//--Счетчик поиска устройств;
//----------------Параметры для функци timeDelay()[и также используются функцией i2cFindAdr()]------
uint32_t startDelay;		    //--Рабочий регистр;
uint8_t  workDelay=0;			//--Рабочий регистр; 
uint8_t  pointDelay;			//--Точка возврата из функции;
uint16_t sizeDelay;				//--Задержка для функции();
//----------------Буферные рабочие переменные (поддержка релевантности)------------------------------
uint8_t  bf[2];              	//--Буфер для временного хранения данных;
uint16_t vLux;					//--Измеренные значения;
//=================================================================================================
void setParam(uint8_t vAdr=0x23, uint8_t vBus=0, uint32_t vSpeed=0){ //++++НАСТРОЙКА параметров
if(custom==0){	//++++Однократная настройка параметров---------------------------------------------
adr=vAdr;  bus=vBus; speed=vSpeed; custom=1;}	//--Настройка адреса устройства,номера шины,скорости шины;
};	//+++++END setParam()==========================================================================

void direct(){									  //++++УПРАВЛЕНИЕ измерением====================== 
//-------------------------------------------------------------------------------------------------
if(run){cmdRun=1;} 
//-------------------------------------------------------------------------------------------------
if(ddr==0) {	//++++Начальная настройка----------------------------------------------------------	
workDelay=err=codeErr=cntErr=cntFind=initDevice=0; 	//--Инициализация рабочих параметров;
fresh=1; ok=0; vLux=200;							//--Инициализация выходных параметров; 
            ddr=90; sizeDelay=200; pointDelay=10;	//--Переход на продолжение через 200ms;
}	//----------------------------------------------------------------------------------------------
if(ddr==10){	//++++Ожидание команды cmdRun(запуск измерения)------------------------------------
if(!cmdRun){return;} step=0;			//-- 
if(initDevice) {ddr=30;} else {ddr=20;}	//--Переход на измерение или на инициализацию;	
}	//---------------------------------------------------------------------------------------------
if(ddr==20){	//++++Инициализация устройства-----------------------------------------------------
init(); 	  												//--Инициализация устройства; 		  
if(err)   {ddr=90; sizeDelay=2000; pointDelay=0;}			//--При ошибке переход ddr=0 через задержку 2000ms;
if(action){return;}   										//--Ожидание завершения измерения;
initDevice=1; step=0; ddr=30;								//--Переход на измерение;
}	//---------------------------------------------------------------------------------------------
if(ddr==30){	//++++Измерение и получение результатов--------------------------------------------
meter();	 	  											//--Пошаговое измерение; 
if(err)   {ddr=90; sizeDelay=500; pointDelay=0;} 			//--Переход на начало через задержку 1000ms;														
if(action){return;}  										//--Ожидание завершения измерения;
cmdRun=0;  ddr=90; sizeDelay=500; pointDelay=10; 			//--Переход на ожидание run через задержку 500ms;;
}	//---------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
if(ddr==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} else {ddr=pointDelay;  return;}
}	//---------------------------------------------------------------------------------------------
};	//+++++END direct()============================================================================

void init(){									  //++++ИНИЦИАЛИЗАЦИЯ устройства===================
if(step==0) {	//++++Подготовка внутренних параметров функции-------------------------------------           
workDelay=err=codeErr=0; action=1; 					//--Очистка параметров;
  cntErr=0; step=90; sizeDelay=50; pointDelay=10;	//--Переход на продолжение через задержку 50мс;     
}	//---------------------------------------------------------------------------------------------
if(step==10){	//++++Инициализация шины-----------------------------------------------------------
if(!RT_HW_Base.i2cInitDevice(codeErr,adr,bus)){return;} //--Инициализация шины;
if(codeErr){err=1; codeErr=80;         return;}			//--Выход по ошибке инициализации шины; 
else       {step=15;      cntFind=0; cntErr=0;}			//--Безусловный переход на продолжение;
}	//---------------------------------------------------------------------------------------------
if(step==15){	//++++Поиск устройства на шине i2c-------------------------------------------------
if(!RT_HW_Base.i2cFindAdr(codeErr,adr,bus,speed)){return;};	  //--Поиск устройства на на шине i2c;     
//codeErr=RT_HW_Base.i2cGetCodeErr(bus);					  //--Получение кода ошибки;  	
if(codeErr){step=85;                pointDelay=15; cntFind=0;}//--При ошибке переход на динамическую задержку; 
else       {step=90; sizeDelay=50;  pointDelay=15; cntErr=0;  //--Переход на повторение поиска с задержкой 50ms;
            if(++cntFind>=3){step=20;  return;}              }//--Переход на продолжение в следующем цикле после 3-кратного обнаружения;
}	//--------------------------------------------------------------------------------------------
if(step==20){	//++++Включение питания]----------------------------------------------------------	
if(!RT_HW_Base.i2cSendByte(codeErr,0x01,adr,bus,speed)){return;}//--Отправка кода [0x01]; 	  
//codeErr=RT_HW_Base.i2cGetCodeErr(bus);					//--Получение кода ошибки; 
if(codeErr){step=85;               pointDelay=20;}		//--При ошибке повтор через динамическую задержку; 
else       {step=60;							 }		//--При ok переход на завершение;
}	//---------------------------------------------------------------------------------------------
if(step==60){	//++++Завершение инициализации-----------------------------------------------------   
codeErr=0; action=0; step=0; return;
}	//---------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
if(step==85){	//++++Динамическая задержка--------------------------------------------------------
if(++cntErr>6){cntErr=6; codeErr=82; err=1;} sizeDelay=cntErr*cntErr*RT_HW_DEVICE_DELAY; step=90;
}	//---------------------------------------------------------------------------------------------
if(step==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} step=pointDelay;
}	//---------------------------------------------------------------------------------------------
};	//====END init()===============================================================================

void meter(){									  //++++ИЗМЕРЕНИЕ параметров======================= 
if(step==0) {	//++++Подготовка внутренних параметров функции-------------------------------------
workDelay=err=codeErr=0; action=1; 						//--Очистка параметров;
     cntErr=0; step=10;									//--Переход на продолжение; 
}	//---------------------------------------------------------------------------------------------
if(step==10){	//++++Команда запуска измерения----------------------------------------------------
if(!RT_HW_Base.i2cSendByte(codeErr,0x10,adr,bus,speed)){return;}//-Отправка команды;	 
if(codeErr){step=85;                pointDelay=10;}		//--При ошибке повтор через динамическую  задержку;
else       {step=90; sizeDelay=150; pointDelay=20;cntErr=0;}//-При ok переход через фиксированную задержку 150ms;
}	//---------------------------------------------------------------------------------------------
if(step==20){	//++++Запрос состояния устройства[0x71]--------------------------------------------
if(!RT_HW_Base.i2cGetBytes(codeErr,bf,2,adr,bus,speed)){return;}//--Запрос состояния устройства;	 
if(codeErr){step=85; 		       pointDelay=10;}		//--При ошибке повтор через динамическую  задержку;
else       {step=25;                                   }//-При ok переход на завершение;
}	//---------------------------------------------------------------------------------------------
if(step==25){	//++++Чтение результатов измерения-------------------------------------------------    
vLux=((uint16_t)(bf[0]<<8) | bf[1])/1.2; 		 			   			 
fresh=ok=1; action=0; step=0;	return;						//--Выход по успешному завершению;	
}	//---------------------------------------------------------------------------------------------
//=================================================================================================
if(step==85){	//++++Динамическая задержка с выходом при многократной ошибке----------------------
if(++cntErr>3){cntErr=3; codeErr=84; err=1;} sizeDelay=cntErr*cntErr*RT_HW_DEVICE_DELAY; step=90;
}	//---------------------------------------------------------------------------------------------
if(step==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} step=pointDelay;
}	//---------------------------------------------------------------------------------------------											 
};	//====void meter()-----------------------------------------------------------------------------	
};	//++++END class RT_HW_BH1750====================================================================
#endif
//=================================================================================================
