//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			Sensor BME280
// 0 = Success
// 1 = Data too long to fit in transmit buffer
// 2 = Received NACK on transmit of address
// 3 = Received NACK on transmit of data
// 4 = Other error
// Oversampling: 0 to 3, higher numbers are slower, higher-res outputs (->delay=5,8,14,26).
// Will return delay in ms to wait, or 0 if I2C error.  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  ecoins (ecoins@mail.ru) 
//  2022-22-18: Begin version
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-------------------------------------------------------------------------------------------------
#ifndef RT_HW_BME280_h
#define RT_HW_BME280_h
#define RT_HW_BME280_VER 		 10		//--Номер версии библиотеки;
#include "RT_HW_BASE.h"					//--Подключение базовой библиотеки;
//-------------------------------------------------------------------------------------------------
#define RT_HW_BME280_DELAY_ERR 	50 		//--Базовая задержка при повторов после ошибки;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                     КЛАСС УПРАВЛЕНИЯ BME280
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class RT_HW_BME280{
public:
//-----------------Основные параметры управления---------------------------------------------------
uint8_t  custom=0;				//--Состояние настройки ID;  
//----------------Перенастраиваемые параметры пользователем----------------------------------------
char	 mi='I';				//--Тип интерфейса;
uint8_t  bus=0;					//--Номер шины;
uint8_t  adr=0x76;				//--Адрес i2c по умолчанию;
uint32_t speed=RT_HW_I2C_SPEED;	//--Скорость шины;
//----------------Основные переменные управления измерением-----------------------------------------
uint8_t  runTemp=0, cmdRunTemp=0;  //--Флаг для внешнего запуска измерений;
uint8_t  runPress=0,cmdRunPress=0; //--Флаг для внешнего запуска измерений;
uint8_t  runHum=0,  cmdRunHum=0;   //--Флаг для внешнего запуска измерений;
uint8_t  ddr=0;					//--Этапы основной программы управления устройствами;
uint8_t  step;					//--Управление этапами управления в функциях измерения;
//----------------Выходные параметры-----------------------------------------------------------------
uint8_t  fresh=1;				//--Флаг "свежих" данных;
uint8_t  ok=0;					//--Флаг "Устройство в норме;
uint8_t  codeErr,err;			//--Код ошибки при обмене;
//----------------Дополнительные переменные управления измерением-----------------------------------
uint8_t  initDevice=0, action;	//--Рабочие флаги;
uint8_t  cntErr;				//--Счетчик повторных ошибок;
uint8_t  cntFind;				//--Счетчик поиска устройств;
//----------------Параметры для функци timeDelay()[и также используются функцией i2cFindAdr()]------
uint32_t startDelay;		    //--Рабочий регистр;
uint8_t  workDelay=0;			//--Рабочий регистр; 
uint8_t  pointDelay;			//--Точка возврата из функции;
uint16_t sizeDelay;				//--Задержка для функции();
//----------------Буферные рабочие переменные (поддержка релевантности)------------------------------
uint32_t temp;					//--Рабочий регистр;
uint8_t  cmd;					//--Рабочий регистр;
uint8_t  idx,idx1;				//--Индексы;
uint8_t  bf8;					//--Буфер для чтения байта;
uint8_t  bf[10];              	//--Буфер для временного хранения данных;
//-------------------------------------------------------------------------------------------------
uint8_t  permitTemp=1;  		//--Разрешение на проведение измерения температуры;
uint8_t  permitPress=1; 		//--Разрешение на проведение измерения давления;
uint8_t  permitHum=1; 			//--Разрешение на проведение измерения влажности;
//----Определение значения параметров сенсора (могут быть изменены пользователем)------------------
uint8_t  mode   =0b011;		//--Режимы работы:  =00      спящий режим->нет операций, доступны все регистры, наименьшее энергопотребление, выбирается после запуска; 
                            //                  =01 & 10 принудительный режим->выполняет одно измерение, сохраняет результаты и возвращается в  спящий режим;
							//				    =11      нормальный режим-> постоянный цикл измерений и неактивные периоды;
uint8_t  t_sb   =0b000;		//--Период между измерениями температуры->=000:0.5ms; =001:62.5ms; =010:125ms; =011:250ms; =100:500ms; =101:1000ms; =110:10ms; =111:20ms;
uint8_t  filter	=0b001;		//--Коэф.фильтраци: =000 =фильтр отключен;
							//                  =001 =2;
							//                  =010 =4;
							//                  =011 =8;
							//                  =101 =16;							
uint8_t  osrs_t	 =0b001;	//--Коэф.передискредитации (усреднения) данных температуры=16; При включенном фильтре разрешение АЦП=20 бит.);
uint8_t  osrs_h	 =0b001;	//--Коэф.передискредитации (усреднения) данных влажности  =16; Разрешение АЦП=16 бит.);
uint8_t  osrs_p	 =0b001;	//--Коэф.передискредитации (усреднения) данных давления   =16; При включенном фильтре разрешение АЦП=20 бит.);
uint8_t  spi3w_en=0b000;	//--Режим работы SPI: =0 -четырехпроводный; =1 -трехпроводнsй;
uint8_t  modeChip=0;		//--Идентифицируемый из устройства код сенсора; 
//-------------------------------------------------------------------------------------------------
//uint8_t  vCONFIG;			//--Расчитываемое слово конфигурации;
//uint8_t  vCTRL_HUM;			//--Рассчитываемое слово настройки влажности;
//uint8_t  vCTRL_MEAS;		//--Рассчитываемое слово настройки meas;		
uint8_t  chipID;			//--Идентификатор чипа из сенсора;
int32_t  t_fine;
int32_t  t_fine_adjust=0;
uint8_t  bfVar;				//--буфер для значений из сенсора
//----Коэффициенты---------------------------------------------------------------------------------
uint16_t dig_T1; int16_t dig_T2, dig_T3;	//--
uint8_t  dig_H1; int16_t dig_H2; uint8_t dig_H3; int16_t dig_H4, dig_H5; int8_t dig_H6;
uint16_t dig_P1; int16_t dig_P2, dig_P3, dig_P4, dig_P5, dig_P6, dig_P7, dig_P8, dig_P9;
//-------------------------------------------------------------------------------------------------
int32_t  adc_T,adc_H,adc_P; 
uint32_t hH;
int32_t  var1, var2, var3, var4, var5;
int64_t  pVar1,pVar2,pP;
float    vTemp,vPress,vHum;	//--Измеренные значения;
//=================================================================================================
void setParam(uint8_t vMi='I', uint8_t vAdr=0x76, uint8_t vBus=0, uint32_t vSpeed=0){ //++++НАСТРОЙКА параметров=======================
if(custom==0){	//++++Однократная настройка параметров---------------------------------------------
mi=vMi; adr=vAdr;  bus=vBus; speed=vSpeed;	custom=1;}	//--Настройка адреса устройства,номера шины,скорости шины;
};	//+++++END setParam()==========================================================================

void direct(){								//++++УПРАВЛЕНИЕ измерением============================ 
//------------------------,-------------------------------------------------------------------------
if(runTemp) {cmdRunTemp=1;}			//--Фиксация внешней команды на запуск измерения температуры; 
if(runPress){cmdRunPress=1;}		//--Фиксация внешней команды на запуск измерения давления;  
if(runHum)  {cmdRunHum=1;} 
//-------------------------------------------------------------------------------------------------
if(ddr==0) {	//++++Начальная настройка----------------------------------------------------------	
workDelay=err=codeErr=cntErr=cntFind=initDevice=0; 	//--Инициализация рабочих параметров;
fresh=1; ok=0;  vTemp=0.11;  vPress=0.2; vHum=0.3;	//--Инициализация выходных параметров; 
            ddr=90; sizeDelay=200;  pointDelay=10;	//--Переход на продолжение через 200ms;
}	//----------------------------------------------------------------------------------------------
if(ddr==10){	//++++Выбор перехода[cmdRunXXXб,iniDevice] на инициализацию или на измерение-------
if((!cmdRunTemp)&&(!cmdRunPress)&&(!cmdRunHum)){return;} step=0;//--Ожидание команд cmdRunTemp,cmdRunPres,Hum;
if(initDevice==0){step=0; ddr=20;} else {ddr=30;}	//--Выбор переход на init() или на meterTemp();	
}	//---------------------------------------------------------------------------------------------
if(ddr==20){	//++++Инициализация устройства-----------------------------------------------------
init(); 	  										//--Пошаговая инициализация; 		  
if(err)    {ddr=90; sizeDelay=2000;pointDelay=0; cmdRunTemp=cmdRunPress=cmdRunHum=0;} //--Переход с задержкой на начало(ddr=0);
if(!action){ddr=30; initDevice=1; }   				//--Ожидание завершения инициализации;
if(ddr==20){return;}								//--Выход при продолжении работы meterTemp();
}	//---------------------------------------------------------------------------------------------
if(ddr==30){	//++++Проверка на запуск измерения температуры-------------------------------------
if(cmdRunTemp){cmdRunTemp=0;if(permitTemp){ddr=35; step=0;}};//-Проверка на запуск измерения температуры;
if(ddr==30){ddr=40;}								//--Если нет запуска по температуре, переход проверку cmdRunPress; 
}	//----------------------------------------------------------------------------------------------
if(ddr==35){	//++++Измерение температуры---------------------------------------------------------
meterTemp();	 	  								//--Пошаговое измерение температуры; 
if(err)    {ddr=90; sizeDelay=500;pointDelay=0;} 	//--Переход с задержкой при ошибке  meterTemp();
if(!action){ddr=90; sizeDelay=50; pointDelay=40;} 	//--Переход при успешном завершении meterTemp();
if(ddr==35){return;}								//--Выход   при продолжении работы  meterTemp();
}	//---------------------------------------------------------------------------------------------
if(ddr==40){	//++++Проверка на запуск измерения давления----------------------------------------
if(cmdRunPress){cmdRunPress=0;if(permitPress){ddr=45;step=0;}};//-Проверка на запуск измерения давления;
if(ddr==40){ddr=50;}								//--Если нет запуска по давлению, переход проверку cmdRunРгь;  
}	//---------------------------------------------------------------------------------------------
if(ddr==45){	//++++Измерение давления-----------------------------------------------------------
meterPress();	 	  								//--Пошаговое измерение давления; 
if(err)    {ddr=90;  sizeDelay=500;pointDelay=0;} 	//--Переход с задержкой при ошибке  meterPress();
if(!action){ddr=90;  sizeDelay=50; pointDelay=50;} 	//--Переход при успешном завершении meterPress();
if(ddr==45){return;}								//--Выход   при продолжении работы  meterPress();
}	//---------------------------------------------------------------------------------------------
if(ddr==50){	//++++Проверка на запуск измерения влажности---------------------------------------
if(cmdRunHum){cmdRunHum=0;if(permitHum){ddr=55;step=0;}};//-Проверка на запуск измерения температуры;
if(ddr==50){ddr=60;}								//--Если нет запуска, переход на завершение; 
}	//---------------------------------------------------------------------------------------------
if(ddr==55){	//++++Измерение давления-----------------------------------------------------------
meterHum();	 	  									//--Пошаговое измерение влажности; 
if(err)    {ddr=90;  sizeDelay=500;pointDelay=0;} 	//--Переход с задержкой при ошибке  meterHum();
if(!action){ddr=90;  sizeDelay=50; pointDelay=60;} 	//--Переход при успешном завершении meterHum();
if(ddr==55){return;}								//--Выход   при продолжении работы  meterHum();
}	//---------------------------------------------------------------------------------------------
if(ddr==60){	//++++Завершение измерения---------------------------------------------------------
			ddr=90; sizeDelay=500; pointDelay=10;	//--Переход на начало с задержкой 500ms;
}	//---------------------------------------------------------------------------------------------
if(ddr==90){	//++++Задержка по времени с возвратом в заданную точку-----------------------------	
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} else {ddr=pointDelay;  return;}
}	//---------------------------------------------------------------------------------------------
};	//+++++END direct()============================================================================

void init(){								//++++ПОЛУЧЕНИЕ настроечных параметров=================
if(step==0) {	//++++Подготовка внутренних параметров функции-------------------------------------           
workDelay=err=codeErr=0; action=1; 					//--Очистка параметров;
//vCTRL_HUM  = osrs_h & 0b00000111;							//--Настройка управляющего слова CTRL_HUM;
//vCONFIG    =(t_sb   << 5) | (filter << 2) | spi3w_en;	//--Настройка управляющего слова CONFIG;	
//vCTRL_MEAS =(osrs_t << 5) | (osrs_p << 2) | mode;		//--Настройка управляющего слова CTRL_MEAS; 
  cntErr=0; step=90; sizeDelay=50; pointDelay=10;	//--Переход на продолжение через задержку 50мс;     
}	//---------------------------------------------------------------------------------------------
if(step==10){	//++++Инициализация шины-----------------------------------------------------------
if(!RT_HW_Base.i2cInitDevice(codeErr,adr,bus)){return;}     //--Инициализация шины;
if(codeErr){err=1; codeErr=80;         return;}		//--Выход по ошибке инициализации шины; 
else       {step=15;      cntFind=0; cntErr=0;}		//--Безусловный переход на продолжение;
}	//---------------------------------------------------------------------------------------------
if(step==15){	//++++Поиск устройства на шине i2c-------------------------------------------------
if(!RT_HW_Base.i2cFindAdr(codeErr,adr,bus,speed)){return;};	//--Поиск устройства на на шине i2c;     
//codeErr=RT_HW_Base.i2cGetCodeErr(bus);				//--Получение кода ошибки;  	
if(codeErr){step=85;                pointDelay=15; cntFind=0;}//--При ошибке переход на динамическую задержку; 
else       {step=90; sizeDelay=50;  pointDelay=15; cntErr=0;  //--Переход на повторение поиска с задержкой 50ms;
            if(++cntFind>=3){step=20;  return;}}     //--Переход на продолжение в следующем цикле после 3-кратного обнаружения;
}	//--------------------------------------------------------------------------------------------
if(step==20){	//++++Чтение номера чипа----------------------------------------------------------
if(!readReg(0XD0)){return;}						//--Чтение из регистра 0xD0 в bf8 номер чипа;   
if(codeErr){step=85;               pointDelay=20;}	//--При ошибке повтор через динамическую задержку; 
else       {step=21;				 chipID=bf8; }	//--Сохранение номера чипа и переход;
}	//---------------------------------------------------------------------------------------------
if(step==21){	//++++Настройки под тип сенсора----------------------------------------------------
	 if(chipID==0x56) {modeChip=1; permitHum=0;} 	//--Сенсор типа BMP280;
else if(chipID==0x57) {modeChip=1; permitHum=0;} 	//--Сенсор типа BMP280;
else if(chipID==0x58) {modeChip=1; permitHum=0;} 	//--Сенсор типа BMP280;
else if(chipID==0x60) {modeChip=2;             } 	//--Сенсор типа BME280;
else 	              {modeChip=0; codeErr=150; err=1;return;}	//--Выход по неверному коду чипа;	
              step=25;				   return;		//--Переход в следующем цикле;
}	//---------------------------------------------------------------------------------------------
if(step==25) {	//++++Программный сброс------------------------------------------------------------
if(!sendReg(0xE0,0xB6)){return;}					//--Запись 0xB6->0xE0;
if(codeErr){step=85;                pointDelay=25;}	//--При ошибке переход на динамическую задержку;
else	   {step=90; sizeDelay=10;	pointDelay=30;}	//--Переход через фиксированную задержку 10ms;	
}	//---------------------------------------------------------------------------------------------
if(step==30){	//++++Проверка разрешения работы с температурой------------------------------------
if(permitTemp){step=31;} else{step=40;}
}	//---------------------------------------------------------------------------------------------
if(step==31){	//++++Чтение 6 байт для расчета коээфицинтов температуры---------------------------
if(!readRegs(0x88,6)){return;}
if(codeErr){step=85;                pointDelay=31;}	//--При ошибке переход на динамическую задержку;
else	   {dig_T1 =(bf[1]<<8) | bf[0];
			dig_T2 =(bf[3]<<8) | bf[2];
			dig_T3 =(bf[5]<<8) | bf[4];	
			step=90; sizeDelay=10;	pointDelay=40;} //--Переход через фиксированную задержку 10ms; 
}	//---------------------------------------------------------------------------------------------			   
if(step==40){	//++++Проверка разрешения работы с давлением---------------------------------------
if(permitPress){step=41;} else{step=50;}
}	//---------------------------------------------------------------------------------------------
if(step==41){	//++++Чтение 6 байт для расчета коэффицинтов давления 1,2,3------------------------
if(!readRegs(0x8E,6)){return;}
if(codeErr){step=85;                pointDelay=41;}	//--При ошибке переход на динамическую задержку;
else	   {dig_P1 =(bf[1]<< 8) | bf[0];
			dig_P2 =(bf[3]<< 8) | bf[2];
			dig_P3 =(bf[5]<< 8) | bf[4];		
			step=90; sizeDelay=10;	pointDelay=42;}	//--Переход через фиксированную задержку 10ms;
}	//--------------------------------------------------------------------------------------------- 
if(step==42){	//++++Чтение 6 байт для расчета коэффицинтов давления 4,5,6------------------------
if(!readRegs(0x94,6)){return;}
if(codeErr){step=85;                pointDelay=43;}	//--При ошибке переход на динамическую задержку;
else	   {dig_P4 =(bf[1]<< 8) | bf[0];
			dig_P5 =(bf[3]<< 8) | bf[2];
			dig_P6 =(bf[5]<< 8) | bf[4];		
			step=90; sizeDelay=10;	pointDelay=43;} //--Переход через фиксированную задержку 10ms; 
}	//----------------------------------------------------------------------------------------------
if(step==43){	//++++Чтение 6 байт для расчета коэффицинтов давления 4,5,6------------------------
if(!readRegs(0x9A,6)){return;}
if(codeErr){step=85;                pointDelay=43;}	//--При ошибке переход на динамическую задержку;
else	   {dig_P7 =(bf[1]<< 8) | bf[0];
			dig_P8 =(bf[3]<< 8) | bf[2];
			dig_P9 =(bf[5]<< 8) | bf[4];		
			step=90; sizeDelay=10;	pointDelay=50;}	//--Переход через фиксированную задержку 10ms;  
}	//----------------------------------------------------------------------------------------------
if(step==50){	//++++Проверка разрешения работы с влажностью--------------------------------------
if(permitPress){step=51;} else{step=60;}
}	//---------------------------------------------------------------------------------------------
if(step==51){	//++++Чтение байт для расчета коэффицинтов влажности 1-----------------------------
if(!readReg(0xA1)){return;}
if(codeErr){step=85;                pointDelay=51;}	//--При ошибке переход на динамическую задержку;
else	   {dig_H1=bf8;		
			step=90; sizeDelay=10;	pointDelay=52;}	//--Переход через фиксированную задержку 10ms;
}	//--------------------------------------------------------------------------------------------- 
if(step==52){	//++++Чтение 7 байт для расчета коэффицинтов влажности 2,3,4,5,6-------------------
if(!readRegs(0xE1,7)){return;}
if(codeErr){step=85;                pointDelay=52;}	//--При ошибке переход на динамическую задержку;
else	   {dig_H2 =(bf[1]<<8) | bf[0];
			dig_H3 = bf[2]<<8;
	        dig_H4 =(bf[3]<<4) |(bf[4] &0x0F);
			dig_H5 =(bf[5]<<4) |((bf[4]>>4) & 0x0F);
			dig_H6 = bf[6];		
			step=90; sizeDelay=10;	pointDelay=60;} //--Переход через фиксированную задержку 10ms; 
}	//----------------------------------------------------------------------------------------------
if(step==60){	//++++Настройка CTRL_HUM------------------------------------------------------------
bf8=osrs_h & 0b00000111;
if(!sendReg(0xF2,bf8)){return;}
if(codeErr){step=85;                pointDelay=60;}	//--При ошибке переход на динамическую задержку;
else	   {step=90; sizeDelay=10;	pointDelay=61;}	//--Переход через фиксированную задержку 10ms;  
}	//----------------------------------------------------------------------------------------------
if(step==61){	//++++Настройка CONFIG--------------------------------------------------------------
bf8=(t_sb   << 5) | (filter << 2) | spi3w_en;
if(!sendReg(0xF5,bf8)){return;}
if(codeErr){step=85;                pointDelay=61;}	//--При ошибке переход на динамическую задержку;
else	   {step=90; sizeDelay=10;	pointDelay=62;}	//--Переход через фиксированную задержку 10ms;  
}	//----------------------------------------------------------------------------------------------
if(step==62){	//++++Настройка CTRL_MEAS-----------------------------------------------------------
bf8=(osrs_t << 5) | (osrs_p << 2) | mode;
if(!sendReg(0xF4,bf8)){return;}
if(codeErr){step=85;                pointDelay=62;}	//--При ошибке переход на динамическую задержку;
else	   {step=90; sizeDelay=10;	pointDelay=70;}	//--Переход через фиксированную задержку 10ms;  
}	//----------------------------------------------------------------------------------------------
if(step==70){	//++++Контрольное чтение номера чипа------------------------------------------------
if(!readReg(0xD0)){return;}						//--Чтение из регистра 0xD0 в bf8 номер чипа;   
if(codeErr){step=85;                  pointDelay=20;}  //--При ошибке повтор через динамическую задержку; 
else       {if(chipID!=bf8){step=85;  pointDelay=20;}  //--Проверка совпадения с ранее
            else{step=90;sizeDelay=10;pointDelay=75;}} //        считанным номером чипа;
}	//--------------------------------------------------------------------------------------------- 
if(step==75){	//++++Успешное завершение----------------------------------------------------------
//--------------------------------------------------------
/*
	Serial.print(F("dig_T1="));  Serial.print(dig_T1);
	Serial.print(F(" dig_T2=")); Serial.print(dig_T2);
	Serial.print(F(" dig_T3=")); Serial.print(dig_T3);
	Serial.println();
	
    Serial.print(F("dig_P1="));  Serial.print(dig_P1);
	Serial.print(F(" dig_P2=")); Serial.print(dig_P2);
	Serial.print(F(" dig_P3=")); Serial.print(dig_P3);
	Serial.print(F(" dig_P4=")); Serial.print(dig_P4);
	Serial.print(F(" dig_P5=")); Serial.print(dig_P5);
	Serial.print(F(" dig_P6=")); Serial.print(dig_P6);
	Serial.println();

    Serial.print(F("dig_H1="));  Serial.print(dig_H1);
	Serial.print(F(" dig_H2=")); Serial.print(dig_H2);
	Serial.print(F(" dig_H3=")); Serial.print(dig_H3);
	Serial.print(F(" dig_H4=")); Serial.print(dig_H4);
	Serial.print(F(" dig_H5=")); Serial.print(dig_H5);
	Serial.print(F(" dig_H6=")); Serial.print(dig_H6);
	Serial.println();
*/
//---------------------------------------------------------
initDevice=1; action=0;   return;
}	//--------------------------------------------------------------------------------------------- 
//-------------------------------------------------------------------------------------------------
if(step==85){	//++++Динамическая задержка--------------------------------------------------------
if(++cntErr>6){cntErr=6; codeErr=82; err=1;} sizeDelay=cntErr*cntErr*RT_HW_DEVICE_DELAY; step=90;
}	//---------------------------------------------------------------------------------------------
if(step==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} step=pointDelay;
}	//---------------------------------------------------------------------------------------------
};	//++++END init()===============================================================================
void meterTemp(){						 	//++++ЧТЕНИЕ ТЕМПЕРАТУРЫ===============================		
if(step==0) {	//++++Подготовка внутренних параметров функции-------------------------------------
workDelay=err=codeErr=0; action=1; 						//--Очистка параметров;
     cntErr=0; step=10;									//--Переход на продолжение;           
}	//---------------------------------------------------------------------------------------------
if(step==10){	//++++Чтение температуры-----------------------------------------------------------
if(!readRegs(0xFA,3)){return;}						//--Чтение из регистров 0xFA,FB,FC;
if(codeErr){step=85; codeErr=110;  pointDelay=10;}	//--При ошибке переход на динамическую задержку;
if(!readReg (0xD0))  {return;}						//--Контрольное чтение номера чипа из регистра 0xD0;
if(codeErr){         step=85; codeErr=111; pointDelay=10;}	//--При ошибке повтор через динамическую задержку; 
else{if(chipID!=bf8){step=85; codeErr=112; pointDelay=10;}	//--Контрольное чтение чипа,при err->дин.задержку;
	 else{adc_T  =(uint32_t)bf[0] << 12;
		  adc_T |=(uint32_t)bf[1] <<  4;
		  adc_T |=(         bf[2] >>  4) & 0x0F;	
                     step=20; return;}}  					//--Переход на вычисление в следующем цикле;       
}	//---------------------------------------------------------------------------------------------
if(step==20){	//++++Расчет температуры----------------------------------------------------------- 
var1 = (int32_t)((adc_T / 8) - ((int32_t)dig_T1 * 2));
var1 = (var1 * ((int32_t)dig_T2)) / 2048;
var2 = (int32_t)((adc_T / 16) - ((int32_t)dig_T1));
var2 = (((var2 * var2) / 4096) * ((int32_t)dig_T3)) / 16384;
t_fine = var1 + var2 + t_fine_adjust;
int32_t T = (t_fine * 5 + 128) / 256;
vTemp=(float)T / 100;
fresh=ok=1; action=0; return;						//--Успешный выход;
}	//---------------------------------------------------------------------------------------------		
//-------------------------------------------------------------------------------------------------
if(step==85){	//++++Динамическая задержка--------------------------------------------------------
if(++cntErr>6){cntErr=6; codeErr=82; err=1;} sizeDelay=cntErr*cntErr*RT_HW_DEVICE_DELAY; step=90;
}	//---------------------------------------------------------------------------------------------
if(step==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} step=pointDelay;
}	//---------------------------------------------------------------------------------------------
};	//++++END meterTemp()==========================================================================

void meterHum(){						 	//++++ЧТЕНИЕ ВЛАЖНОСТИ=================================	
if(step==0) {	//++++Подготовка внутренних параметров функции-------------------------------------
workDelay=err=codeErr=0; action=1; 						//--Очистка параметров;
     cntErr=0; step=10;									//--Переход на продолжение;           
}	//---------------------------------------------------------------------------------------------
if(step==10){	//++++Чтение влажности-------------------------------------------------------------
if(!readRegs(0xFD,2)){return;}						//--Чтение из регистров 0xFA,FB,FC;
if(codeErr){step=85; codeErr=120;               pointDelay=10;}	//--При ошибке переход на динамическую задержку;
if(!readReg (0xD0))  {return;}						//--Контрольное чтение номера чипа из регистра 0xD0;
if(codeErr){step=85; codeErr=121;               pointDelay=10;}	//--При ошибке повтор через динамическую задержку; 
else{if(chipID!=bf8){step=85;pointDelay=10;}	//--Контрольное чтение чипа,при err->дин.задержку;
	     else{adc_H  =(uint32_t)bf[0] << 8;
		      adc_H |=(uint32_t)bf[1]; 
		      if(adc_H==0x8000){codeErr=123; step=85;  pointDelay=10;}
              else{  step=20;    return;}}} //--Переход на продолжение в следующем цикле;
}	//---------------------------------------------------------------------------------------------				 
if(step==20){	//++++Вычисление влажности---------------------------------------------------------
var1 = t_fine - ((int32_t)76800);
var2 = (int32_t)(adc_H * 16384);
var3 = (int32_t)(((int32_t)dig_H4) * 1048576);
var4 = ((int32_t)dig_H5) * var1;
var5 = (((var2 - var3) - var4) + (int32_t)16384) / 32768;
var2 = (var1 * ((int32_t)dig_H6)) / 1024;
var3 = (var1 * ((int32_t)dig_H3)) / 2048;
var4 = ((var2 * (var3 + (int32_t)32768)) / 1024) + (int32_t)2097152;
var2 = ((var4 * ((int32_t)dig_H2)) + 8192) / 16384;
var3 = var5 * var2;
var4 = ((var3 / 32768) * (var3 / 32768)) / 128;
var5 = var3 - ((var4 * ((int32_t)dig_H1)) / 16);
var5 = (var5 < 0 ? 0 : var5);
var5 = (var5 > 419430400 ? 419430400 : var5);
uint32_t hH = (uint32_t)(var5 / 4096);
vHum=(float)hH / 1024.0;
fresh=ok=1; action=0; return;						//--Успешный выход;
}	//---------------------------------------------------------------------------------------------		   
//-------------------------------------------------------------------------------------------------
if(step==85){	//++++Динамическая задержка--------------------------------------------------------
if(++cntErr>6){cntErr=6; codeErr=82; err=1;} sizeDelay=cntErr*cntErr*RT_HW_DEVICE_DELAY; step=90;
}	//---------------------------------------------------------------------------------------------
if(step==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} step=pointDelay;
}	//---------------------------------------------------------------------------------------------
};	//++++END meterHum()===============================================================================


void meterPress(){						 	//++++ЧТЕНИЕ ДАВЛЕНИЯ==================================	
if(step==0) {	//++++Подготовка внутренних параметров функции-------------------------------------
workDelay=err=codeErr=0; action=1; 						//--Очистка параметров;
     cntErr=0; step=10;									//--Переход на продолжение;           
}	//---------------------------------------------------------------------------------------------
if(step==10){	//++++Чтение температуры-----------------------------------------------------------
if(!readRegs(0xF7,3)){return;}						//--Чтение из регистров 0xFA,FB,FC;
if(codeErr){step=85; codeErr=110;  pointDelay=10;}	//--При ошибке переход на динамическую задержку;
if(!readReg (0xD0))  {return;}						//--Контрольное чтение номера чипа из регистра 0xD0;
if(codeErr){         step=85; codeErr=111; pointDelay=10;}	//--При ошибке повтор через динамическую задержку; 
else{if(chipID!=bf8){step=85; codeErr=112; pointDelay=10;}	//--Контрольное чтение чипа,при err->дин.задержку;
	 else{adc_P  = (uint32_t)bf[0] << 12;
 		  adc_P |= (uint32_t)bf[1] << 4;
		  adc_P |= (         bf[2] >> 4) & 0x0F;	
                     step=20; return;}}  					//--Переход на вычисление в следующем цикле;       
}	//---------------------------------------------------------------------------------------------
if(step==20){	//++++Вычисление давления----------------------------------------------------------
pVar1 = ((int64_t)t_fine) - 128000;
pVar2 =  pVar1 *   pVar1 * (int64_t)dig_P6;
pVar2 =  pVar2 + ((pVar1 * (int64_t)dig_P5)<<17);
pVar2 =  pVar2 + (((int64_t)dig_P4)<<35);
pVar1 =((pVar1 *   pVar1 * (int64_t)dig_P3)>>8) + ((pVar1 * (int64_t)dig_P2)<<12);
pVar1 =(((((int64_t)1)<<47)+pVar1))*((int64_t)dig_P1)>>33;
if(pVar1 == 0){action=0; return;}
pP = 1048576 - adc_P;
pP = (((pP << 31) - pVar2)*3125)/pVar1;
pVar1 = (((int64_t)dig_P9) * (pP >> 13) * (pP >> 13)) >> 25;
pVar2 = (((int64_t)dig_P8) *  pP)>> 19;
pP    = ((pP + pVar1 + pVar2) >> 8) + (((int64_t)dig_P7)<<4);
pP    = pP >> 8; // /256
vPress= pP/100.0;
fresh=ok=1; action=0; return;
}	//---------------------------------------------------------------------------------------------	
//-------------------------------------------------------------------------------------------------
if(step==85){	//++++Динамическая задержка--------------------------------------------------------
if(++cntErr>6){cntErr=6; codeErr=82; err=1;} sizeDelay=cntErr*cntErr*RT_HW_DEVICE_DELAY; step=90;
}	//---------------------------------------------------------------------------------------------
if(step==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} step=pointDelay;
}	//---------------------------------------------------------------------------------------------
};	//++++END meterPress()=========================================================================
//=================================================================================================
//=================================================================================================
uint8_t sendReg (uint8_t vReg, uint8_t var){	//++++Запись байта в регистр=======================
if(mi=='I'){bf[0]=vReg; bf[1]=var;
if(!RT_HW_Base.i2cSendBytes(codeErr,bf,2,adr,bus,speed)){return 0;}	 return 1;
}	//---------------------------------------------------------------------------------------------
if(mi=='S'){	//++++SPI.Чтение байта из регистра-------------------------------------------------
codeErr=0; return 1;
}	//---------------------------------------------------------------------------------------------	
if(mi=='s'){	//++++SSPI.Чтение байта из регистра------------------------------------------------
codeErr=0; return 1;
}	//---------------------------------------------------------------------------------------------
return 1;};	//++++END sendReg()===========================================================================
uint8_t readReg (uint8_t vReg){					//++++Чтение байта из регистра в bf8===============
if(mi=='I'){	//++++i2c.Чтение байта из регистра-------------------------------------------------
if(!RT_HW_Base.i2cReadReg (codeErr,vReg,bf8,adr,bus,speed,1)){return 0;}  return 1;
}	//--------------------------------------------------------------------------------------------- 			
if(mi=='S'){	//++++SPI.Чтение байта из регистра-------------------------------------------------
codeErr=0; return 1;
}	//---------------------------------------------------------------------------------------------	
if(mi=='s'){	//++++SSPI.Чтение байта из регистра------------------------------------------------
codeErr=0; return 1;
}	//---------------------------------------------------------------------------------------------
return 1;};	//++++END readReg()===========================================================================	
uint8_t readRegs(uint8_t vReg, uint8_t qnt){	//++++Чтение qnt байт из регистра в bf[]===========
//-------------------------------------------------------------------------------------------------
if(mi=='I'){	//++++i2c.Чтение байтов из регистра------------------------------------------------
if(!RT_HW_Base.i2cReadRegs(codeErr,vReg,bf,qnt,adr,bus,speed,1)){return 0;}  return 1;
}	//----------------------------------------------------------------------------------------------- 
if(mi=='S'){	//++++SPI.Чтение байтов из регистра------------------------------------------------
codeErr=0; return 1;
}	//---------------------------------------------------------------------------------------------	
if(mi=='s'){	//++++SSPI.Чтение байтов из регистра-----------------------------------------------
codeErr=0; return 1;
}	//---------------------------------------------------------------------------------------------
return 1;};	//++++END ReadRegs()
float getTempCompensation() {					//++++Получение температатурной компенсации========
return float((t_fine_adjust*5)>>8)/100.0;}; 					
void  setTempCompensation(float adjustment){	//++++Установка темературной компенсации===========
t_fine_adjust=((int32_t(adjustment*100)<<8))/5;};
//=================================================================================================	
};	//++++END Class RT_HW_BME280===================================================================
#endif
//=================================================================================================	

/*
//-------------------------------------------------------------------------------------------------
uint8_t  dir=0;						//--Управление этапами управления в функции direct();
uint8_t  step;						//--Управление этапами управления в функциях измерения;
char	 mi='I';					//--Тип интерфейса;
uint8_t  bus=0;						//--Номер шины;
uint8_t  adr=0x76;					//--Адрес i2c по умолчанию;
uint32_t speed=400000UL;			//--Скорость шины;
uint8_t  sck=255;					//--Номер пина clk  для SSPI;
uint8_t  miso=255;					//--Номер пина miso для SSPI;
uint8_t  mosi=255;					//--Номер пина mosi для SSPI;
uint8_t  cs=255;					//--Номер пина cs   для SSPI(SPI)
uint8_t  codeErr;					//--Код ошибки при обмене;
uint8_t  qntMaxErr=5;				//--Максимальное кол-во повторяемых ошибок;
uint8_t  cnt;						//--Рабочий счетчик;
uint32_t timeBegin;					//--Используется функцией timeDelay(timeBegin,stepDelay,sizeDelay);
uint8_t  stepDelay=0;				//--Используется функцией timeDelay();
uint16_t sizeDelay;					//--Вычисляемая задержка для функции timeDelay(),используется также для поиска на шине i2c;		
uint8_t  run=1;         //--Флаг для внешнего запуска измерений;
uint8_t  finish=0;      //--Флаг завершения измерения;
uint8_t  fresh;         //--Флаг готовности данных;
uint8_t  runTemp:1,runPress:1,runHum:1;    //--Флаги запуска измерений внешней функцией;
uint8_t  permitTemp=1;  //--Разрешение на проведение измерения температуры (может быть отключено внешней функцией);
uint8_t  permitPress=1; //--Разрешение на проведение измерения давления    (может быть отключено внешней функцией);
uint8_t  permitHum=1;   //--Разрешение на проведение измерения влажности   (может быть отключено внешней функцией);
uint8_t  initFresh:1,ok:1,action:1,err:1;



*/


/*
if(step==50){if(!permitHum){step=12; return;};
			   //bme280_readByte(0xA1); 	    	if(codeErr){step=50; return;} 	
			   dig_H1=val;  
			   step++; return;}	//--DIG_H1=0xA1;
if(step==11){uint8_t data[7];
			   //bme280_readBytes(0xE1,7,data); 	if(codeErr){step=50; return;}	//--DIG_H2_LSB=0x9A,0x9B,0x9C,0x9D,0x9E,0x9F;
			   dig_H2 =(data[1] << 8) | data[0];
			   dig_H3 = data[2] << 8;
	           dig_H4 =(data[3] << 4) |(data[4] &0x0F);
			   dig_H5 =(data[5] << 4) |((data[4] >> 4) & 0x0F);
			   dig_H6 = data[6];
			   step++; return;}
*/	
		   
/*			   
if(step==12){val=osrs_h & 0b00000111;																	//--Настройка CTRL_HUM;
				//bme280_writeByte(0xF2,val);		if(codeErr){step=50; return;} 
				step++; return;} 	//--Запись в регистр CTRL_HUM;  
if(step==13){val=(t_sb   << 5) | (filter << 2) | spi3w_en;										//--Настройка CONFIG;
				//bme280_writeByte(0xF5,val);		if(codeErr){step=50; return;} 
				step++; return;}   //--Запись в регистр CONFIG;      
if(step==14){val=(osrs_t << 5) | (osrs_p << 2) | mode;											//--Настройка CTRL_MEAS;
				//bme280_writeByte(0xF4,val); 	if(codeErr){step=50; return;} 
				step++; return;} 	//--Запись в регистр CTRL_MEAS;
if(step==15){//bme280_readByte(0xD0); 	 		if(codeErr){step=50; return;}						//--Контрольное чтение chipID; 
                if(val!=chipID) {err=1;  return;} 							  
				step++; return;}	//--Чтение кода сенсора;
//------------------------------------------------------------
//if(step==16) {initFresh=1; action=0;    return;}
*/

/*
			   step++; return;}
if(step==7) {if(!permitPress){step=10; return;};
			   uint8_t data[6];
			   //bme280_readBytes(0x8E,6,data); 	if(codeErr){step=50; return;}	//--DIG_P1_LSB=0x8E,0x8F,0x90,0x91,0x92,0x93;			   
			   dig_P1 = (data[1] << 8) | data[0];
			   dig_P2 = (data[3] << 8) | data[2];
			   dig_P3 = (data[5] << 8) | data[4];	
			   step++; return;}
if(step==8) {uint8_t data[6];
			   //bme280_readBytes(0x94,6,data); 	if(codeErr){step=50; return;}	//--DIG_P4_LSB=0x94,0x95,0x96,0x97,0x98,0x99;
			   dig_P4 = (data[1] << 8) | data[0];
			   dig_P5 = (data[3] << 8) | data[2];
			   dig_P6 = (data[5] << 8) | data[4];	
			   step++; return;}
if(step==9) {uint8_t data[6];
			   //bme280_readBytes(0x9A,6,data); 	if(codeErr){step=50; return;}	//--DIG_P7_LSB=0x9A,0x9B,0x9C,0x9D,0x9E,0x9F;
			   dig_P7 = (data[1] << 8) | data[0];
			   dig_P8 = (data[3] << 8) | data[2];
			   dig_P9 = (data[5] << 8) | data[4];	 		   
			   step++; return;}
*/


/*

void bme280(){
//-------------------------------------------------------------------------------------------------
// 							Настройка параметров
//-------------------------------------------------------------------------------------------------
if(dir==0) {runTemp=runPress=runHum=1; initFresh=0; ok=0; fresh=1; 
			  vTemp=-0.1; vPress= 0.2;					//--Начальные параметры для вывода;
			  codeErr=RT_HW_Base.i2cInitMaster(bus); 	//--Настройка шины i2c;
			  if(codeErr){dir=95; return;}				//--Выход по ошибке;
              sizeDelay=0; dir=5;}						//--Подготовка параметров и переход;
//-------------------------------------------------------------------------------------------------
// 							Поиск устройства на шине i2c
//-------------------------------------------------------------------------------------------------
if(dir==5) {if(RT_HW_Base.i2cFindAdr(timeBegin,sizeDelay,adr,bus)){dir=15;} return;}
//-------------------------------------------------------------------------------------------------
// 							Настройка параметров
//-------------------------------------------------------------------------------------------------
if(dir==15){step=0; dir++;}				//--Настройка параметров;
if(dir==16){bme280_parameters();		//--Пошаговая настройка параметров; 
            if(err){dir=0;   return;} 	//--Переход на начало по ошибке;
			if(action)      {return;} 	//--Ожидание завершения;
			dir=20;}						//--Переход;
//-------------------------------------------------------------------------------------------------
// 							Ожидание флагов запуска измерений
//-------------------------------------------------------------------------------------------------
if(dir==20){if(runTemp || runHum || runPress) {finish=0; dir=30; return;} else {return;}}			
//-------------------------------------------------------------------------------------------------
// 							Измерение температуры
//-------------------------------------------------------------------------------------------------
if(dir==30){if(runTemp) {runTemp=0; step=0; if(permitTemp){dir++;} else{dir=40;}}  else {dir=40; return;}}	//--Проверка флага;
if(dir==31){bme280_MeasurumentTemp();	//--Пошаговое измерение; 
              if(err){dir=0;   return;} //--Переход на начало по ошибке;
			  if(action)      {return;} //--Ожидание завершения функции калибровки;
			  dir=40;}					//--Переход;
//-------------------------------------------------------------------------------------------------
// 							Измерение Влажности
//-------------------------------------------------------------------------------------------------
if(dir==40){if(runHum)  {runHum=0; step=0; if(permitHum){dir++;} else{dir=50;}}  else {dir=50; return;}}	//--Проверка флага;
if(dir==41){bme280_MeasurumentHum();	//--Пошаговая настройка параметров; 
              if(err){dir=0;   return;} //--Переход на начало по ошибке;
			  if(action)      {return;} //--Ожидание завершения функции калибровки;
			  dir=50;}					//--Переход;
//-------------------------------------------------------------------------------------------------
// 							Измерение давления
//-------------------------------------------------------------------------------------------------
if(dir==50){if(runPress){runPress=0;step=0;if(permitPress){dir++;} else{dir=60;}}  else {dir=60; return;}}	//--Проверка флага;
if(dir==51){bme280_MeasurumentPress();	//--Пошаговая настройка параметров; 
              if(err){dir=0;   return;} //--Переход на начало по ошибке;
			  if(action)      {return;} //--Ожидание завершения функции калибровки;
			  dir=60;}					//--Переход;
//-------------------------------------------------------------------------------------------------
if(dir==60){
	//RT_HW_Base.consoleVar(String("***T=")); RT_HW_Base.consoleVar(vTemp,';'); 
	//RT_HW_Base.consoleVar(String(" H="));   RT_HW_Base.consoleVar(vHum,';'); 
	//RT_HW_Base.consoleVar(String(" P="));   RT_HW_Base.consoleVar(vPress,'E'); 		
	dir=80;}
//-------------------------------------------------------------------------------------------------
// 					Задержка с последующим возвратом на цикл измерения
//-------------------------------------------------------------------------------------------------
if(dir==80){stepDelay=0; dir++;}
if(dir==81){if(!RT_HW_Base.timeDelay(timeBegin,stepDelay,50)){return;} else {finish=1; dir=20; return;}}
//-------------------------------------------------------------------------------------------------
}
*/
