//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			СЕНСОР HTU21
//  ecoins (ecoins@mail.ru) 
//  22.10.2022: Begin version
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-------------------------------------------------------------------------------------------------
#ifndef RT_HW_HTU21_h
#define RT_HW_HTU21_h
#define RT_HW_HTU21_VER 				10			//--Номер версии библиотеки;
#include "RT_HW_BASE.h"								//--Подключение базовой библиотеки;
//-------------------------------------------------------------------------------------------------
//#define RT_HW_HTU21_ADDRESS 			0x40  		//--Адрес сенсора на шине i2c
#define RT_HW_HTU21_DELAY_ERR   		50 			//--Базовая задержка при повторов после ошибки;
#define RT_HW_HTU21_TIME_METER_TMP		50  		//--Время измерения(mc) в сенсоре после запроса;
#define RT_HW_HTU21_TIME_METER_HUM		70  		//--Время(сенсора) измерения в сенсоре после запроса;
#define RT_HW_HTU21_TIME_REPEAT_RESET   1000		//--Период(bc повторного программного сброса; 
#define RT_HW_HTU21_MAX_NUM_ERR			3			//--Максимальное кол-во тиков с ошибками;
//#define RT_HW_HTU21_FIND_MAX			20			//--Максимальное кол-во циклов при поиске на шине i2c;
#define RT_HW_HTU21_TRIG_TMP_HOLD	 	0xE3		//--Измерение температуры c  удержанием шины;
#define RT_HW_HTU21_TRIG_HUM_HOLD		0xE5		//--Измерение влажности   с  удержанием шины;
#define RT_HW_HTU21_TRIG_TMP_NOHOLD	 	0xF3		//--Измерение темературы без удержания  шины;
#define RT_HW_HTU21_TRIG_HUM_NOHOLD		0xF5		//--Измерение влажности  без удержания  шины;
#define RT_HW_HTU21_SOFT_RESET			0xFE		//--Код программного сброса сенсора
#define RT_HW_HTU21_SHIFTED_DIVISOR 	0x988000	//--Полином 0x0131 = x^8 + x^5 + x^4 + 1;	
#define RT_HW_HTU21_TMP_DEFAULT			-1			//--Значение температуры default;
#define RT_HW_HTU21_HUM_DEFAULT			 2			//--Значение влажности   default;	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                     КЛАСС УПРАВЛЕНИЯ SHT3X
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class RT_HW_HTU21{
public:
//-----------------Основные параметры управления---------------------------------------------------
uint8_t  custom=0;				//--Состояние настройки ID;  
//----------------Перенастраиваемые параметры пользователем----------------------------------------
uint8_t  bus=0;					//--Номер шины;
uint8_t  adr=0x40;				//--Адрес i2c по умолчанию;
uint32_t speed=RT_HW_I2C_SPEED;	//--Скорость шины;
//----------------Основные переменные управления измерением----------------------------------------
uint8_t  runTemp=0,cmdRunTemp=0;//--Флаг для внешнего запуска измерений;
uint8_t  runHum=0, cmdRunHum=0; //--Флаг для внешнего запуска измерений;
uint8_t  ddr=0;					//--Этапы основной программы управления устройствами;
uint8_t  step;					//--Управление этапами управления в функциях измерения;
//----------------Выходные параметры---------------------------------------------------------------
uint8_t  fresh=1;				//--Флаг "свежих" данных;
uint8_t  ok=0;					//--Флаг "Устройство в норме;
uint8_t  codeErr,err;			//--Код ошибки при обмене;
//----------------Дополнительные переменные управления измерением----------------------------------
uint8_t  initDevice=0, action;	//--Рабочие флаги;
uint8_t  cntErr;				//--Счетчик повторных ошибок;
uint8_t  cntFind;				//--Счетчик поиска устройств;
//----------------Параметры для функци timeDelay()[и также используются функцией i2cFindAdr()]-----
uint32_t startDelay;		    //--Рабочий регистр;
uint8_t  workDelay=0;			//--Рабочий регистр; 
uint8_t  pointDelay;			//--Точка возврата из функции;
uint16_t sizeDelay;				//--Задержка для функции();
//----------------Буферные рабочие переменные (поддержка релевантности)----------------------------
uint8_t  crc;
uint16_t raw;
uint32_t divisor=RT_HW_HTU21_SHIFTED_DIVISOR; //--Полином
uint32_t remainder;
float 	 hHum,vTemp,vHum;
uint8_t  idx;					//--Индексы;
uint8_t  bf[3];              	//--Буфер для временного хранения данных;
//-------------------------------------------------------------------------------------------------
uint8_t  permitTemp=1;  		//--Разрешение на проведение измерения температуры;
uint8_t  permitHum=1; 		    //--Разрешение на проведение измерения влажности;
//=================================================================================================
void setParam(uint8_t vAdr=0x40, uint8_t vBus=0, uint32_t vSpeed=0){ //++++НАСТРОЙКА параметров=======================
if(custom==0){	//++++Однократная настройка параметров---------------------------------------------
adr=vAdr;  bus=vBus; speed=vSpeed;	custom=1;}	//--Настройка адреса устройства,номера шины,скорости шины;
};	//+++++END setParam()==========================================================================

void direct(){									  //++++УПРАВЛЕНИЕ измерением====================== 
//-------------------------------------------------------------------------------------------------
if(runTemp) {cmdRunTemp=1;}			//--Фиксация внешней команды на запуск измерения температуры; 
if(runHum)  {cmdRunHum=1;}			//--Фиксация внешней команды на запуск измерения влажности;  
//-------------------------------------------------------------------------------------------------
if(ddr==0) {	//++++Начальная настройка----------------------------------------------------------	
workDelay=err=codeErr=cntErr=cntFind=initDevice=0; 	//--Инициализация рабочих параметров;
fresh=1; ok=0; vTemp=0.11; vHum=0.12;				//--Инициализация выходных параметров; 
            ddr=90; sizeDelay=200;  pointDelay=10;	//--Переход на продолжение через 200ms;
}	//----------------------------------------------------------------------------------------------
if(ddr==10){	//++++Выбор перехода[cmdRunXXXб,iniDevice] на инициализацию или на измерение-------
if((!cmdRunTemp)&&(!cmdRunHum)){return;} step=0;	//--Ожидание команд cmdRunTemp,cmdRunHum;
if(initDevice==0){step=0; ddr=20;} else {ddr=30;}	//--Выбор переход на init() или на meterTemp();	
}	//---------------------------------------------------------------------------------------------
if(ddr==20){	//++++Инициализация устройства-----------------------------------------------------
init(); 	  										//--Пошаговая инициализация; 		  
if(err)    {ddr=90; sizeDelay=2000;pointDelay=0; cmdRunTemp=cmdRunHum=0;} //--Переход с задержкой на начало(ddr=0);
if(!action){ddr=30; initDevice=1; }   				//--Ожидание завершения инициализации;
if(ddr==20){return;}								//--Выход при продолжении работы meterTemp();
}	//---------------------------------------------------------------------------------------------
if(ddr==30){	//++++Проверка на запуск измерения температуры-------------------------------------
if(cmdRunTemp){cmdRunTemp=0;if(permitTemp){ddr=35; step=0;}};//-Проверка на запуск измерения температуры;
if(ddr==30){ddr=40;}								//--Если нет запуска по температуре, переход проверку cmdRunPress; 
}	//----------------------------------------------------------------------------------------------
if(ddr==35){	//++++Измерение температуры---------------------------------------------------------
meterTemp();	 	  								//--Пошаговое измерение температуры; 
if(err)    {ddr=90; sizeDelay=500;pointDelay=0;} 	//--Переход с задержкой при ошибке  meterTemp();
if(!action){ddr=90; sizeDelay=50; pointDelay=40;} 	//--Переход при успешном завершении meterTemp();
if(ddr==35){return;}								//--Выход   при продолжении работы  meterTemp();
}	//---------------------------------------------------------------------------------------------
if(ddr==40){	//++++Проверка на запуск измерения давления----------------------------------------
if(cmdRunHum){cmdRunHum=0;if(permitHum){ddr=45;step=0;}};//-Проверка на запуск измерения давления;
if(ddr==40){ddr=60;}								//--Если нет запуска, переход на завершение; 
}	//---------------------------------------------------------------------------------------------
if(ddr==45){	//++++Измерение давления-----------------------------------------------------------
meterHum();	 	  								//--Пошаговое измерение давления; 
if(err)    {ddr=90;  sizeDelay=500;pointDelay=0;} 	//--Переход с задержкой при ошибке  meterPress();
if(!action){ddr=90;  sizeDelay=50; pointDelay=60;} 	//--Переход при успешном завершении meterPress();
if(ddr==45){return;}								//--Выход   при продолжении работы  meterPress();
}	//---------------------------------------------------------------------------------------------
if(ddr==60){	//++++Завершение измерения---------------------------------------------------------
			ddr=90; sizeDelay=500; pointDelay=10;	//--Переход на начало с задержкой 500ms;
}	//---------------------------------------------------------------------------------------------
if(ddr==90){	//++++Задержка по времени с возвратом в заданную точку-----------------------------	
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} else {ddr=pointDelay;  return;}
}	//---------------------------------------------------------------------------------------------
};	//+++++END direct()============================================================================

void init(){									  //++++ИНИЦИАЛИЗАЦИЯ устройства===================
if(step==0) {	//++++Подготовка внутренних параметров функции-------------------------------------           
workDelay=err=codeErr=0; action=1; 					//--Очистка параметров;
  cntErr=0; step=90; sizeDelay=50; pointDelay=10;	//--Переход на продолжение через задержку 50мс;     
}	//---------------------------------------------------------------------------------------------
if(step==10){	//++++Инициализация шины-----------------------------------------------------------
if(!RT_HW_Base.i2cInitDevice(codeErr,adr,bus)){return;}     	//--Инициализация шины;
if(codeErr){err=1; codeErr=80;         return;}			//--Выход по ошибке инициализации шины; 
else       {step=15;      cntFind=0; cntErr=0;}			//--Безусловный переход на продолжение;
}	//---------------------------------------------------------------------------------------------
if(step==15){	//++++Поиск устройства на шине i2c-------------------------------------------------
if(!RT_HW_Base.i2cFindAdr(codeErr,adr,bus,speed)){return;};		//--Поиск устройства на на шине i2c;     
//codeErr=RT_HW_Base.i2cGetCodeErr(bus);					//--Получение кода ошибки;  	
if(codeErr){step=85;                pointDelay=15; cntFind=0;}//--При ошибке переход на динамическую задержку; 
else       {step=90; sizeDelay=50;  pointDelay=15; cntErr=0;  //--Переход на повторение поиска с задержкой 50ms;
            if(++cntFind>=3){step=60;  return;}              }//--Переход на продолжение в следующем цикле после 3-кратного обнаружения;
}	//--------------------------------------------------------------------------------------------
if(step==60){	//++++Завершение инициализации---------------------------------------------------- 
codeErr=0; action=0; step=0; return;
}	//---------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
if(step==85){	//++++Динамическая задержка--------------------------------------------------------
if(++cntErr>6){cntErr=6; codeErr=82; err=1;} sizeDelay=cntErr*cntErr*RT_HW_DEVICE_DELAY; step=90;
}	//---------------------------------------------------------------------------------------------
if(step==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} step=pointDelay;
}	//---------------------------------------------------------------------------------------------
};	//====END init()===============================================================================

void meterTemp(){								  //++++ИЗМЕРЕНИЕ ТЕМПЕРАТУРЫ======================
if(step==0) {	//++++Подготовка внутренних параметров функции-------------------------------------           
workDelay=err=codeErr=0; action=1; 						//--Очистка параметров;
   cntErr=0; step=10;									//--Переход на продолжение; 
}	//---------------------------------------------------------------------------------------------
if(step==10){	//++++Отправка команды запуска измерения температуры-------------------------------
if(!RT_HW_Base.i2cSendByte(codeErr,RT_HW_HTU21_TRIG_TMP_NOHOLD,adr,bus,speed)){return;}	//--Отправка команды;  
//codeErr=RT_HW_Base.i2cGetCodeErr(bus); 						//--Получение кода ошибки;
if(codeErr){step=85;               pointDelay=10;}			//--При ошибке повтор через динамическую задержку; 
else       {step=90; sizeDelay=50; pointDelay=20;cntErr=0;}	//--При ok    переход через фиксированную задержку 25ms;
}	//---------------------------------------------------------------------------------------------
if(step==20){	//++++Чтение 3-х байт--------------------------------------------------------------
if(!RT_HW_Base.i2cGetBytes(codeErr,bf,3,adr,bus,speed)){return;}	//--Запрос 3-х байтов;	 
//codeErr=RT_HW_Base.i2cGetCodeErr(bus);						//--Получение кода ошибки;
if(codeErr){step=85; 		       pointDelay=20;}			//--При ошибке повтор через динамическую  задержку;
else       {step=30;	cntErr=0; 			    return;}	//--Переход на продолжение в следующем цикле;
}	//----------------------------------------------------------------------------------------------	
if(step==30){	//++++Обработка 3-х байт------------------------------------------------------------											
crc=bf[2];
raw=((uint16_t)bf[0]<<8)|(uint16_t)bf[1]; 
if(crc8(raw,crc)!=0){codeErr=110; err=1; return;} 				//--Выход по ошибке CRC;
raw&=0xFFFC;  vTemp= -46.85 + (175.72 * (raw/(float)65536)); 	//--Вычисление температуры float;
fresh=ok=1; action=0; return;
}	//---------------------------------------------------------------------------------------------				  														   
//-------------------------------------------------------------------------------------------------
if(step==85){	//++++Динамическая задержка с выходом при многократной ошибке----------------------
if(++cntErr>3){cntErr=3; codeErr=84; err=1;} sizeDelay=cntErr*cntErr*RT_HW_DEVICE_DELAY; step=90;
}	//---------------------------------------------------------------------------------------------
if(step==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} step=pointDelay;
}	//---------------------------------------------------------------------------------------------
};	//++++END meterTemp()==========================================================================

void meterHum(){								  //++++ИЗМЕРЕНИЕ ВЛАЖНОСТИ========================
if(step==0) {	//++++Подготовка внутренних параметров функции-------------------------------------           
workDelay=err=codeErr=0; action=1; 						//--Очистка параметров;
   cntErr=0; step=10;									//--Переход на продолжение;   
}	//---------------------------------------------------------------------------------------------
if(step==10){	//++++Отправка команды запуска измерения температуры-------------------------------
if(!RT_HW_Base.i2cSendByte(codeErr,RT_HW_HTU21_TRIG_HUM_NOHOLD,adr,bus,speed)){return;}	//--Отправка команды;  
//codeErr=RT_HW_Base.i2cGetCodeErr(bus); 						//--Получение кода ошибки;
if(codeErr){step=85;               pointDelay=10;}			//--При ошибке повтор через динамическую задержку; 
else       {step=90; sizeDelay=50; pointDelay=20;cntErr=0;}	//--При ok    переход через фиксированную задержку 25ms;
}	//---------------------------------------------------------------------------------------------
if(step==20){	//++++Чтение 3-х байт--------------------------------------------------------------
if(!RT_HW_Base.i2cGetBytes(codeErr,bf,3,adr,bus,speed)){return;}	//--Запрос 3-х байтов;	 
//codeErr=RT_HW_Base.i2cGetCodeErr(bus);						//--Получение кода ошибки;  
if(codeErr){step=85; 		       pointDelay=20;}			//--При ошибке повтор через динамическую  задержку;
else       {step=30;	cntErr=0; 			    return;}	//--Переход на продолжение в следующем цикле;
}	//----------------------------------------------------------------------------------------------	
if(step==30){	//++++Обработка 3-х байт------------------------------------------------------------	 	
crc=bf[2];													//--Сохранение crc;		 
raw=((uint16_t)bf[0]<<8)|(uint16_t)bf[1]; 					//--Расчет температуры во временной переменной;
if(crc8(raw,crc)!=0){codeErr=110; err=1; return;} 			//--Выход по ошибке CRC;
raw&=0xFFFC;  	vHum=-6+(125*(raw/(float)65536));   		//--Вычисление влажности float;
fresh=ok=1; action=0; return;								//--Успешный выход;
}	//---------------------------------------------------------------------------------------------				  														   
//-------------------------------------------------------------------------------------------------
if(step==85){	//++++Динамическая задержка с выходом при многократной ошибке----------------------
if(++cntErr>3){cntErr=3; codeErr=84; err=1;} sizeDelay=cntErr*cntErr*RT_HW_DEVICE_DELAY; step=90;
}	//---------------------------------------------------------------------------------------------
if(step==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} step=pointDelay;
}	//---------------------------------------------------------------------------------------------
};	//++++END meterTemp()==========================================================================

uint8_t crc8(uint16_t dataSensor, uint8_t crcSensor){//++++РАСЧЕТ CRC8=============================
divisor   = (uint32_t)RT_HW_HTU21_SHIFTED_DIVISOR;	//--Запоминаем 3-х байтовый полином;
remainder = (uint32_t)dataSensor << 8;	    //--Сдвиг измеренного значения на один байт влево (для работы с 3-байтовым полиномом); 
remainder|= crcSensor; 				//--Добавление в младший байт crc из сенсора;
for (idx=0;idx<16;idx++){if(remainder & (uint32_t)1<<(23-idx)) remainder^= divisor; divisor>>=1;} //--Расчет остатка после проверки crc;
return (uint8_t)remainder;
};	//++++END crc8()=============================================================================== 
//=================================================================================================
};
//=================================================================================================
#endif
//=================================================================================================
