//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			СЕНСОР SHT3X
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  ecoins (ecoins@mail.ru) 
//  22.10.2022: Begin version
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-------------------------------------------------------------------------------------------------
#ifndef RT_HW_SHT3X_h
#define RT_HW_SHT3X_h
#define RT_HW_SHT3X_VER 		10		//--Номер версии библиотеки;
#include "RT_HW_BASE.h"					//--Подключение базовой библиотеки;
//-------------------------------------------------------------------------------------------------
//#define RT_HW_SHTX_DEFAULT_ADDR          	0x44   //--SHTX Default Address;
#define RT_HW_SHTX_SOFTRESET 				0x30A2 //--Soft Reset;
#define RT_HW_SHTX_MEAS_HIGHREP_STRETCH  	0x2C06 //--Measurement High   Repeatability with Clock Stretch Enabled;
#define RT_HW_SHTX_MEAS_MEDREP_STRETCH   	0x2C0D //--Measurement Medium Repeatability with Clock Stretch Enabled;
#define RT_HW_SHTX_MEAS_LOWREP_STRETCH    	0x2C10 //--Measurement Low    Repeatability with Clock Stretch Enabled;
#define RT_HW_SHTX_MEAS_HIGHREP           	0x2400 //--Measurement High   Repeatability with Clock Stretch Disabled;
#define RT_HW_SHTX_MEAS_MEDREP          	0x240B //--Measurement Medium Repeatability with Clock Stretch Disabled;
#define RT_HW_SHTX_MEAS_LOWREP     		    0x2416 //--Measurement Low    Repeatability with Clock Stretch Disabled;
#define RT_HW_SHTX_READSTATUS 				0xF32D //--Read Out of Status Register;
#define RT_HW_SHTX_CLEARSTATUS 			    0x3041 //--Clear Status;
#define RT_HW_SHTX_REG_HEATER_BIT 			0x0d   //--Status Register Heater Bit;
#define RT_HW_SHTX_HEATEREN 				0x306D //--Heater Enable;
#define RT_HW_SHTX_HEATERDIS 				0x3066 //--Heater Disable;
//=================================================================================================
//};
//=================================================================================================
//#endif
//=================================================================================================

//=================================================================================================
//                            КЛАСС СЕНСОРА SHT3X
//=================================================================================================
class RT_HW_SHT3X{
public:
//-----------------Основные параметры управления---------------------------------------------------
uint8_t  custom=0;				//--Состояние настройки ID;  
//----------------Перенастраиваемые параметры пользователем----------------------------------------
uint8_t  bus=0;					//--Номер шины;
uint8_t  adr=0x44;				//--Адрес i2c по умолчанию;
uint32_t speed=RT_HW_I2C_SPEED;	//--Скорость шины;
//----------------Основные переменные управления измерением-----------------------------------------
uint8_t  run=0, cmdRun=0;       //--Флаг для внешнего запуска измерений;
uint8_t  ddr=0;					//--Этапы основной программы управления устройствами;
uint8_t  step;					//--Управление этапами управления в функциях измерения;
//----------------Выходные параметры-----------------------------------------------------------------
uint8_t  fresh=1;				//--Флаг "свежих" данных;
uint8_t  ok=0;					//--Флаг "Устройство в норме;
uint8_t  codeErr, err;			//--Код ошибки при обмене;
//----------------Дополнительные переменные управления измерением-----------------------------------
uint8_t  initDevice, action;	//--Рабочие флаги;
uint8_t  cntErr;				//--Счетчик повторных ошибок;
uint8_t  cntFind;				//--Счетчик поиска устройств;
uint16_t cmnd=0x2400;			//--Table 9 page 10 datasheet;
//----------------Параметры для функци timeDelay()[и также используются функцией i2cFindAdr()]------
uint32_t startDelay;		    //--Рабочий регистр;
uint8_t  workDelay=0;			//--Рабочий регистр; 
uint8_t  pointDelay;			//--Точка возврата из функции;
uint16_t sizeDelay;				//--Задержка для функции();
//----------------Буферные рабочие переменные (поддержка релевантности)------------------------------
const uint8_t POLYNOMIAL=0x31;
uint8_t  crc;
int32_t  stemp;
uint32_t shum;
uint32_t temp;					//--Рабочий регистр;
uint8_t  cmd;					//--Рабочий регистр;
uint8_t  idx,idx1;				//--Индексы;
uint8_t  bf[8];              	//--Буфер для временного хранения данных;
float    vTemp, vHum;			//--Измеренные значения;
//=================================================================================================
void setParam(uint8_t vAdr=0x44, uint8_t vBus=0, uint32_t vSpeed=0){ //++++НАСТРОЙКА параметров
if(custom==0){	//++++Однократная настройка параметров---------------------------------------------
adr=vAdr;  bus=vBus; speed=vSpeed;	custom=1;}	//--Настройка адреса устройства,номера шины,скорости шины;
};	//+++++END setParam()==========================================================================

void direct(){									  //++++УПРАВЛЕНИЕ измерением====================== 
//-------------------------------------------------------------------------------------------------
if(run){cmdRun=1;} 
//-------------------------------------------------------------------------------------------------
if(ddr==0) {	//++++Начальная настройка----------------------------------------------------------	
workDelay=err=codeErr=cntErr=cntFind=initDevice=0; 	//--Инициализация рабочих параметров;
fresh=1; ok=0; vTemp=0.11; vHum=0.12;				//--Инициализация выходных параметров; 
            ddr=90; sizeDelay=200; pointDelay=10;	//--Переход на продолжение через 200ms;
}	//----------------------------------------------------------------------------------------------
if(ddr==10){	//++++Ожидание команды cmdRun(запуск измерения)------------------------------------
if(!cmdRun){return;} step=0;			//-- 
if(initDevice) {ddr=30;} else {ddr=20;}	//--Переход на измерение или на инициализацию;	
}	//---------------------------------------------------------------------------------------------
if(ddr==20){	//++++Инициализация устройства-----------------------------------------------------
init(); 	  												//--Инициализация устройства; 		  
if(err)   {ddr=90; sizeDelay=2000; pointDelay=0;}			//--При ошибке переход ddr=0 через задержку 2000ms;
if(action){return;}   										//--Ожидание завершения измерения;
initDevice=1; step=0; ddr=30;								//--Переход на измерение;
}	//---------------------------------------------------------------------------------------------
if(ddr==30){	//++++Измерение и получение результатов--------------------------------------------
meter();	 	  											//--Пошаговое измерение; 
if(err)   {ddr=90; sizeDelay=500; pointDelay=0;} 			//--Переход на начало через задержку 1000ms;														
if(action){return;}  										//--Ожидание завершения измерения;
cmdRun=0;  ddr=90; sizeDelay=500; pointDelay=10; 			//--Переход на ожидание run через задержку 500ms;;
}	//---------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
//if(ddr==60){	//++++Динамическая задержка--------------------------------------------------------
//if(++cntErr>6){cntErr=6; codeErr=80; err=1;} sizeDelay=cntErr*cntErr*RT_HW_DEVICE_DELAY; ddr=70;} 
if(ddr==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} else {ddr=pointDelay;  return;}
}	//---------------------------------------------------------------------------------------------
};	//+++++END direct()============================================================================

void init(){									  //++++ИНИЦИАЛИЗАЦИЯ устройства===================
if(step==0) {	//++++Подготовка внутренних параметров функции-------------------------------------           
workDelay=err=codeErr=0; action=1; 					//--Очистка параметров;
  cntErr=0; step=90; sizeDelay=50; pointDelay=10;	//--Переход на продолжение через задержку 50мс;     
}	//---------------------------------------------------------------------------------------------
if(step==10){	//++++Инициализация шины-----------------------------------------------------------
if(!RT_HW_Base.i2cInitDevice(codeErr,adr,bus)){return;}     	//--Инициализация шины;
if(codeErr){err=1; codeErr=80;         return;}			//--Выход по ошибке инициализации шины; 
else       {step=15;      cntFind=0; cntErr=0;}			//--Безусловный переход на продолжение;
}	//---------------------------------------------------------------------------------------------
if(step==15){	//++++Поиск устройства на шине i2c-------------------------------------------------
if(!RT_HW_Base.i2cFindAdr(codeErr,adr,bus,speed)){return;};		//--Поиск устройства на на шине i2c;     
//codeErr=RT_HW_Base.i2cGetCodeErr(bus);					//--Получение кода ошибки;  	
if(codeErr){step=85;                pointDelay=15; cntFind=0;}//--При ошибке переход на динамическую задержку; 
else       {step=90; sizeDelay=50;  pointDelay=15; cntErr=0;  //--Переход на повторение поиска с задержкой 50ms;
            if(++cntFind>=3){step=60;  return;}              }//--Переход на продолжение в следующем цикле после 3-кратного обнаружения;
}	//--------------------------------------------------------------------------------------------
if(step==60){	//++++Завершение инициализации---------------------------------------------------- 
codeErr=0; action=0; step=0; return;
}	//---------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
if(step==85){	//++++Динамическая задержка--------------------------------------------------------
if(++cntErr>6){cntErr=6; codeErr=82; err=1;} sizeDelay=cntErr*cntErr*RT_HW_DEVICE_DELAY; step=90;
}	//---------------------------------------------------------------------------------------------
if(step==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} step=pointDelay;
}	//---------------------------------------------------------------------------------------------
};	//====END init()===============================================================================

void meter(){									  //++++ИЗМЕРЕНИЕ параметров======================= 
if(step==0) {	//++++Подготовка внутренних параметров функции-------------------------------------
workDelay=err=codeErr=0; action=1; 						//--Очистка параметров;
     cntErr=0; step=10;									//--Переход на продолжение; 
}	//---------------------------------------------------------------------------------------------
if(step==10){	//++++Подготовка команды запуска измерения [0xAС,0x33,00]--------------------------
bf[0]=cmnd>>8;	bf[1]=cmnd&0xFF;						//--Подготовка параметров команды;
			 step=11;									//--Переход;
}	//---------------------------------------------------------------------------------------------
if(step==11){	//++++Команда запуска измерения [0xAС,0x33,00],timeDelay>=40ms---------------------
if(!RT_HW_Base.i2cSendBytes(codeErr,bf,2,adr,bus,speed)){return;}//-Отправка команды;	
//codeErr=RT_HW_Base.i2cGetCodeErr(bus);					//--Получение кода ошибки;  
if(codeErr){step=85;                pointDelay=11;}		//--При ошибке повтор через динамическую  задержку;
else       {step=90; sizeDelay=100; pointDelay=20;cntErr=0;}//-При ok переход через фиксированную задержку 100ms;
}	//---------------------------------------------------------------------------------------------
if(step==20){	//++++Чтение результатов измерения-------------------------------------------------    
if(!RT_HW_Base.i2cGetBytes(codeErr,bf,6,adr,bus,speed)){return;}//--Запрос 7-ми байтов;	
//codeErr=RT_HW_Base.i2cGetCodeErr(bus);					//--Получение кода ошибки;  
if(codeErr){step=85;                pointDelay=20;}		//--При ошибке повтор через динамическую  задержку;
else       {step=25;                            return;}//--Переход на обработку в следующем цикле; 			  
}	//---------------------------------------------------------------------------------------------
if(step==25){	//++++Обработка результатов измерения---------------------------------------------- 											//--Выход по ошибке обмена;
if(bf[2]!=crc8(bf,2))  {codeErr=110; err=1; return;}
if(bf[5]!=crc8(bf+3,2)){codeErr=111; err=1; return;}
stemp=(int32_t)(((uint32_t)bf[0]<<8) | bf[1]);
stemp=((4375 * stemp) >> 14) - 4500; 
vTemp = (float)stemp / 100.0f;

shum = ((uint32_t)bf[3] << 8) | bf[4];
shum = (625 * shum) >> 12; 
vHum = (float)shum / 100.0f;		 			   

fresh=ok=1; action=0; step=0;	return;						//--Выход по успешному завершению;	
}	//---------------------------------------------------------------------------------------------
//=================================================================================================
if(step==85){	//++++Динамическая задержка с выходом при многократной ошибке----------------------
if(++cntErr>3){cntErr=3; codeErr=84; err=1;} sizeDelay=cntErr*cntErr*RT_HW_DEVICE_DELAY; step=90;
}	//---------------------------------------------------------------------------------------------
if(step==90){	//++++Фиксированная задержка-------------------------------------------------------
if(!RT_HW_Base.timeDelay(startDelay,workDelay,sizeDelay)){return;} step=pointDelay;
}	//---------------------------------------------------------------------------------------------											 
};	//====void meter()-----------------------------------------------------------------------------	
//=================================================================================================
//		 Calcuration CRC8 polynomial: CRC[7: 0] = X8 + X5 + X4 + 1  -  0x1 0011 0001 - 0x131
// CRC-8 formula from page 14 of SHT spec pdf
// Test data 0xBE, 0xEF should yield 0x92
// Initialization data 0xFF
// Polynomial 0x31 (x8 + x5 +x4 +1)
// Final XOR 0x00
//=================================================================================================
uint8_t crc8(const uint8_t *data, int len) {
crc=0xFF;
for(idx1 = len; idx1; --idx1) {crc ^= *data++;
 for(int idx = 8; idx; --idx) {crc = (crc & 0x80) ? (crc << 1) ^ POLYNOMIAL : (crc << 1);}}
  return crc;
};	//++++END crc8=================================================================================
};	//++++END class RT_HW_SHT3X====================================================================
#endif
//=================================================================================================